var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

var url = require("/carmalib/url");

var gps = require("gps/gps_manager");

exports.definition = {
    config: {
        adapter: {
            type: "persistentrest",
            persistentType: "nosql",
            collectionName: "crew",
            merge: false,
            personal: true
        },
        idAttribute: "id"
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            parse: function(response) {
                var profile = response.profile;
                TM.push(function() {
                    var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
                    ProfileService.saveProfileToCache(profile);
                }, process.backgroundDomain);
                return {
                    id: profile.userId,
                    name: profile.alias,
                    photoUrl: profile.photoUrl,
                    relativeDistance: profile.relativeDistance,
                    reason: response.reason,
                    canUserGiveFreeDrive: profile.canUserGiveFreeDrive,
                    gender: profile.gender
                };
            }
        });
        return Model;
    },
    extendCollection: function(Collection) {
        _.extend(Collection.prototype, {
            parse: function(response) {
                var members = response && response.members || [];
                return members;
            },
            fetchCrew: function(userId, options) {
                options = options || {};
                var location = gps.lastLocation();
                options.url = location ? _.template(url.CREW_WITH_LOCATION, {
                    userId: userId,
                    lat: location.lat,
                    lon: location.lon
                }) : _.template(url.CREW, {
                    userId: userId
                });
                return this.fetch(options);
            }
        });
        return Collection;
    }
};

model = Alloy.M("crew", exports.definition, []);

collection = Alloy.C("crew", exports.definition, model);

exports.Model = model;

exports.Collection = collection;