var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

var url = require("/carmalib/url");

exports.definition = {
    config: {
        adapter: {
            type: "persistentrest",
            persistentType: "nosql",
            collectionName: "favourite_user",
            merge: false,
            personal: true
        },
        idAttribute: "id"
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            parse: function(response) {
                var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
                ProfileService.saveProfileToCache(response);
                return {
                    userId: response.userId,
                    photoUrl: response.photoUrl,
                    alias: response.alias,
                    canUserTakeFreeRide: response.canUserTakeFreeRide,
                    gender: response.gender
                };
            }
        });
        return Model;
    },
    extendCollection: function(Collection) {
        _.extend(Collection.prototype, {
            _urlTemplate: url.FAVOURITE_USER,
            parse: function(response) {
                var users = response.users || response.profile || [];
                return users;
            },
            fetchFavouriteUsers: function(userId, options) {
                options = options || {};
                options.url = _.template(this._urlTemplate, {
                    userId: userId,
                    pageSize: options.pageSize
                });
                return this.fetch(options);
            },
            save: function(models, options) {
                this.reset(models, options);
                return this.sync("update", this, options);
            }
        });
        return Collection;
    }
};

model = Alloy.M("favourite_user", exports.definition, []);

collection = Alloy.C("favourite_user", exports.definition, model);

exports.Model = model;

exports.Collection = collection;