var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

exports.definition = {
    config: {
        adapter: {
            type: "persistentrest",
            persistentType: "nosql",
            collectionName: "free_riders",
            merge: false,
            personal: true
        },
        idAttribute: "userId"
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            parse: function(response) {
                var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
                ProfileService.saveProfileToCache(response);
                return {
                    userId: response.userId,
                    photoUrl: response.photoUrl,
                    alias: response.alias,
                    gender: response.gender
                };
            }
        });
        return Model;
    },
    extendCollection: function(Collection) {
        _.extend(Collection.prototype, {
            url: "FREE_RIDERS",
            userId: null,
            initialize: function(attributes, options) {
                options = options || {};
                attributes = attributes || {};
                this.userId = attributes.userId || options.userId || require("services/resource_owner_service").singleton().get("userId");
            },
            getCache: function() {
                return this.fetch({
                    localOnly: true,
                    urlParams: {
                        userId: this.userId
                    }
                });
            },
            updateCache: function(attributes) {
                return this.save(attributes, {
                    localOnly: true,
                    urlParams: {
                        userId: this.userId
                    }
                });
            },
            findAll: function(options) {
                return this.fetch(_.extend({}, options, {
                    urlParams: {
                        userId: this.userId
                    }
                }));
            },
            addFreeRider: function(user, options) {
                return Alloy.createModel("free_rider").addFreeRider(user.userId, options);
            },
            deleteFreeRider: function(user, options) {
                return Alloy.createModel("free_rider").deleteFreeRider(user.userId, options);
            },
            parse: function(response) {
                return response && response.users ? response.users : [];
            },
            save: function(models, options) {
                this.reset(models, options);
                return this.sync("update", this, options);
            }
        });
        return Collection;
    }
};

model = Alloy.M("free_riders", exports.definition, []);

collection = Alloy.C("free_riders", exports.definition, model);

exports.Model = model;

exports.Collection = collection;