var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

exports.definition = {
    config: {
        adapter: {
            type: "restapi"
        }
    },
    extendModel: function(Model) {
        var self = this;
        _.extend(Model.prototype, {
            route: function(spec) {
                var options = {};
                options.url = _.template("http://maps.googleapis.com/maps/api/directions/json?origin=<%= originLat %>,<%= originLon %>&destination=<%= destinationLat %>,<%= destinationLon %>&sensor=false", spec);
                return this.fetch(options).then(function(data) {
                    var route = data.routes[0];
                    if (route) return {
                        route: self._getPoints(route),
                        path: route.overview_polyline.points
                    };
                    return null;
                });
            }
        });
        return Model;
    },
    _getPoints: function(route) {
        var step = route.legs[0].steps;
        var intStep = 0, intSteps = step.length, points = [];
        var decodedPolyline, intPoint = 0, intPoints = 0;
        for (intStep = 0; intSteps > intStep; intStep += 1) {
            decodedPolyline = this._decodeLine(step[intStep].polyline.points);
            intPoints = decodedPolyline.length;
            for (intPoint = 0; intPoints > intPoint; intPoint += 1) null !== decodedPolyline[intPoint] && points.push({
                latitude: decodedPolyline[intPoint][0],
                longitude: decodedPolyline[intPoint][1]
            });
        }
        return points;
    },
    _decodeLine: function(encoded) {
        var len = encoded.length;
        var index = 0;
        var array = [];
        var lat = 0;
        var lng = 0;
        while (len > index) {
            var b;
            var shift = 0;
            var result = 0;
            do {
                b = encoded.charCodeAt(index++) - 63;
                result |= (31 & b) << shift;
                shift += 5;
            } while (b >= 32);
            var dlat = 1 & result ? ~(result >> 1) : result >> 1;
            lat += dlat;
            shift = 0;
            result = 0;
            do {
                b = encoded.charCodeAt(index++) - 63;
                result |= (31 & b) << shift;
                shift += 5;
            } while (b >= 32);
            var dlng = 1 & result ? ~(result >> 1) : result >> 1;
            lng += dlng;
            array.push([ 1e-5 * lat, 1e-5 * lng ]);
        }
        return array;
    }
};

model = Alloy.M("points_route", exports.definition, []);

collection = Alloy.C("points_route", exports.definition, model);

exports.Model = model;

exports.Collection = collection;