var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

var DEFAULT_EXPIRY_TIME = 31536e3;

exports.definition = {
    config: {
        adapter: {
            type: "properties",
            collection_name: "resource_owner"
        }
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            validate: function(attrs) {
                var validator = require("validator/validator_factory").number();
                for (var key in attrs) {
                    var value = attrs[key];
                    if (!("userId" !== key || value && validator.isNumber(value))) return "Error: Invalid userId!";
                    if ("token" === key) {
                        if (!value || value.token.length <= 0) return "Error: Invalid token!";
                        var expiresIn = value.expiresIn;
                        var now = Date.now();
                        var expiryTime = now + 1e3 * (expiresIn || DEFAULT_EXPIRY_TIME);
                        value.expiry = expiryTime;
                    }
                    if ("location" === key && value && value.length <= 0) return "Error: No location!";
                    if ("country" === key && value && value.length <= 0) return "Error: No country!";
                }
            },
            hasLogin: function() {
                if (!this.has("userId")) {
                    LOGGER.debug("No user Id...");
                    return false;
                }
                if (!this.has("token")) {
                    LOGGER.debug("No token...");
                    return false;
                }
                var expiry = this.get("token").expiry;
                if (!expiry || Date.now() > expiry) {
                    LOGGER.debug("Access token has expired...");
                    return false;
                }
                return true;
            }
        });
        return Model;
    }
};

model = Alloy.M("resource_owner", exports.definition, []);

collection = Alloy.C("resource_owner", exports.definition, model);

exports.Model = model;

exports.Collection = collection;