var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

var url = require("/carmalib/url");

exports.definition = {
    config: {
        adapter: {
            type: "restapi"
        }
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            url: "USER_RIDE_BOOKING",
            findByStates: function(userId, tripId, baseEffectiveTime, states, includeOutgoing) {
                var options = {}, query = "ACCEPTED";
                states && (query = states);
                query += includeOutgoing ? "&includeOutgoing=true" : "&includeOutgoing=false";
                baseEffectiveTime && (query = query + "&baseEffectiveTime=" + baseEffectiveTime);
                options.url = _.template(require("/carmalib/url").USER_RIDE_BOOKING_STATE, {
                    userId: userId,
                    tripId: tripId
                }) + "?states=" + query + "&userFields=GENDER,PHOTO_URL,ALIAS";
                return this.fetch(options);
            },
            findAll: function(userId, onlyConfirmed) {
                var options = {}, query = "ACCEPTED";
                onlyConfirmed || (query += ",REQUESTED");
                if (!userId) {
                    var resourceOwner = require("services/resource_owner_service").singleton();
                    userId = resourceOwner.get("userId");
                }
                options.url = _.template(require("/carmalib/url").USER_RIDE_BOOKING, {
                    userId: userId
                }) + "?states=" + query + "&userFields=PHOTO_URL,ALIAS,PHONE_NUMBER,IS_FREE_DRIVER";
                return this.fetch(options);
            },
            accept: function(userId, bookingId, msg) {
                var options = {};
                options.url = _.template(require("/carmalib/url").USER_RIDE_BOOKING, {
                    userId: userId
                }) + "/" + bookingId + "/accept?generateUserMessages=true&responseNote=" + encodeURIComponent(msg);
                return this.save({
                    responseNote: msg
                }, options);
            },
            decline: function(userId, bookingId, msg) {
                var options = {};
                options.url = _.template(require("/carmalib/url").USER_RIDE_BOOKING, {
                    userId: userId
                }) + "/" + bookingId + "/cancel?generateUserMessages=true&responseNote=" + encodeURIComponent(msg);
                return this.save({
                    responseNote: msg
                }, options);
            }
        });
        return Model;
    }
};

model = Alloy.M("ride_booking", exports.definition, []);

collection = Alloy.C("ride_booking", exports.definition, model);

exports.Model = model;

exports.Collection = collection;