var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

var LOC_PRECISION = 1e6;

var roundLocationCoords = function(location) {
    if (location) {
        location.lat && (location.lat = Math.round(location.lat * LOC_PRECISION) / LOC_PRECISION);
        location.lon && (location.lon = Math.round(location.lon * LOC_PRECISION) / LOC_PRECISION);
    }
    return location;
};

exports.definition = {
    config: {
        adapter: {
            type: "mergeupdateproperties",
            collection_name: "trip_state"
        }
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            isValid: function() {
                var invalid = this.get("invalid");
                return !(invalid && true === invalid);
            },
            isInCar: function() {
                return this.get("inCar");
            },
            isRated: function() {
                var rated = this.get("rated");
                return rated && true === rated;
            },
            getRole: function() {
                return this.get("role");
            },
            isDriverTrip: function() {
                var role = this.get("role");
                return role && "DRIVER" === role;
            },
            setUXBoardingDetails: function(timestamp, location, driverId, driverIdType) {
                this.set("inCar", true);
                this.set("startTimestamp", timestamp);
                this.set("startLocation", roundLocationCoords(location));
                if ("PHONE" === driverIdType) {
                    this.set("phone", driverId);
                    this.set("driver", {
                        phone: driverId
                    });
                } else this.set("driver", {
                    id: driverId
                });
                this.save();
            },
            setUXDropoffDetails: function(timestamp, location, isFreeRide) {
                this.set("current", false);
                this.set("inCar", false);
                this.set("endTimestamp", timestamp);
                this.set("endLocation", roundLocationCoords(location));
                this.set("isFreeRide", isFreeRide);
                this.save();
            },
            setUXRatingDetails: function(rating, comment) {
                this.set("rated", true);
                this.set("rating", rating);
                this.set("comment", comment);
                this.save();
            },
            setServerBoardingDetails: function(driverId, journeyId) {
                this.set("driver", {
                    id: driverId
                });
                this.set("journeyId", journeyId);
                this.save();
            },
            setServerRiderDetails: function(timestamp, location, riderId, journeyId) {
                this.set("inCar", true);
                this.set("startTimestamp", timestamp);
                this.set("startLocation", roundLocationCoords(location));
                this.set("rider", {
                    id: riderId
                });
                this.set("journeyId", journeyId);
                this.save();
            },
            setServerDropOffDetails: function(rideRecordId, distance, cost, earning, startAddress, endAddress) {
                this.set("rideRecordId", rideRecordId);
                this.set("distance", distance);
                this.set("cost", cost);
                this.set("earning", earning);
                this.set("startAddress", startAddress);
                this.set("endAddress", endAddress);
                this.save();
            },
            setServerRatedDetails: function(timestamp) {
                this.set("ratedTimestamp", timestamp);
                this.save();
            }
        });
        return Model;
    },
    extendCollection: function(Collection) {
        _.extend(Collection.prototype, {});
        return Collection;
    }
};

model = Alloy.M("trip_state", exports.definition, []);

collection = Alloy.C("trip_state", exports.definition, model);

exports.Model = model;

exports.Collection = collection;