var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

var Q = require("vendor/q");

exports.definition = {
    config: {
        adapter: {
            type: "persistentrest",
            persistentType: "nosql",
            collectionName: "user_chat",
            merge: false,
            cappedSize: 300,
            personal: true
        },
        idAttribute: "id"
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            _pageSize: 25,
            _currentPage: 1,
            _totalPages: null,
            _threadId: null,
            _refreshPaginator: true,
            fetch: function(options) {
                options = options || {};
                this._threadId = options.threadId || this._threadId;
                var user = require("services/resource_owner_service").singleton();
                options.url = _.template(require("/carmalib/url").USER_CHAT, {
                    userId: user.get("userId"),
                    threadId: this._threadId
                });
                var pageSize = options.pageSize || this._pageSize;
                pageSize && (options.url = options.url + "?pageSize=" + pageSize);
                var currentPage = options.pageNum || this._currentPage;
                if (currentPage) {
                    options.url = options.url + (pageSize ? "&" : "?");
                    options.url = options.url + "pageNum=" + currentPage;
                }
                options.url = options.url + (pageSize || currentPage ? "&" : "?");
                options.url = options.url + "profileLevel=LIMITED";
                options.startId && (options.url = options.url + "&startId=" + options.startId);
                if ("refreshPaginator" in options) {
                    this._refreshPaginator = options.refreshPaginator;
                    delete options.refreshPaginator;
                }
                return this.constructor.__super__.fetch.apply(this, [ options ]);
            },
            parse: function(response) {
                if (response.messages.paginator && this._refreshPaginator) {
                    this._totalPages = response.messages.paginator.totalPages;
                    this._refreshPaginator = false;
                }
                return response;
            },
            hasNext: function() {
                if (this._currentPage < this._totalPages) return true;
                return false;
            },
            next: function() {
                if (!this.hasNext()) return Q.fcall(function() {
                    return;
                });
                ++this._currentPage;
                return this.fetch({
                    remoteOnly: true,
                    refreshPaginator: false
                });
            }
        });
        return Model;
    }
};

model = Alloy.M("user_chat", exports.definition, []);

collection = Alloy.C("user_chat", exports.definition, model);

exports.Model = model;

exports.Collection = collection;