var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

"use strict";

var Q = require("vendor/q"), network = require("bootstrap/network");

exports.definition = {
    config: {
        adapter: {
            type: "persistentrest",
            persistentType: "nosql",
            collectionName: "user_conversations",
            merge: false,
            cappedSize: 50,
            personal: true
        },
        idAttribute: "id"
    },
    extendCollection: function(Collection) {
        _.extend(Collection.prototype, {
            _pageSize: 15,
            _currentPage: 1,
            _totalPages: -1,
            _refreshPaginator: true,
            fetch: function(options) {
                options = options || {};
                var user = require("services/resource_owner_service").singleton();
                options.url = _.template(require("/carmalib/url").USER_CONVERSATIONS, {
                    userId: user.get("userId")
                });
                options.url = options.url + "?userFields=ALIAS,PHOTO_URL,GENDER";
                var pageSize = options.pageSize || this._pageSize;
                pageSize && (options.url = options.url + "&pageSize=" + pageSize);
                var currentPage = options.pageNum || this._currentPage;
                currentPage && (options.url = options.url + "&pageNum=" + currentPage);
                this._refreshPaginator = "refreshPaginator" in options ? options.refreshPaginator : true;
                return this.constructor.__super__.fetch.apply(this, [ options ]);
            },
            parse: function(response) {
                if (response.paginator && this._refreshPaginator) {
                    this._totalPages = response.paginator.totalPages;
                    this._refreshPaginator = false;
                }
                var threads = [];
                _.each(response.threads, function(conversation) {
                    (conversation && conversation.associatedUser || conversation && conversation.associatedGroup) && threads.push(conversation);
                });
                return threads;
            },
            hasNext: function() {
                return this._currentPage < this._totalPages;
            },
            next: function() {
                if (!this.hasNext()) return Q.fcall(function() {
                    return;
                });
                ++this._currentPage;
                return this.fetch({
                    remoteOnly: true,
                    refreshPaginator: false
                });
            },
            deleteConversation: function() {}
        });
        return Collection;
    }
};

model = Alloy.M("user_conversations", exports.definition, []);

collection = Alloy.C("user_conversations", exports.definition, model);

exports.Model = model;

exports.Collection = collection;