function WPATH(s) {
    var index = s.lastIndexOf("/");
    var path = -1 === index ? "ma.car.app.samantha/" + s : s.substring(0, index) + "/ma.car.app.samantha/" + s.substring(index + 1);
    return true && 0 !== path.indexOf("/") ? "/" + path : path;
}

function __processArg(obj, key) {
    var arg = null;
    if (obj) {
        arg = obj[key] || null;
        delete obj[key];
    }
    return arg;
}

function Controller() {
    function startSamanthaFinding(trip, noPopup) {
        var userRole = "RIDER";
        "DRIVE" === trip.type && (userRole = "DRIVER");
        Alloy.createModel("carpool_finding").find(trip.id, userRole, trip.schedule.baseEffectiveTime).then(function(data) {}).catch(function(e) {
            LOGGER.error("[samantha] - carpool_finding: " + JSON.stringify(e));
        }).finally(function() {}).done();
    }
    function confirmTrip(trip, willGoing, options) {
        $.toggleSpinner(true);
        options = options || {};
        Alloy.createModel("trip_confirm").confirm(trip.id, trip.schedule.baseEffectiveTime, !!willGoing).then(function(data) {
            willGoing && trip.matchesCount > 0 && startSamanthaFinding(trip, options.noPopupPrompt);
        }).catch(function(e) {
            $.toggleSpinner(false);
            $.whisper("samantha.show.ErrorDialog", _.extend({
                action: "USER_TRIP_CONFIRM"
            }, e));
        }).finally(function() {
            $.toggleSpinner(false);
            options.noPopupPrompt || Ti.App.fireEvent("app.carma.profile.change");
            options.callback && _.isFunction(options.callback) && options.callback();
        });
    }
    function turnOffTrip(trip) {
        if (!trip || !trip.id) return;
        $.toggleSpinner(true);
        Alloy.createModel("trip").getTrip(trip.id).then(function(v1Trip) {
            trip.enabled = false;
            trip.matchesCount && delete trip.matchesCount;
            v1Trip.baseTrip.enabled = false;
            v1Trip.baseTrip.matchesCount && delete v1Trip.baseTrip.matchesCount;
            Alloy.createModel("trip").saveTrip(v1Trip.baseTrip).then(function() {
                Ti.App.fireEvent("app.carma.profile.change");
            }).catch(function(e) {
                $.whisper("samantha.show.ErrorDialog", _.extend({
                    action: "TURN_OFF_TRIP"
                }, e));
            }).finally(function() {
                $.toggleSpinner(false);
            }).done();
        }).catch(function(e) {
            $.whisper("samantha.show.ErrorDialog", _.extend({
                action: "TURN_OFF_TRIP"
            }, e));
        }).finally(function() {
            $.toggleSpinner(false);
        }).done();
    }
    function editTripAction(params) {
        var trip = nextTrip;
        if (trip.schedule.confirmed && trip.autoRideMatchStatus) switch (params.index) {
          case 0:
            deviateCurrentTrip(trip, 5);
            break;

          case 1:
            deviateCurrentTrip(trip, 15);
            break;

          case 2:
            deviateCurrentTrip(trip, 60);
            break;

          case 3:
            if ("tripBookings" === $.currentCard) $.whisper("tripBookings.toggle.editing"); else {
                confirmTrip(trip, false);
                TM.push(function() {
                    analytics.track(trackingEvent.SAMANTHA_NOT_GOING, {
                        tripId: trip.id
                    });
                }, process.backgroundDomain);
            }
            break;

          case 4:        } else switch (params.index) {
          case 0:
            deviateCurrentTrip(trip, 5);
            break;

          case 1:
            deviateCurrentTrip(trip, 15);
            break;

          case 2:
            deviateCurrentTrip(trip, 60);
            break;

          case 3:
            Alloy.createWidget("ma.car.app.trip", {
                tripId: trip.id
            }).getView().open();
            TM.push(function() {
                analytics.track(trackingEvent.SAMANTHA_EDIT_TRIP, {
                    tripId: trip.id
                });
            }, process.backgroundDomain);
            break;

          case 4:
            var buttons = [ L("btn_no"), L("btn_yes") ], cancelIndex = 0;
            buttons = [ L("btn_yes"), L("btn_no") ];
            cancelIndex = 1;
            var dialog = Ti.UI.createAlertDialog({
                title: "",
                message: L("widget_samantha_cancell_trip_confirm_msg"),
                buttonNames: buttons,
                cancel: cancelIndex
            });
            dialog.addEventListener("click", function clickHandler(e) {
                dialog.removeEventListener("click", clickHandler);
                if (e.index !== cancelIndex) {
                    confirmTrip(trip, false);
                    TM.push(function() {
                        analytics.track(trackingEvent.SAMANTHA_NOT_GOING, {
                            tripId: trip.id
                        });
                    }, process.backgroundDomain);
                }
            });
            dialog.show();
            break;

          case 5:        }
    }
    function deviateCurrentTrip(trip, deviationMins) {
        $.toggleSpinner(true);
        var curCardWidget = $._cardWithName($.currentCard);
        curCardWidget && _.isFunction(curCardWidget.delayTripTime) && curCardWidget.delayTripTime(deviationMins);
        Alloy.createModel("trip_deviation").deviate(trip.id, trip.schedule.baseEffectiveTime, deviationMins).then(function(data) {
            Ti.App.fireEvent("app.carma.profile.change");
        }).catch(function(e) {
            LOGGER.error("[samantha] - deviateCurrentTrip: " + JSON.stringify(e));
            $.toggleSpinner(false);
            $.whisper("samantha.show.ErrorDialog", _.extend({
                action: "USER_TRIP_DEVIATION"
            }, e));
        }).finally(function() {
            $.toggleSpinner(false);
            TM.push(function() {
                analytics.track(trackingEvent.SAMANTHA_DEVIATE_TRIP, {
                    minutes: deviationMins,
                    tripId: trip.id
                });
            }, process.backgroundDomain);
        }).done();
    }
    function acceptBooking(booking, callback) {
        var id = booking.id, self = this;
        samanthaService.acceptBooking(id).then(function() {
            callback && callback();
            Ti.App.fireEvent("carma:booking.changed");
            bookingService.emit(bookingService.BOOKING_UPDATED);
        }).catch(function(e) {
            callback && callback();
            $.whisper("samantha.show.ErrorDialog", _.extend({
                action: "USER_RIDE_BOOKING_ACCEPT"
            }, e));
        }).done();
        TM.push(function() {
            analytics.track(trackingEvent.SAMANTHA_TRIP_REQUEST_ACCEPTED_BY_DRIVER, {
                bookingId: id
            });
        }, process.backgroundDomain);
    }
    function denyBooking(booking, callback) {
        var self = this, id = booking.id;
        samanthaService.declineBooking(id).then(function() {
            callback && callback();
            Ti.App.fireEvent("carma:booking.changed");
            bookingService.emit(bookingService.BOOKING_UPDATED);
        }).catch(function(e) {
            callback && callback();
            $.whisper("samantha.show.ErrorDialog", _.extend({
                action: "USER_RIDE_BOOKING_CANCEL"
            }, e));
        }).done();
        TM.push(function() {
            analytics.track(trackingEvent.SAMANTHA_TRIP_REQUEST_DENIED_BY_DRIVER, {
                bookingId: id
            });
        }, process.backgroundDomain);
    }
    function openMap(options) {
        if (!Alloy.Globals.Map) return;
        options = options || {};
        var trip = options.trip, buddies = options.buddies || null;
        if (!trip) return;
        var wrapperWindow = Alloy.createController("simpleWindow").getView();
        var mapView = Alloy.createWidget("ma.car.app.common", "map", {
            trip: trip,
            buddies: buddies,
            dontUseTime: options.dontUseTime
        });
        var navigation = Alloy.Globals.Nav.MapView = Alloy.createController("navigation");
        navigation.init({
            title: L(buddies ? "widget_samantha_confirmed_trip_map" : "widget_samantha_trip_map"),
            index: mapView,
            historyLimit: 20,
            mainWindow: wrapperWindow,
            transition: "none",
            modal: true
        });
        wrapperWindow.backgroundColor = "#fff";
        wrapperWindow.addEventListener("close", function handler() {
            wrapperWindow.removeEventListener("close", handler);
            Alloy.Globals.Nav.MapView = null;
            mapView.onDestroy();
            mapView = null;
        });
        wrapperWindow.open();
    }
    function showErrorDialog(options) {
        errorHandler.noticeError(_.extend({
            callback: function() {
                Ti.App.fireEvent("carma:booking.changed");
                bookingService.emit(bookingService.BOOKING_UPDATED);
            }
        }, options));
    }
    function doUpdateSamanthaContent(options) {
        $.toggleSpinner(true);
        if (!network.isConnectionGood() || samanthaRequestTimeout && null === $.currentCard) {
            $.toggleSpinner(false);
            samanthaRequestTimeout = false;
            samanthaRefreshing = false;
            if (null === $.currentCard) {
                $.showCard("noInternet", {});
                $.trigger("tripcard.show.inviteFriend");
                Ti.App.addEventListener("carma:network.change", function internetHandler(e) {
                    if (e.connected && e.experience) {
                        Ti.App.removeEventListener("carma:network.change", internetHandler);
                        samanthaRequestTimeout = false;
                        samanthaRefreshing = false;
                        doUpdateSamanthaContent(options);
                    }
                });
            }
            return Q.fcall(function() {
                return true;
            });
        }
        startTimeoutTimer();
        return loadBookingandTrips(options);
    }
    function startTimeoutTimer() {
        samanthaRequestTimeout = false;
        clearTimeout(samanthaRequestTimeoutTimer);
        samanthaRequestTimeoutTimer = setTimeout(function() {
            samanthaRequestTimeout = true;
            doUpdateSamanthaContent($.screenOptions);
        }, 1e4);
    }
    function loadBookingandTrips(options) {
        $.toggleSpinner(true);
        samanthaService.getNext2Trips().then(function(data) {
            clearTimeout(samanthaRequestTimeoutTimer);
            var userTrips = JSON.parse(data.apiCallResponses[0].body);
            var trips = JSON.parse(data.apiCallResponses[1].body).trips;
            checkNextDisplayTrip(userTrips, trips, options);
        }).catch(function(e) {
            lastData = null;
            $.disappear();
            e && e.errorCode && e.status && scheduleNextSamanthaRefreshTime(options, 5e3);
        }).finally(function() {
            samanthaRefreshing = false;
            $.toggleSpinner(false);
        }).done();
    }
    function checkNextDisplayTrip(userTrips, trips, options) {
        var numEnabledTrips = 0;
        var numDisabledTrips = 0;
        _.each(userTrips.trips, function(trip) {
            trip.enabled ? numEnabledTrips++ : numDisabledTrips++;
        });
        if (!trips || trips.length < 1) {
            var notrips = {};
            notrips.numEnabledTrips = numEnabledTrips;
            notrips.numDisabledTrips = numDisabledTrips;
            notrips.type = notrips.userRole;
            var currentData = JSON.stringify(notrips);
            if (currentData === lastData && !options.forceUpdate) {
                LOGGER.error("[samantha] - skipping rendering for the same data...");
                return;
            }
            lastData = currentData;
            pickACard(notrips, options);
            return;
        }
        var trip1 = trips[0], trip2 = trips.length > 1 ? trips[1] : {};
        trip1.numEnabledTrips = numEnabledTrips;
        trip1.numDisabledTrips = numDisabledTrips;
        trip1.type = trip1.userRole;
        trip2.numEnabledTrips = numEnabledTrips;
        trip2.numDisabledTrips = numDisabledTrips;
        trip2.type = trip2.userRole;
        if (trip1.schedule.effectiveTime >= Date.now() - 9e5) {
            if (trip1.bookings && trip1.bookings.length > 0) for (var i = 0; i < trip1.bookings.length; i++) {
                var booking = trip1.bookings[i];
                if ("AUTOMATIC" === booking.trigger && !("COMPLETED" !== trip1.autoRideMatchStatus && "EXPIRED" !== trip1.autoRideMatchStatus || "COMPLETED_RIDER_PICKUP" !== booking.state && "COMPLETED_DRIVER_PICKUP" !== booking.state)) {
                    var currentData = JSON.stringify(trip2);
                    if (currentData === lastData && !options.forceUpdate) {
                        LOGGER.error("[samantha] - skipping rendering for the same data...");
                        return;
                    }
                    lastData = currentData;
                    pickACard(trip2, options);
                    return;
                }
            }
            var currentData = JSON.stringify(trip1);
            if (currentData === lastData && !options.forceUpdate) {
                LOGGER.error("[samantha] - skipping rendering for the same data...");
                return;
            }
            lastData = currentData;
            pickACard(trip1, options);
            return;
        }
        if (trip1.bookings && trip1.bookings.length > 0) for (var i = 0; i < trip1.bookings.length; i++) {
            var booking = trip1.bookings[i];
            if ("AUTOMATIC" === booking.trigger && ("ACCEPTED" === booking.state || "REQUESTED" === booking.state) && "ACCEPTED" === getUserState(booking, true)) {
                var currentData = JSON.stringify(trip1);
                if (currentData === lastData && !options.forceUpdate) {
                    LOGGER.error("[samantha] - skipping rendering for the same data...");
                    return;
                }
                lastData = currentData;
                pickACard(trip1, options);
                return;
            }
        }
        var currentData = JSON.stringify(trip2);
        if (currentData === lastData && !options.forceUpdate) {
            LOGGER.error("[samantha] - skipping rendering for the same data...");
            return;
        }
        lastData = currentData;
        pickACard(trip2, options);
        return;
    }
    function scheduleNextSamanthaRefreshTime(options, timeout) {
        clearTimeout(refreshTimeoutId);
        refreshTimeoutId = setTimeout(function() {
            $.updateSamanthaContent(options);
        }, timeout || 6e5);
    }
    function checkTripTime(trip) {
        var _effectiveDate = new Date(parseFloat(trip.schedule.effectiveTime)), _currentDate = new Date(), bygone = false;
        bygone = _effectiveDate.getTime() - _currentDate.getTime() < -9e5;
        _effectiveDate.setHours(0, 0, 0, 0);
        _currentDate.setHours(0, 0, 0, 0);
        isToday = _effectiveDate.getTime() === _currentDate.getTime();
        return !bygone && isToday;
    }
    function getConfirmedBookingsFromTrip(trips, trip) {
        var bookings = [], myUserId = require("services/resource_owner_service").singleton().get("userId");
        _.each(trips.bookings, function(booking) {
            if ("ACCEPTED" === booking.state && "AUTOMATIC" === booking.trigger) {
                var isNextTrip = (booking.sourceTrip && booking.trip && myUserId === booking.sourceTrip.ownerUid ? booking.sourceTrip.baseTrip.id : booking.trip.baseTrip.id) === trip.id;
                if (isNextTrip) {
                    booking.buddy = myUserId === booking.driver.userId ? booking.rider : booking.driver;
                    bookings.push(booking);
                }
            }
        });
        return bookings;
    }
    function getBookingRequestsForNextTrip(trips, trip) {
        var requests = [], waitings = [], myUserId = require("services/resource_owner_service").singleton().get("userId");
        _.each(trips.bookings, function(booking) {
            var _trip = "DRIVE" === trip.type ? booking.trip : booking.sourceTrip;
            if ("AUTOMATIC" === booking.trigger && "REQUESTED" === booking.state && _trip && _trip.baseTrip.id === trip.id) {
                booking.buddy = myUserId === booking.driver.userId ? booking.rider : booking.driver;
                "REQUESTED" !== booking.targetUserState || "REQUESTED" !== booking.sourceUserState || didITriggeredSamantha(booking) ? didITriggeredSamantha(booking) && "ACCEPTED" === booking.targetUserState && "REQUESTED" === booking.sourceUserState ? requests.push(booking) : "ACCEPTED" === booking.targetUserState && "REQUESTED" === booking.sourceUserState && waitings.push(booking) : requests.push(booking);
            }
        });
        return _.union(requests, waitings);
    }
    function getExpiredBookingRequestFrom(trips, trip) {
        var expired = [], myUserId = require("services/resource_owner_service").singleton().get("userId");
        _.each(trips.bookings, function(booking) {
            if ("AUTOMATIC" === booking.trigger) if ("ACCEPTED" === getUserState(booking, true) && "CANCELLED" !== getUserState(booking, false) && ("CANCELLED_TOO_OLD" === booking.state || "CANCELLED" === booking.state) || !didITriggeredSamantha(booking) && "REQUESTED" === booking.targetUserState && "REQUESTED" === booking.sourceUserState && "CANCELLED" === booking.state) {
                if (!samanthaService.isBookingRead(booking.id)) {
                    booking.tripMatchState = "EXPIRED";
                    expired.push(booking);
                }
            } else if ("CANCELLED" === booking.state && "CANCELLED" === getUserState(booking, false)) {
                if (!samanthaService.isBookingRead(booking.id)) {
                    booking.tripMatchState = "CANCELLED";
                    expired.push(booking);
                }
            } else if ("DECLINED" === booking.state && "DECLINED" === getUserState(booking, false) && "ACCEPTED" === getUserState(booking, true) && !samanthaService.isBookingRead(booking.id)) {
                booking.tripMatchState = "DECLINED";
                expired.push(booking);
            }
        });
        return expired;
    }
    function getUserState(tripRequest, isForMe) {
        var initiator = "RIDER" === tripRequest.initiator ? tripRequest.rider : tripRequest.driver, myUserId = require("services/resource_owner_service").singleton().get("userId"), myState = null, otherState = null;
        if (initiator.userId !== myUserId) {
            myState = tripRequest.targetUserState;
            otherState = tripRequest.sourceUserState;
        } else {
            otherState = tripRequest.targetUserState;
            myState = tripRequest.sourceUserState;
        }
        return isForMe ? myState : otherState;
    }
    function didITriggeredSamantha(tripRequest) {
        var initiator = "RIDER" === tripRequest.initiator ? tripRequest.rider : tripRequest.driver, myUserId = require("services/resource_owner_service").singleton().get("userId");
        return initiator.userId.toString() === myUserId.toString();
    }
    function isSamanthaFailed(trips, trip) {
        var bookings = [];
        _.each(trips.bookings, function(booking) {
            var _isDriving = "DRIVE" === trip.type;
            var _trip = "DRIVE" === trip.type ? booking.trip : booking.sourceTrip;
            var _candidateTrip = "DRIVE" === trip.type ? booking.sourceTrip : booking.trip;
            if ("CANCELLED_TOO_OLD" === booking.state || "DECLINED" === booking.state || "CANCELLED" === booking.state) return;
            if (!booking.targetUserState && ("CANCELLED_TOO_OLD" === booking.targetUserState || "DECLINED" === booking.targetUserState || "CANCELLED" === booking.targetUserState)) return;
            if (!booking.sourceUserState && ("CANCELLED_TOO_OLD" === booking.sourceUserState || "DECLINED" === booking.sourceUserState || "CANCELLED" === booking.sourceUserState)) return;
            return bookings.push(booking);
        });
        return 0 === bookings.length;
    }
    function addAlertWindow(bookings) {
        var canceledUserNames = [], declinedUserNames = [], expiredUserNames = [], canceledUsers = "", declinedUsers = "", expiredUsers = "", okText = L("btn_ok");
        var myUserId = require("services/resource_owner_service").singleton().get("userId");
        for (var i = 0; i < bookings.length; i++) {
            var booking = bookings[i];
            var coordinator = booking.rider.userId === myUserId ? booking.driver : booking.rider;
            switch (booking.tripMatchState) {
              case "DECLINED":
                declinedUserNames.push(coordinator.alias);
                samanthaService.markBookingAsRead(booking.id, booking.timestamp);
                break;

              case "CANCELLED":
                canceledUserNames.push(coordinator.alias);
                samanthaService.markBookingAsRead(booking.id, booking.timestamp);
                break;

              case "EXPIRED":
                expiredUserNames.push(coordinator.alias);
                samanthaService.markBookingAsRead(booking.id, booking.timestamp);
            }
        }
        canceledUsers = generateAlertName(canceledUserNames);
        declinedUsers = generateAlertName(declinedUserNames);
        expiredUsers = generateAlertName(expiredUserNames);
        if (declinedUsers.length > 0) {
            var title = L("widget_samantha_declined_trip_match_alert_title");
            var message = L("widget_samantha_declined_trip_match_caption");
            var alert = Ti.UI.createAlertDialog({
                title: title,
                message: String.format(message, declinedUsers),
                ok: okText
            });
            alert.addEventListener("click", function(e) {
                showAlertWindow();
            });
            alertWindows.push(alert);
            $.lastMatchUserName = declinedUsers;
        }
        if (canceledUsers.length > 0) {
            var title = L("widget_samantha_cancelled_trip_match_alert_title");
            var message = L("widget_samantha_cancelled_trip_match_caption");
            var alert = Ti.UI.createAlertDialog({
                title: title,
                message: String.format(message, canceledUsers),
                ok: okText
            });
            alert.addEventListener("click", function(e) {
                showAlertWindow();
            });
            alertWindows.push(alert);
            $.lastMatchUserName = canceledUsers;
        }
        if (expiredUsers.length > 0) {
            var title = L("widget_samantha_expired_trip_match_alert_title");
            var message = L("widget_samantha_expired_trip_match_caption");
            var alert = Ti.UI.createAlertDialog({
                title: title,
                message: String.format(message, expiredUsers),
                ok: okText
            });
            alert.addEventListener("click", function(e) {
                showAlertWindow();
            });
            alertWindows.push(alert);
            $.lastMatchUserName = expiredUsers;
        }
        showAlertWindow();
    }
    function generateAlertName(names) {
        var name = "";
        for (var i = 0; i < names.length; i++) name += i == names.length - 1 && names.length > 1 ? " " + L("widget_home_msg_and") + " " + names[i] : i > 0 && names.length > 1 && i < names.length ? ", " + names[i] : names[i];
        return name;
    }
    function showAlertWindow() {
        alertWindows.length > 0 && alertWindows.pop().show();
    }
    function showNextMatchAlert(bookings) {
        if (!bookings || bookings.length < 1) return;
        var booking = bookings.shift(), title = null, okText = L("btn_ok"), message = null;
        var myUserId = require("services/resource_owner_service").singleton().get("userId"), coordinator = booking.rider.userId === myUserId ? booking.driver : booking.rider;
        switch (booking.tripMatchState) {
          case "DECLINED":
            title = L("widget_samantha_title_sorry");
            message = L("widget_samantha_declined_trip_match_caption");
            $.lastMatchUserName = coordinator.alias;
            break;

          case "CANCELLED":
            title = L("widget_samantha_title_sorry");
            message = L("widget_samantha_cancelled_trip_match_caption");
            $.lastMatchUserName = coordinator.alias;
            break;

          case "EXPIRED":
            title = L("widget_samantha_title_sorry");
            message = L("widget_samantha_expired_trip_match_caption");
            $.lastMatchUserName = coordinator.alias;
        }
        var alert = Ti.UI.createAlertDialog({
            title: title,
            message: String.format(message, coordinator.alias),
            ok: okText
        });
        alert.addEventListener("click", function(e) {
            showNextMatchAlert(bookings);
        });
        samanthaService.markBookingAsRead(booking.id, booking.timestamp);
        alert.show();
    }
    function getNumberOfMatches(tripRequest, trip) {
        var numberOfMatches = 0, myUserId = require("services/resource_owner_service").singleton().get("userId");
        _.each(tripRequest, function(booking) {
            didITriggeredSamantha(booking) ? "ACCEPTED" === booking.targetUserState && "REQUESTED" === booking.sourceUserState && numberOfMatches++ : "REQUESTED" === booking.targetUserState && "REQUESTED" === booking.sourceUserState && numberOfMatches++;
        });
        return numberOfMatches;
    }
    function pickACard(trips, options) {
        try {
            samanthaRefreshing = false;
            nextTrip = trips || {};
            if (0 === nextTrip.numEnabledTrips && 0 === nextTrip.numDisabledTrips) return $.showCard("noTrips", {});
            if (0 === trips.numEnabledTrips) return $.showCard("tripsDisabled", {});
            if (!nextTrip.schedule) return $.showCard("noTrips", {});
            nextTrip.matchesCount = trips.hasMatches;
            options = options || {};
            var availableScreenHeight = options.availableScreenHeight, params = {
                trip: nextTrip,
                availableScreenHeight: availableScreenHeight
            };
            nextTrip.schedule.confirmed = nextTrip.confirmed;
            if (0 == nextTrip.schedule.confirmed) {
                params.state = "CANCELED";
                $.showCard("tripConfirmation", params);
                return;
            }
            var confirmedBookings = getConfirmedBookingsFromTrip(trips, nextTrip);
            var nextTripRequests = getBookingRequestsForNextTrip(trips, nextTrip);
            var expiredBookings = getExpiredBookingRequestFrom(trips, nextTrip);
            if (expiredBookings.length > 0) {
                addAlertWindow(expiredBookings);
                bookingService.emit(bookingService.BOOKING_UPDATED);
            }
            if (nextTripRequests && nextTripRequests.length >= 1) {
                var tripRequest = nextTripRequests[0], numOfMatches = getNumberOfMatches(nextTripRequests, nextTrip);
                $.checkTripTimeDelay(tripRequest);
                if (didITriggeredSamantha(tripRequest)) {
                    if ("ACCEPTED" === tripRequest.targetUserState && "REQUESTED" === tripRequest.sourceUserState) {
                        $.showCard("tripRequest", _.extend(params, {
                            booking: tripRequest,
                            type: "rider",
                            numOfMatches: numOfMatches
                        }));
                        return;
                    }
                } else {
                    if ("REQUESTED" === tripRequest.targetUserState && "REQUESTED" === tripRequest.sourceUserState) {
                        $.showCard("tripRequest", _.extend(params, {
                            bookings: nextTripRequests,
                            type: "driver",
                            numOfMatches: numOfMatches
                        }));
                        return;
                    }
                    if ("ACCEPTED" === tripRequest.targetUserState && "REQUESTED" === tripRequest.sourceUserState) {
                        $.showCard("tripBookings", _.extend(params, {
                            bookings: confirmedBookings,
                            requests: nextTripRequests
                        }));
                        return;
                    }
                }
            }
            if (confirmedBookings && confirmedBookings.length > 0) {
                $.checkTripTimeDelay(confirmedBookings[0]);
                $.showCard("tripBookings", _.extend(params, {
                    bookings: confirmedBookings
                }));
            } else {
                var tripConfirmed = nextTrip.schedule.confirmed;
                if (tripConfirmed || 0 != nextTrip.matchesCount) {
                    var state = trips.autoRideMatchStatus;
                    if (tripConfirmed && state) if ("COMPLETED" === state || "EXPIRED" === state) {
                        params.state = state;
                        $.showCard("noSamanthaReplies", params);
                    } else $.showCard("tripConfirmed", params); else $.showCard("tripConfirmation", params);
                } else $.showCard("noMatches", params);
            }
        } catch (e) {
            LOGGER.error("[Samantha] Caught Exception: " + JSON.stringify(e));
            $.disappear();
        } finally {
            $.toggleSpinner(false);
            samanthaService.bookingsRecord().clearBefore(Date.now());
        }
    }
    var Widget = new (require("alloy/widget"))("ma.car.app.samantha");
    this.__widgetId = "ma.car.app.samantha";
    require("alloy/controllers/BaseController").apply(this, Array.prototype.slice.call(arguments));
    this.__controllerPath = "widget";
    if (arguments[0]) {
        var __parentSymbol = __processArg(arguments[0], "__parentSymbol");
        var $model = __processArg(arguments[0], "$model");
        var __itemTemplate = __processArg(arguments[0], "__itemTemplate");
    }
    var $ = this;
    var exports = {};
    var __defers = {};
    $.__views.layoutContainer = Ti.UI.createView({
        top: 0,
        width: Ti.UI.FILL,
        height: Ti.UI.SIZE,
        id: "layoutContainer"
    });
    $.__views.layoutContainer && $.addTopLevelView($.__views.layoutContainer);
    $.__views.cardsContainer = Ti.UI.createView({
        top: 0,
        visible: "false",
        width: Ti.UI.FILL,
        height: Ti.UI.SIZE,
        layout: "vertical",
        id: "cardsContainer"
    });
    $.__views.layoutContainer.add($.__views.cardsContainer);
    $.__views.editTripDialog = Ti.UI.createOptionDialog({
        id: "editTripDialog"
    });
    $.__views.transitionContainer = Ti.UI.createView({
        top: 0,
        visible: "false",
        width: Ti.UI.FILL,
        height: 0,
        id: "transitionContainer"
    });
    $.__views.layoutContainer.add($.__views.transitionContainer);
    $.__views.findingMatchContainer = Ti.UI.createView({
        top: 0,
        visible: "false",
        layout: "vertical",
        width: Ti.UI.FILL,
        height: 0,
        id: "findingMatchContainer"
    });
    $.__views.transitionContainer.add($.__views.findingMatchContainer);
    $.__views.spinnerImage = Ti.UI.createImageView({
        defaultImage: "/images/none.png",
        top: "120dp",
        image: "/images/SamanthaProgressRing.png",
        id: "spinnerImage"
    });
    $.__views.findingMatchContainer.add($.__views.spinnerImage);
    $.__views.findingMatchTitle = Ti.UI.createLabel({
        color: "#333",
        text: L("widget_samantha_confirmtip_title"),
        id: "findingMatchTitle"
    });
    $.__views.findingMatchContainer.add($.__views.findingMatchTitle);
    $.__views.findingMatchMessage = Ti.UI.createLabel({
        color: "#333",
        font: {
            fontSize: "15dp"
        },
        text: L("widget_samantha_confirmtip_message"),
        id: "findingMatchMessage"
    });
    $.__views.findingMatchContainer.add($.__views.findingMatchMessage);
    exports.destroy = function() {};
    _.extend($, $.__views);
    "use strict";
    var trackingEvent = require("/tracking/tracking_event"), analytics = require("/tracking/tracking_manager"), samanthaService = require("services/samantha_service"), errorHandler = require("/utils/errorHandler"), network = require("/bootstrap/network"), css = require("ui/css"), Q = require("vendor/q"), i18n = require("i18n/i18n"), Globals = require("/v6/infrastructure/Globals"), bookingService = Globals.IOC.get("bookingService");
    var _allowSamantha = 0, samanthaRefreshing = false, samanthaRequestTimeoutTimer = null, samanthaRequestTimeout = false, lastData = null, nextTrip = null, alertWindows = [], lastTripMatchId = null, lastTripMatchTime = null;
    exports = new (require("/carmalib/controller").extend($, {
        cards: [],
        previousCard: null,
        currentCard: null,
        screenOptions: null,
        stopSpinner: false,
        tripMatchFound: false,
        lastMatchUserName: null,
        hasPopUp: false,
        popUpWindows: [],
        isBookingConfirmed: false,
        events: {},
        configure: function() {
            $.editTripDialog.addEventListener("click", editTripAction);
        },
        display: function() {
            $.cardsContainer.show();
            $.trigger("samantha.show");
        },
        disappear: function() {
            this.cards = [];
            _.each($.cardsContainer.children, function(view) {
                $.cardsContainer.remove(view);
            });
            $.cardsContainer.hide();
            this.previousCard = null;
            this.currentCard = null;
            $.trigger("samantha.hided");
        },
        _switchCards: function(options, animate, animationDelegate) {
            var self = this;
            var fromCardWidget = options.fromCardWidget, toCardWidget = options.toCardWidget, displayOptions = options.displayOptions, needRemove = fromCardWidget && fromCardWidget.cardName !== toCardWidget.cardName, self = this;
            if (animate) if (needRemove) fromCardWidget.disappear(function() {
                toCardWidget.display(displayOptions);
                self._displayCardWidget(toCardWidget);
            }); else {
                toCardWidget.display(displayOptions);
                this._displayCardWidget(toCardWidget);
            } else if (toCardWidget.waitUntilReady) {
                if (needRemove && fromCardWidget.getView()) {
                    var originalHeight = fromCardWidget.getView().size.height;
                    originalHeight > 0 && $.layoutContainer.applyProperties({
                        height: originalHeight
                    });
                }
                var _finishSwitch = function() {
                    $.layoutContainer.applyProperties({
                        height: Ti.UI.SIZE
                    });
                    needRemove && fromCardWidget.disappear();
                    self._displayCardWidget(toCardWidget);
                };
                toCardWidget.display(displayOptions, function() {
                    animationDelegate && _.isFunction(animationDelegate) ? animationDelegate(_finishSwitch) : _finishSwitch();
                });
            } else {
                needRemove && fromCardWidget.disappear();
                toCardWidget.getView().applyProperties({
                    height: Ti.UI.SIZE
                });
                toCardWidget.display(displayOptions);
                this._displayCardWidget(toCardWidget);
            }
        },
        _displayCardWidget: function(widget) {
            if (!widget) return;
            widget.finishDisplay && _.isFunction(widget.finishDisplay) ? widget.finishDisplay() : widget.getView().show();
            LOGGER.error("[samantha] - show card: " + widget.cardName);
            $.trigger("samantha.showCard", {
                cardName: widget.cardName
            });
        },
        showCard: function(cardName, options) {
            LOGGER.info("[samantha] - show card: " + cardName);
            if (!cardName) return;
            options = options || {};
            this.previousCard = this.currentCard;
            this.currentCard = cardName;
            LOGGER.error("[samantha] - currentCard: " + this.currentCard + ", previousCard: " + this.previousCard);
            var cardWidget = this._cardWithName(cardName), prevCardWidget = this._cardWithName(this.previousCard);
            if (!cardWidget) {
                cardWidget = Alloy.createWidget("ma.car.app.samantha", cardName, {
                    samantha: this
                });
                this.cards.push(cardWidget);
                $.cardsContainer.add(cardWidget.getView());
                TM.push(function() {
                    analytics.track(trackingEvent.SAMANTHA_SHOW_CARD, {
                        cardName: cardName
                    });
                }, process.backgroundDomain);
            }
            var self = this, switchCardsOptions = {
                fromCardWidget: prevCardWidget,
                toCardWidget: cardWidget,
                displayOptions: options
            }, animationDelegate = null;
            if (prevCardWidget && prevCardWidget.customAnimation || cardWidget.customAnimation) this._switchCards(switchCardsOptions, true); else if (this.currentCard !== this.previousCard) if (prevCardWidget && "slideUpDown" === prevCardWidget.animationName || "slideUpDown" === cardWidget.animationName) this.dropCardSwitchTransition(switchCardsOptions); else {
                animationDelegate = this.defaultCardSwitchTransition();
                this._switchCards(switchCardsOptions, false, animationDelegate);
            } else this._switchCards(switchCardsOptions);
            if ("tripBookings" !== this.currentCard) {
                lastTripMatchTime = null;
                lastTripMatchId = null;
            }
            "tripBookings" !== this.currentCard && "tripRequest" !== this.currentCard && (this.isBookingConfirmed = false);
        },
        removeCard: function(card) {
            var cardWidget = card;
            "[object String]" === Object.prototype.toString.call(card) && (cardWidget = this._cardWithName(card));
            this.cards = _.without(this.cards, cardWidget);
            $.cardsContainer.remove(cardWidget.getView());
            cardWidget = null;
        },
        _cardWithName: function(cardName) {
            if (!cardName) return;
            var cardWidget = _.find(this.cards, function(card) {
                return card.cardName === cardName;
            });
            return cardWidget;
        },
        updateSamanthaVisibility: function() {
            _allowSamantha = require("featuretoggles/featuretoggles").getValue("samantha");
            var cachedPreferences = require("services/cached_preferences_service").singleton();
            cachedPreferences.getPreference("toggle_afr") && "no" === cachedPreferences.getPreference("toggle_afr") && (_allowSamantha = false);
            _allowSamantha ? this.display() : this.disappear();
        },
        updateSamanthaContent: function(options) {
            var allowSamantha = require("featuretoggles/featuretoggles").getValue("samantha");
            var cachedPreferences = require("services/cached_preferences_service").singleton();
            cachedPreferences.getPreference("toggle_afr") && "no" === cachedPreferences.getPreference("toggle_afr") && (_allowSamantha = false);
            options.forceUpdate = _allowSamantha != allowSamantha;
            _allowSamantha = allowSamantha;
            var self = this;
            TM.push(function() {
                self.updateSamanthaVisibility();
                if (!_allowSamantha) return;
                try {
                    if (samanthaRefreshing) return;
                    samanthaRefreshing = true;
                    scheduleNextSamanthaRefreshTime(options);
                    self.display();
                    !self.screenOptions && options && (self.screenOptions = options);
                    doUpdateSamanthaContent(options);
                } finally {}
            });
        },
        toggleSpinner: function(loading) {
            return;
        },
        whisper: function(words, options) {
            if (!words) return;
            options = options || {};
            switch (words) {
              case "samantha.open.map":
                openMap(options);
                break;

              case "samantha.confirm.trip":
                var trip = options.trip;
                if (!trip || trip.schedule.confirmed && trip.autoRideMatchStatus) return Q.reject();
                confirmTrip(trip, true);
                return;

              case "samantha.cancel.trip":
                var trip = options.trip;
                if (!trip) return Q.reject();
                confirmTrip(trip, false);
                return;

              case "samantha.editTripAction.dialog":
                var className = "unconfirmed-trip-options";
                nextTrip.schedule.confirmed && nextTrip.autoRideMatchStatus && (className = "confirmed-trip-options");
                css.setStyle($, {
                    "#editTripDialog": className
                });
                $.editTripDialog.show();
                break;

              case "samantha.accept.booking":
                var booking = options.booking, callback = options.callback;
                if (!booking) return;
                acceptBooking(booking, function() {
                    options.confirmTrip && options.trip && !options.trip.schedule.confirmed ? confirmTrip(options.trip, true, {
                        callback: callback,
                        noPopupPrompt: true
                    }) : callback && _.isFunction(callback) && callback();
                });
                break;

              case "samantha.deny.booking":
                var booking = options.booking, callback = options.callback;
                if (!booking) return;
                denyBooking(booking, callback);
                break;

              case "samantha.show.ErrorDialog":
                showErrorDialog(options);
            }
            $.trigger(words, options);
        },
        defaultCardSwitchTransition: function() {
            var matrix = Ti.UI.create2DMatrix();
            matrix = matrix.scale(.1, .1);
            var _animation = Ti.UI.createAnimation({
                opacity: 0,
                duration: 500
            });
            matrix = matrix.scale(10, 10);
            var _reverseAnimation = Ti.UI.createAnimation({
                opacity: 1,
                duration: 500
            });
            var self = this;
            return function(callback) {
                $.cardsContainer.animate(_animation, function() {
                    callback && _.isFunction(callback) && callback.call(self);
                    $.cardsContainer.animate(_reverseAnimation);
                });
            };
        },
        dropCardSwitchTransition: function(options) {
            var self = this, displayOptions = options.displayOptions, fromCardWidget = options.fromCardWidget, toCardWidget = options.toCardWidget;
            this.whisper("samantha.slideupdown.animation", {
                animateSlideup: !!fromCardWidget,
                callback: function() {
                    fromCardWidget && fromCardWidget.disappear();
                    toCardWidget.display(displayOptions);
                }
            });
        },
        checkTripTimeDelay: function(bookingRequest) {
            if ("DRIVE" === nextTrip.type || !nextTrip || !bookingRequest) return;
            var bookingTrip = bookingRequest.trip && bookingRequest.trip.baseTrip, bookingTime = null, delayedInMins = null, buddy = bookingRequest.buddy;
            bookingTrip.schedule && (bookingTime = bookingRequest.trip.baseTrip.schedule.effectiveTime);
            if (bookingTrip && lastTripMatchId === bookingTrip.id) {
                if (bookingTime && lastTripMatchTime) {
                    var timeDiff = bookingTime - lastTripMatchTime;
                    delayedInMins = Math.floor(timeDiff / 6e4);
                }
                if (delayedInMins) {
                    var message = L("widget_samantha_trip_delay_popup_message_male");
                    "f" === buddy.gender && (message = L("widget_samantha_trip_delay_popup_message_female"));
                    Ti.UI.createAlertDialog({
                        title: L("widget_samantha_trip_delay_popup_title"),
                        message: String.format(message, buddy.alias, delayedInMins.toString())
                    }).show();
                }
            }
            lastTripMatchId = bookingTrip.id;
            lastTripMatchTime = bookingTime;
        },
        setTripMatchFound: function(found) {
            this.tripMatchFound = found;
        },
        setLastMatchUserName: function(name) {
            this.lastMatchUserName = name;
        },
        addPopUp: function(popUpWin) {
            if (this.hasPopUp) this.popUpWindows.push(popUpWin); else {
                popUpWin.presentModalView({
                    animated: true,
                    numOfMatches: 1
                });
                this.hasPopUp = true;
            }
        },
        getNumOfTripMatch: function() {
            var num = 0;
            for (var i = 0; i < this.popUpWindows.length; i++) this.popUpWindows[i] && this.popUpWindows[i].isTripMatch() && num++;
            return num;
        },
        setPopupState: function(state) {
            this.hasPopUp = state;
        },
        checkPopUps: function() {
            this.hasPopUp = false;
            var self = this;
            if (!this.hasPopUp && this.popUpWindows.length > 0) {
                var numOfMatches = this.getNumOfTripMatch();
                var popUpWin = this.popUpWindows.pop();
                setTimeout(function() {
                    popUpWin.presentModalView({
                        animated: true,
                        numOfMatches: numOfMatches
                    });
                }, 1e3);
                this.hasPopUp = true;
            }
        },
        checkBookingStatus: function() {
            return this.isBookingConfirmed;
        },
        setBookingStatus: function(state) {
            this.isBookingConfirmed = state;
        }
    }))();
    var refreshTimeoutId = null;
    _.extend($, exports);
    $.postConstruct && $.postConstruct();
}

var Alloy = require("alloy"), Backbone = Alloy.Backbone, _ = Alloy._;

module.exports = Controller;