function WPATH(s) {
    var index = s.lastIndexOf("/");
    var path = -1 === index ? "ma.car.app.tripcard/" + s : s.substring(0, index) + "/ma.car.app.tripcard/" + s.substring(index + 1);
    return true && 0 !== path.indexOf("/") ? "/" + path : path;
}

function __processArg(obj, key) {
    var arg = null;
    if (obj) {
        arg = obj[key] || null;
        delete obj[key];
    }
    return arg;
}

function Controller() {
    function openMap(options) {
        if (!Alloy.Globals.Map) return;
        options = options || {};
        var trip = options.trip, buddies = options.buddies || null;
        if (!trip) return;
        var wrapperWindow = Alloy.createController("simpleWindow").getView();
        var mapView = Alloy.createWidget("ma.car.app.common", "map", {
            trip: trip,
            buddies: buddies
        });
        var navigation = Alloy.Globals.Nav.MapView = Alloy.createController("navigation");
        navigation.init({
            title: String.format(L("widget_profile_mapview_title"), i18n.formatTime(moment(trip.schedule.effectiveTime)), L("RIDE" === trip.type ? "mode_rider" : "mode_driver")),
            index: mapView,
            historyLimit: 20,
            mainWindow: wrapperWindow,
            transition: "none",
            modal: true
        });
        wrapperWindow.backgroundColor = "#fff";
        wrapperWindow.addEventListener("close", function handler() {
            wrapperWindow.removeEventListener("close", handler);
            mapView.onDestroy();
            mapView = null;
            Alloy.Globals.Nav.MapView = null;
        });
        wrapperWindow.open();
    }
    function showErrorDialog(options) {
        errorHandler.noticeError(_.extend({
            callback: function() {
                Ti.App.fireEvent("carma:booking.changed");
                var Globals = require("/v6/infrastructure/Globals"), bookingService = Globals.IOC.get("bookingService");
                bookingService.emit(bookingService.BOOKING_UPDATED);
            }
        }, options));
    }
    function doUpdateTripCardContent(options) {
        if (!network.isConnectionGood() || tripcardRequestTimeout && null === $.currentCard) {
            tripcardRequestTimeout = false;
            tripcardRefreshing = false;
            if (null === $.currentCard) {
                $.showCard("noInternet", {});
                $.trigger("tripcard.show.inviteFriend");
                Ti.App.addEventListener("carma:network.change", function internetHandler(e) {
                    if (e.connected && e.experience) {
                        Ti.App.removeEventListener("carma:network.change", internetHandler);
                        tripcardRequestTimeout = false;
                        tripcardRefreshing = false;
                        doUpdateTripCardContent(options).done();
                    }
                });
            }
            return Q.fcall(function() {
                return true;
            });
        }
        startTimeoutTimer();
        return loadBookingandTrips(options);
    }
    function doForceUpdateTripCardContent(options) {
        if (network.isConnectionGood()) return loadBookingandTrips(options);
        if (null === $.currentCard) {
            $.showCard("noInternet", {});
            $.trigger("tripcard.show.inviteFriend");
        }
        return Q.fcall(function() {
            return true;
        });
    }
    function startTimeoutTimer() {
        tripcardRequestTimeout = false;
        clearTimeout(tripcardRequestTimeoutTimer);
        tripcardRequestTimeoutTimer = setTimeout(function() {
            tripcardRequestTimeout = true;
            doUpdateTripCardContent($.screenOptions).done();
        }, 1e4);
    }
    function loadBookingandTrips(options) {
        var bookings = null;
        var promise = require("services/booking_service").findAll(false);
        promise.then(function(data) {
            bookings = data;
            return require("services/trip_service").bestMatches();
        }).then(function(trips) {
            clearTimeout(tripcardRequestTimeoutTimer);
            trips.bookings = bookings.rideBookings || [];
            trips.nextTripTimestamp = -1;
            var currentData = JSON.stringify(trips);
            if (currentData === lastData && !options.forceUpdate) {
                LOGGER.error("[tripcard] - skipping rendering for the same data...");
                tripcardRefreshing = false;
                return;
            }
            lastData = currentData;
            pickACard(trips, options);
        }).catch(function(e) {
            lastData = null;
            $.disappear();
            tripcardRefreshing = false;
            scheduleNextTripCardRefreshTime(options, 5e3);
        }).finally(function() {
            $.loading.applyProperties({
                style: Ti.UI.ActivityIndicatorStyle.DARK
            });
            $.toggleSpinner(false);
            tripcardRefreshing = false;
        });
        return promise;
    }
    function scheduleNextTripCardRefreshTime(options, timeout) {
        clearTimeout(refreshTimeoutId);
        refreshTimeoutId = setTimeout(function() {
            $.updateTripCardContent(options);
        }, timeout || 6e5);
    }
    function getConfirmedBookingsFromTrip(trips, trip) {
        var bookings = [], myUserId = require("services/resource_owner_service").singleton().get("userId");
        _.each(trips.bookings, function(booking) {
            if ("ACCEPTED" === booking.state && "MANUAL" === booking.trigger) {
                var isNextTrip = (booking.sourceTrip && myUserId === booking.sourceTrip.ownerUid ? booking.sourceTrip.baseTrip.id : booking.trip.baseTrip.id) === trip.id;
                if (isNextTrip) {
                    booking.buddy = myUserId === booking.driver.userId ? booking.rider : booking.driver;
                    bookings.push(booking);
                }
            }
        });
        return bookings;
    }
    function pickACard(trips, options) {
        try {
            nextTrip = trips && trips.tripMatchedAgainst && trips.tripMatchedAgainst.baseTrip || null;
            if (!nextTrip && 0 === trips.numDisabledTrips) {
                $.trigger("tripcard.show.inviteFriend");
                return $.showCard("noTrips", {});
            }
            if (0 === trips.numEnabledTrips) {
                $.trigger("tripcard.show.inviteFriend");
                return $.showCard("tripsDisabled", {});
            }
            nextTrip.matchesCount = trips.paginator.totalResults;
            options = options || {};
            var availableScreenHeight = options.availableScreenHeight, params = {
                trip: nextTrip,
                availableScreenHeight: availableScreenHeight
            };
            if (0 === nextTrip.matchesCount && trips.bookings.length <= 0) if (require("gps/gps_manager").lastLocation()) getInitialActivity(options, trips, params); else {
                Ti.App.addAppEventListener("carma:valid.location.update", function checkSelfLocation() {
                    Ti.App.removeAppEventListener("carma:valid.location.update", checkSelfLocation);
                    LOGGER.info("[GPS Manager] - Location available now. Checking again for Recent Activity");
                    getInitialActivity(options, trips, params);
                });
                $.trigger("tripcard.show.inviteFriend");
                options.bookings = trips.bookings;
                getNext2Trips(options);
            } else {
                $.trigger("tripcard.show.inviteFriend");
                options.bookings = trips.bookings;
                getNext2Trips(options);
            }
        } catch (e) {
            LOGGER.error("[tripcard] Caught Exception: " + JSON.stringify(e));
            tripcardRefreshing = false;
            $.disappear();
        } finally {
            $.toggleSpinner(false);
            tripcardRefreshing = false;
        }
    }
    function getInitialActivity(options, trips, params) {
        require("/services/activity_service").findAll().then(function(data) {
            if (null !== data && data.merged.length > 0) {
                $.trigger("tripcard.show.inviteFriend");
                options.bookings = trips.bookings;
                getNext2Trips(options);
            } else {
                $.showCard("noMatches", params);
                $.trigger("tripcard.show.noMatches");
            }
        });
    }
    function getNext2Trips(options) {
        require("services/trip_service").getNextnTrips(2).then(function(data) {
            if (data.tripList && data.tripList.length > 0) {
                options = options || {};
                var availableScreenHeight = options.availableScreenHeight, params = {
                    trips: data.tripList,
                    bookings: options.bookings,
                    availableScreenHeight: availableScreenHeight
                };
                $.showCard("trips", params);
            } else LOGGER.error("[tripcard] can't find the next 2 trips");
        });
    }
    var Widget = new (require("alloy/widget"))("ma.car.app.tripcard");
    this.__widgetId = "ma.car.app.tripcard";
    require("alloy/controllers/BaseController").apply(this, Array.prototype.slice.call(arguments));
    this.__controllerPath = "widget";
    if (arguments[0]) {
        var __parentSymbol = __processArg(arguments[0], "__parentSymbol");
        var $model = __processArg(arguments[0], "$model");
        var __itemTemplate = __processArg(arguments[0], "__itemTemplate");
    }
    var $ = this;
    var exports = {};
    var __defers = {};
    $.__views.layoutContainer = Ti.UI.createView({
        top: 0,
        width: "100%",
        height: Ti.UI.SIZE,
        layout: "vertical",
        id: "layoutContainer"
    });
    $.__views.layoutContainer && $.addTopLevelView($.__views.layoutContainer);
    $.__views.cardsContainer = Ti.UI.createView({
        top: 0,
        visible: false,
        width: Ti.UI.FILL,
        height: Ti.UI.SIZE,
        layout: "vertical",
        id: "cardsContainer"
    });
    $.__views.layoutContainer.add($.__views.cardsContainer);
    exports.destroy = function() {};
    _.extend($, $.__views);
    "use strict";
    var trackingEvent = require("/tracking/tracking_event"), analytics = require("/tracking/tracking_manager"), errorHandler = require("/utils/errorHandler"), gps = require("gps/gps_manager"), network = require("/bootstrap/network"), i18n = require("i18n/i18n"), Q = require("vendor/q");
    var _allowSamantha = 0, tripcardRefreshing = false, tripcardRequestTimeoutTimer = null, tripcardRequestTimeout = false, lastData = null, nextTrip = null;
    exports = new (require("/carmalib/controller").extend($, {
        cards: [],
        previousCard: null,
        currentCard: null,
        screenOptions: null,
        events: {},
        configure: function() {},
        display: function() {
            $.cardsContainer.show();
            $.trigger("tripcard.show");
        },
        disappear: function() {
            this.cards = [];
            _.each($.cardsContainer.children, function(view) {
                $.cardsContainer.remove(view);
            });
            $.cardsContainer.hide();
            this.previousCard = null;
            this.currentCard = null;
            $.trigger("tripcard.hided");
        },
        _switchCards: function(options, animate) {
            var fromCardWidget = options.fromCardWidget, toCardWidget = options.toCardWidget, displayOptions = options.displayOptions, needRemove = fromCardWidget && fromCardWidget.cardName !== toCardWidget.cardName;
            if (animate) needRemove ? fromCardWidget.disappear(function() {
                toCardWidget.display(displayOptions);
            }) : toCardWidget.display(displayOptions); else {
                needRemove && fromCardWidget.disappear();
                toCardWidget.display(displayOptions);
            }
        },
        showCard: function(cardName, options) {
            LOGGER.info("[tripCard] - show card: " + cardName);
            if (!cardName) return;
            options = options || {};
            this.previousCard = this.currentCard;
            this.currentCard = cardName;
            LOGGER.error("[tripCard] - currentCard: " + this.currentCard + ", previousCard: " + this.previousCard);
            var cardWidget = this._cardWithName(cardName), prevCardWidget = this._cardWithName(this.previousCard);
            if (!cardWidget) {
                cardWidget = Alloy.createWidget("ma.car.app.tripcard", cardName, {
                    tripcard: this
                });
                this.cards.push(cardWidget);
                $.cardsContainer.add(cardWidget.getView());
            }
            var self = this, switchCardsOptions = {
                fromCardWidget: prevCardWidget,
                toCardWidget: cardWidget,
                displayOptions: options
            };
            prevCardWidget && prevCardWidget.customAnimation || cardWidget.customAnimation ? this._switchCards(switchCardsOptions, true) : this._switchCards(switchCardsOptions);
            "trips" === cardName || $.trigger("tripcard.showCard", {
                cardName: cardName
            });
        },
        removeCard: function(card) {
            var cardWidget = card;
            "[object String]" === Object.prototype.toString.call(card) && (cardWidget = this._cardWithName(card));
            this.cards = _.without(this.cards, cardWidget);
            $.cardsContainer.remove(cardWidget.getView());
        },
        _cardWithName: function(cardName) {
            if (!cardName) return;
            var cardWidget = _.find(this.cards, function(card) {
                return card.cardName === cardName;
            });
            return cardWidget;
        },
        updateTripCardVisibility: function() {
            var allowSamanthaNow = require("featuretoggles/featuretoggles").getValue("samantha");
            _allowSamantha = allowSamanthaNow;
            var cachedPreferences = require("services/cached_preferences_service").singleton();
            cachedPreferences.getPreference("toggle_afr") && "no" === cachedPreferences.getPreference("toggle_afr") && (_allowSamantha = false);
            _allowSamantha ? this.disappear() : this.display();
        },
        updateTripCardContent: function(options) {
            _allowSamantha = require("featuretoggles/featuretoggles").getValue("samantha");
            var cachedPreferences = require("services/cached_preferences_service").singleton();
            cachedPreferences.getPreference("toggle_afr") && "no" === cachedPreferences.getPreference("toggle_afr") && (_allowSamantha = false);
            var self = this;
            TM.push(function() {
                self.updateTripCardVisibility();
                if (_allowSamantha) return;
                try {
                    if (tripcardRefreshing) return;
                    tripcardRefreshing = true;
                    !self.screenOptions && options && (self.screenOptions = options);
                    scheduleNextTripCardRefreshTime(options);
                    self.display();
                    doUpdateTripCardContent(options).done();
                } finally {}
            });
        },
        refreshContent: function(options) {
            if (_allowSamantha) return Q.fcall(function() {
                return true;
            });
            if (tripcardRefreshing) return Q.fcall(function() {
                return true;
            });
            tripcardRefreshing = true;
            !this.screenOptions && options && (this.screenOptions = options);
            this.display();
            return doForceUpdateTripCardContent(options);
        },
        toggleSpinner: function(loading) {
            return;
        },
        whisper: function(words, options) {
            var Q = require("/vendor/q");
            if (!words) return;
            options = options || {};
            switch (words) {
              case "tripcard.open.map":
                openMap(options);
                break;

              case "tripcard.show.ErrorDialog":
                showErrorDialog(options);
            }
            $.trigger(words, options);
        },
        defaultCardSwitchTransition: function(callback) {
            var matrix = Ti.UI.create2DMatrix();
            matrix = matrix.scale(.1, .1);
            var _animation = Ti.UI.createAnimation({
                transform: matrix,
                opacity: 0,
                duration: 250
            });
            matrix = matrix.scale(10, 10);
            var _reverseAnimation = Ti.UI.createAnimation({
                transform: matrix,
                opacity: 1,
                duration: 250
            });
            var self = this;
            $.cardsContainer.animate(_animation, function() {
                callback && _.isFunction(callback) && callback.call(self);
                $.cardsContainer.animate(_reverseAnimation);
            });
        },
        dropCardSwitchTransition: function(options) {
            var self = this, displayOptions = options.displayOptions, fromCardWidget = options.fromCardWidget, toCardWidget = options.toCardWidget;
            this.whisper("tripcard.slideupdown.animation", {
                animateSlideup: !!fromCardWidget,
                callback: function() {
                    fromCardWidget && fromCardWidget.disappear();
                    toCardWidget.display(displayOptions);
                }
            });
        },
        markForceFullReloadingNextTime: function() {
            lastData = null;
        }
    }))();
    var refreshTimeoutId = null;
    _.extend($, exports);
    $.postConstruct && $.postConstruct();
}

var Alloy = require("alloy"), Backbone = Alloy.Backbone, _ = Alloy._;

module.exports = Controller;