var Alloy = require("alloy"), _ = Alloy._, Backbone = Alloy.Backbone;

"use strict";

var appState = require("/bootstrap/appstate"), Bootstrap = require("/v6/Bootstrap");

var _handleAndroidIntent = function() {
    var intent = Ti.Android.currentActivity.getIntent();
    if (!intent) return;
    intent.hasExtra("payload") ? require("/pushnotification/pushNotificationHandler").handle(JSON.parse(intent.getStringExtra("payload")), true) : Alloy.Globals.loadUrl = intent.getStringExtra("url") || intent.data || Ti.App.INTENT_DATA;
};

var initTaskManager = function() {
    TM = require("/bootstrap/taskManager");
    var nextTickCalled = false;
    process = {
        nextTick: function(task) {
            nextTickCalled = true;
            TM.push(function() {
                task();
            }, process.backgroundDomain);
        },
        defaultDomain: require("/bootstrap/domain").create("default"),
        backgroundDomain: require("/bootstrap/domain").create("background"),
        domain: null
    };
    process.defaultDomain.enter();
    var Q = require("/vendor/q");
    TM.init(Q);
    Q().then(function() {
        if (!nextTickCalled) {
            LOGGER.error("Q is not setup to call the Carma task manager");
            throw new Error("Q is not setup to call the Carma task manager");
        }
    }).done();
};

var initAppGlobals = function() {
    Ti.App.idleTimerDisabled = false;
    Alloy.Globals.APP = {};
    Alloy.Globals.APP.VERSION = "6.5.4";
    Alloy.Globals.APP.REVISION = "16";
    Alloy.Globals.APP.TIMESTAMP = "1429787466965";
    Alloy.Globals.APP.BUNDLEID = "1429787386";
    Ti.userAgent += "#bundleId=" + Alloy.Globals.APP.BUNDLEID;
    INSTRUMENT = require("debug/instrument").properties;
    INSTRUMENT.log && (Alloy.Globals.LOG_LEVEL = INSTRUMENT.log);
};

!function MAIN() {
    initAppGlobals();
    LOGGER = require("log/logger");
    require("bootstrap/style").preCalculate();
    if (INSTRUMENT.test && !INSTRUMENT.test.ui) return;
    if (appState.isAlloyJsExecuted()) _handleAndroidIntent(); else {
        appState.startAlloyJs();
        initTaskManager();
        Bootstrap();
        TABS_HELPER = require("ui/tabs_helper");
        var comyoziomodule;
        require("bootstrap/env").init();
        Alloy.Globals.ti = {};
        Alloy.Globals.Nav = {};
        Alloy.Globals.Boarding = {};
        Alloy.Globals.Boarding.currentlySyncing = false;
        Alloy.Globals.Map = require("ti.map");
        var eventManager = require("/ui/common/eventManager").createEventManager();
        eventManager.manageEvents("Ti.App", Ti.App);
        var moment = require("vendor/moment");
        moment.lang(Ti.Locale.getCurrentLanguage());
        require("bootstrap/network").init();
        require("featuretoggles/featuretoggles").init();
        require("/pushnotification/registerPushNotification").init();
        require("/bootstrap/patch");
        require("/bootstrap/coachmark_manager");
        require("tracking/tracking_manager").init();
        require("facebook/facebook_manager").init(require("com.ti.facebook"));
        require("ui/common/progress_indicator").init();
        require("services/tracking_config_service").fetchTrackingConfig();
        _handleAndroidIntent();
        require("/pushnotification/pushNotificationHandler").registerNotificationHandler();
        Carma = {};
        Carma.UI = require("ui/carma");
        Carma.Window = {};
        Carma.Window.Tools = false;
        GeoIPCountryCode = "US";
        GeoIPCountryCodeIsDefault = true;
        TM.push(function() {
            var geoip = Alloy.createModel("geoip");
            geoip.fetch().then(function(data) {
                GeoIPCountryCode = data.country_code;
                GeoIPCountryCodeIsDefault = false;
            }).catch(function(e) {
                GeoIPCountryCode = "US";
                GeoIPCountryCodeIsDefault = true;
            });
        }, process.backgroundDomain);
        INSTRUMENT.test && INSTRUMENT.test.ui && (TS = require("/test/testService"));
        require("bootstrap/lifecycle").start();
    }
}();

Alloy.createController("index");