"use strict";

var GlobalEvents = require("/v6/common/GlobalEvents");

var linker = {
    _mappings: {
        verifyemail: /verifyemail\?(.*)$/,
        tolltag: /(\??.*)tolltag#myprofile$/,
        myprofile: /(\??.*)#myprofile$/,
        mytrips: /mytrips\??(.*)$/,
        login: /login\??(.*)$/,
        signup: /signup\?(.*)&skin=signup$/,
        viewprofile: /index\?(.*)#userprofile\/(.*)\/profile$/,
        viewprofile_s_det: /s\/det\/viewprofile\?uid=(.*)$/,
        viewtrip_s_det: /s\/det\/viewtrips\?uid=(.*)$/,
        viewtrip: /index(\?(.*))?#userprofile\/(\d+)\/trip\/id\/(\d+)$/,
        viewtripd: /index\?(.*)#userprofile\/(\d+)\/trip\/(\d+)\/(\d+)$/,
        messageuser: /(\??.*)#messagecompose\/(.+)\/1$/,
        ridesearch: /(\??.*)#rideboard\/(.*)$/,
        resetpassword: /resetpwd\?(.*)$/,
        settings: /settings\?(.*)$/,
        immediateUnsubscribe: /unsubscribe\?(.*)$/,
        bookingAccept: /mybookings\?(.*)#accept\/(.*)$/,
        bookingDecline: /mybookings\?(.*)#decline\/(.*)$/,
        invite: /login\?(.*)&ts=(.*)$/,
        bookings: /showbookingrequest/,
        bookingreminderRider: /bookingreminder:rider/,
        bookingreminderDriver: /bookingreminder:driver/,
        groupinvite: /group\?(.*)#p$/,
        addPhoto: /addPhoto/,
        verifyContactData: /verifycontactdata/,
        userinvite: /userinvite/,
        reachOut: /plantrip/,
        startTrip: /starttrip/,
        tools: /tools(.*)$/,
        showInvitationReward: /inviter_reward\/(\d+)\/(.*)\/(.*)$/
    },
    _navigators: {
        verifyemail: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:verify.email", {
                url: matches[0]
            });
        },
        tolltag: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                action: "tolltag"
            });
        },
        myprofile: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {});
        },
        mytrips: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && GlobalEvents.fireOpenMyTrips();
        },
        login: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() || Alloy.createWidget("ma.car.app.login").getView().open();
        },
        signup: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() || Alloy.createWidget("ma.car.app.signup").getView().open();
        },
        viewprofile: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                userId: matches[2]
            });
        },
        viewprofile_s_det: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                userId: matches[1]
            });
        },
        viewtrip_s_det: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                userId: matches[1]
            });
        },
        viewtrip: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                userId: matches[3],
                tripId: matches[4]
            });
        },
        viewtripd: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                userId: matches[2],
                tripId: matches[4],
                timestamp: parseInt(matches[3], 10)
            });
        },
        messageuser: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:open.conversation", {
                threadId: matches[2]
            });
        },
        ridesearch: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Alloy.createController("shell", {
                path: matches[0]
            }).getView().open();
        },
        resetpassword: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:logout");
            Ti.App.fireEvent("carma:resetpassword", {
                path: matches[1]
            });
        },
        settings: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Alloy.createController("shell", {
                path: matches[0]
            }).getView().open();
        },
        immediateUnsubscribe: function(matches) {
            Alloy.createController("shell", {
                path: matches[0]
            }).getView().open();
        },
        bookingAccept: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:open.conversation");
        },
        bookingDecline: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:open.conversation");
        },
        bookings: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:open.conversation");
        },
        bookingreminderRider: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:starttrip.tab");
        },
        bookingreminderDriver: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:starttrip.tab");
        },
        invite: function(matches) {
            Alloy.createWidget("ma.car.app.login").getView().open();
        },
        groupinvite: function(matches) {},
        addPhoto: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                action: "addphoto"
            });
        },
        verifyContactData: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                action: "verifyContactData"
            });
        },
        userinvite: function(matches) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() && Ti.App.fireEvent("carma:profile.open", {
                action: "invite"
            });
        },
        reachOut: function(matches) {
            Ti.App.fireEvent("carma:plantrip.tab");
        },
        startTrip: function(matches) {
            Ti.App.fireEvent("carma:starttrip.tab", {});
        },
        showInvitationReward: function(matches) {
            var _window = Alloy.createController("simpleWindow").getView();
            _window.backgroundColor = "#fff";
            var _widget = Alloy.createWidget("ma.car.app.invites", "inviteeRewardShare", _.extend({
                window: _window
            }, {
                invitee: matches[1],
                currency: matches[2],
                reward: matches[3]
            }));
            _window.add(_widget.getView());
            _window.open();
        },
        tools: function(matches) {
            Carma.Window.Tools || Alloy.createWidget("ma.car.app.tools").getView().open();
        }
    },
    _getParamsFromUrl: function(urlString) {
        var path = "";
        if (urlString) {
            path = urlString;
            -1 != urlString.indexOf("://") && (path = urlString.split("://")[1]);
        }
        var matches = null, index = 0, mappingKeys = Object.keys(this._mappings), regex = this._mappings[mappingKeys[0]];
        do regex = this._mappings[mappingKeys[index]]; while (!(matches = path.match(regex)) && ++index < mappingKeys.length);
        if (!matches) return null;
        LOGGER.info("[DeepLinker] navigator name: " + mappingKeys[index] + ", matches: " + JSON.stringify(matches));
        return {
            key: mappingKeys[index],
            matches: matches
        };
    },
    link: function(urlString) {
        if (!urlString) return false;
        LOGGER.info("[DeepLinker] load url: " + urlString);
        var params = this._getParamsFromUrl(urlString);
        if (params) {
            this._navigators[params.key].call(this, params.matches);
            return true;
        }
        return false;
    }
};

module.exports = {
    handleUrl: function(urlString) {
        return linker.link(urlString);
    },
    canHandleUrl: function(urlString) {
        var params = linker._getParamsFromUrl(urlString);
        return !!params;
    }
};