"use strict";

var ENV_PROPERTY = "ENV_PROFILE";

var initialised = false;

var PROFILES = {
    production: {
        id: "production",
        name: "Production",
        webURL: "https://carmacarpool.com",
        apiURL: "https://api.carmacarpool.com",
        secret: Ti.App.Properties.getString("carma.client.secret.prod")
    },
    passive: {
        id: "passive",
        name: "Passive",
        webURL: "https://p.carmacarpool.com",
        apiURL: "https://api-p.carmacarpool.com",
        secret: Ti.App.Properties.getString("carma.client.secret.prod")
    },
    development: {
        id: "development",
        name: "Development",
        webURL: "https://rtr-dev.car.ma",
        apiURL: "https://api-dev.car.ma",
        secret: Ti.App.Properties.getString("carma.client.secret.dev")
    },
    ci: {
        id: "ci",
        name: "CI",
        webURL: "https://rtr-ci.car.ma",
        apiURL: "https://api-ci.car.ma",
        secret: Ti.App.Properties.getString("carma.client.secret.ci")
    },
    qa: {
        id: "qa",
        name: "QA",
        webURL: "https://rtr-qa.car.ma",
        apiURL: "https://api-qa.car.ma",
        secret: Ti.App.Properties.getString("carma.client.secret.ci")
    }
};

module.exports = {
    init: function() {
        if (!initialised) {
            var defaultProfile = PROFILES.production;
            var defaultId = Alloy.CFG.envswitch.defaultId;
            if (INSTRUMENT.profile) {
                "dev" === INSTRUMENT.profile && (INSTRUMENT.profile = "development");
                ("development" === INSTRUMENT.profile || "ci" === INSTRUMENT.profile || "qa" === INSTRUMENT.profile) && (defaultId = INSTRUMENT.profile);
            }
            var currentProfile = Ti.App.Properties.getObject(ENV_PROPERTY);
            if (currentProfile) {
                var baseProfile = PROFILES[currentProfile.id];
                defaultProfile = _.isEqual(baseProfile, currentProfile) ? currentProfile : baseProfile;
            } else defaultId && (defaultProfile = PROFILES[defaultId] ? PROFILES[defaultId] : defaultProfile);
            this.update(defaultProfile, true);
            initialised = true;
        }
    },
    update: function(profile, force) {
        LOGGER.info("[Server Switch] Setting environment to '" + profile.name + "'");
        if (force || !this.getProfile() || profile.id !== this.getProfile().id) {
            Ti.App.Properties.setObject(ENV_PROPERTY, profile);
            force || Ti.App.Properties.setBool("carma.server.switch", true);
            this._equals(this.getProfile(), profile) || Ti.App.fireEvent("carma:env.changed", profile);
        }
    },
    _equals: function(profile1, profile2) {
        if (profile1 === profile2) return true;
        if (null === profile1 || null === profile2) return false;
        return profile1.id === profile2.id;
    },
    getProfile: function() {
        return Ti.App.Properties.getObject(ENV_PROPERTY);
    },
    getProfiles: function() {
        return PROFILES;
    },
    getApiURL: function() {
        var profile = this.getProfile();
        return profile.apiURL;
    },
    getAPI1: function() {
        var profile = this.getProfile();
        return profile.apiURL + "/api/object/users/";
    },
    getAPI2: function() {
        var profile = this.getProfile();
        return profile.apiURL + "/api/rtr/v1.0/object/users/";
    },
    getWebURL: function() {
        var profile = this.getProfile();
        return profile.webURL;
    },
    getInviteURL: function() {
        return "http://i.car.ma/mb.c.d?inviteCode=";
    },
    getWebHybridURL: function() {
        var profile = this.getProfile();
        return profile.webURL + "/hybrid-web";
    },
    getNativeWrapperUrl: function() {
        var profile = this.getProfile();
        return profile.webURL + "/hybrid-web/nativeWrapper";
    },
    getOAuth: function() {
        var profile = this.getProfile(), profileId = profile.id;
        var oauth = Alloy.CFG.oauth[profile.id] || Alloy.CFG.oauth[profileId];
        oauth = _.extend(oauth, {
            secret: profile.secret,
            clientSecret: profile.secret
        });
        return oauth;
    },
    isProduction: function() {
        var profile = this.getProfile();
        if ("production" === profile.id) return true;
        return false;
    },
    getCDNUrl: function() {
        return "https://developer.avego.com/libs/rtr-mobile/1.99.0/";
    }
};