"use strict";

var LifeCycle = function() {
    var deepLinker = require("/bootstrap/deepLinker"), datePaused = null, _isInitialized = false;
    var _launch = function() {
        LOGGER.info("[LifeCycle] - app launch");
        TM.push(function() {
            Ti.App.fireEvent("carma:life.cycle.launch");
        }, process.backgroundDomain);
    };
    var _pause = function() {
        LOGGER.info("[LifeCycle] - app pause");
        datePaused = Date.now();
        Ti.App.fireEvent("carma:life.cycle.pause");
    };
    var _resume = function() {
        LOGGER.info("[LifeCycle] - app resumed");
        TM.push(function() {
            Ti.App.fireEvent("carma:life.cycle.resume");
        }, process.backgroundDomain);
        var dateResumed = Date.now();
        if (null === datePaused || dateResumed - datePaused > 3e4) {
            TM.push(function() {
                Ti.App.fireEvent("carma:life.cycle.refresh");
            }, process.backgroundDomain);
            TM.push(function() {
                Ti.App.fireEvent("carma:booking.changed");
            }, process.backgroundDomain);
        }
    };
    var _initListeners = function() {
        try {
            process.backgroundDomain.enter();
            var eventManagerModule = require("/ui/common/eventManager");
            eventManagerModule.setEventDomain("carma:life.cycle.launch", process.backgroundDomain);
            eventManagerModule.setEventDomain("carma:life.cycle.resume", process.backgroundDomain);
            eventManagerModule.setEventDomain("carma:life.cycle.pause", process.backgroundDomain);
            eventManagerModule.setEventDomain("carma:booking.changed", process.backgroundDomain);
            Ti.App.addAppEventListener("paused", _pause);
            Ti.App.addAppEventListener("resumed", _resume);
            Ti.App.once("destroy", function() {
                LOGGER.info("[LifeCycle] - app destroy");
                Ti.App.removeAppEventListener("resumed", _resume);
                Ti.App.removeAppEventListener("paused", _pause);
            });
        } finally {
            process.backgroundDomain.exit();
        }
    };
    return {
        start: function() {
            if (_isInitialized) return;
            _isInitialized = true;
            _initListeners();
            _launch();
        },
        pause: _pause,
        resume: _resume
    };
};

var enableTracking = function(lifeCycle) {
    var aop = require("/vendor/meld"), trackingEvent = require("/tracking/tracking_event"), analytics = require("/tracking/tracking_manager");
    aop.after(lifeCycle, "start", function() {
        analytics.track(trackingEvent.APP_LAUNCH, _getRuntimeInfo());
    });
    aop.after(lifeCycle, "pause", function() {
        analytics.track(trackingEvent.APP_PAUSE);
    });
    aop.after(lifeCycle, "resume", function() {
        analytics.track(trackingEvent.APP_RESUME);
    });
    var _getPlatformInfo = function(name) {
        try {
            return Ti.Platform[name];
        } catch (e) {
            return "";
        }
    };
    var _getRuntimeInfo = function() {
        return {
            jenkins_build_number: Alloy.Globals.APP.REVISION,
            bundle_timestamp: Alloy.Globals.APP.TIMESTAMP,
            bundle_id: Alloy.Globals.APP.BUNDLEID,
            app_version: Ti.App.Properties.getString("carma.revision"),
            carrier: "unknown",
            "device manufacturer": _getPlatformInfo("manufacturer"),
            "device model": _getPlatformInfo("model"),
            os: _getPlatformInfo("name") + "/" + _getPlatformInfo("osname"),
            "os version": _getPlatformInfo("version"),
            mac_address: _getPlatformInfo("macaddress"),
            num_processor: _getPlatformInfo("processorCount"),
            system_user_name: _getPlatformInfo("username"),
            wifi_ip_address: _getPlatformInfo("address"),
            os_type: _getPlatformInfo("ostype"),
            architecture: _getPlatformInfo("architecture"),
            available_memory: _getPlatformInfo("availableMemory"),
            resolution: Ti.Platform.displayCaps.platformWidth + "x" + Ti.Platform.displayCaps.platformHeight,
            display_density: Ti.Platform.displayCaps.density,
            display_dpi: Ti.Platform.displayCaps.dpi,
            locale: _getPlatformInfo("locale")
        };
    };
};

var lifeCycle = new LifeCycle();

enableTracking(lifeCycle);

module.exports = lifeCycle;