"use strict";

var pickUserPhotoError = function(error) {
    502 === error.status ? Carma.UI.createAlertWindow({
        title: L("error_title"),
        message: L("photo_module_upload_timeout")
    }).show() : Carma.UI.createAlertWindow({
        title: L("error_title"),
        message: L("photo_module_upload_error")
    }).show();
};

var handlePickUserPhoto = function(webview) {
    require("photo/photo_manager").pickPhoto().then(function(media) {
        var executor = require("utils/executor");
        var funcs = [];
        funcs.push(function() {
            return require("services/photo_service").saveUserImage(media);
        });
        funcs.push(function(data) {
            webview.native2web("avego.util.bb.globalEvents.trigger", [ "nativeapp.userphoto.uploaded", data ]);
        });
        executor(funcs).error(function(error) {
            pickUserPhotoError(error);
        }).run();
    }).catch(function(error) {});
};

var handlePickGroupPhoto = function(webview, groupId) {
    require("photo/photo_manager").pickPhoto().then(function(media) {
        var executor = require("utils/executor");
        var funcs = [];
        funcs.push(function() {
            require("services/photo_service").saveGroupImage(groupId, media);
        });
        funcs.push(function(data) {
            webview.native2web("avego.rtr.utils.NativeAppConnector.groupPhotoUploaded", []);
        });
        executor(funcs).error(function() {
            webview.native2web("avego.rtr.utils.NativeAppConnector.groupPhotoUploadFailed", []);
        }).run();
    }).catch(function() {
        webview.native2web("avego.rtr.utils.NativeAppConnector.groupPhotoUploadFailed", []);
    });
};

module.exports = {
    init: function(webview) {
        webview.addWeb2NativeHandler("/showPhotoPicker", function() {
            handlePickUserPhoto(webview);
        });
        webview.addWeb2NativeHandler("/showPicker", function() {
            handlePickUserPhoto(webview);
        });
        webview.addWeb2NativeHandler("/showGroupPhotoPicker", function(e) {
            var params = e.url.attr("path").split("/");
            var groupId = params[params.length - 1];
            handlePickGroupPhoto(webview, groupId);
        });
        webview.addWeb2NativeHandler("/showGroupPicker", function(e) {
            var params = e.url.attr("path").split("/");
            var groupId = params[2];
            handlePickGroupPhoto(webview, groupId);
        });
    }
};