"use strict";

var Q = require("vendor/q");

var assert = require("utils/assert");

var _fb = null;

var graphurl = "https://graph.facebook.com/v2.2/me/";

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var _reauthorize = function(permissions) {
    var deferred = Q.defer();
    _fb.reauthorize(permissions, "me", function(e) {
        e.success ? deferred.resolve({}) : deferred.reject(e);
    });
    return deferred.promise;
};

var _createOpenGraphObject = function(type, data) {
    var deferred = Q.defer();
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            var result = this.responseText;
            deferred.resolve(JSON.parse(result).id);
        },
        onerror: function() {
            var error = this.responseText;
            try {
                error = JSON.parse(error);
            } catch (e) {}
            deferred.reject(error);
        },
        timeout: 1e4
    });
    xhr.open("POST", graphurl + "objects/" + type, true);
    LoggingService.log("network", graphurl + "objects/" + type);
    xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");
    xhr.send({
        access_token: _fb.accessToken,
        object: JSON.stringify(data),
        "fb:explicitly_shared": true
    });
    return deferred.promise;
};

var _publishGraphObject = function(action, ogId) {
    var deferred = Q.defer();
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            var result = this.responseText;
            deferred.resolve(JSON.parse(result).id);
        },
        onerror: function() {
            var error = this.responseText;
            try {
                error = JSON.parse(error);
            } catch (e) {}
            deferred.reject(error);
        },
        timeout: 1e4
    });
    xhr.open("POST", graphurl + action, true);
    LoggingService.log("network", graphurl + action);
    xhr.setRequestHeader("Content-Type", "application/json");
    xhr.send({
        access_token: _fb.accessToken,
        schedule: ogId
    });
    return deferred.promise;
};

module.exports = {
    failedCount: 0,
    init: function(fb) {
        _fb = fb;
        fb.appid = Ti.App.Properties.getString("ti.facebook.appid", "");
        fb.permissions = [ "public_profile", "email", "user_friends", "friends_work_history" ];
        fb.forceDialogAuth = false;
        fb.initialize(1e4);
    },
    createActivityWorker: function(win) {
        return _fb.createActivityWorker({
            lifecycleContainer: win
        });
    },
    isLoggedIn: function() {
        return _fb.loggedIn;
    },
    accessToken: function() {
        return _fb.accessToken;
    },
    login: function(options) {
        assert.required(options, [ "success", "error" ]);
        if (_fb.loggedIn) {
            options.success({
                token: _fb.accessToken
            });
            return;
        }
        LOGGER.debug("Authorizing the user with FB...");
        _fb.addEventListener("login", function handler(e) {
            _fb.removeEventListener("login", handler);
            if (e.success) {
                var user = e.data;
                user.firstName = user.firstName && user["first_name"];
                user.lastName = user.lastName && user["last_name"];
                _fb.forceDialogAuth = false;
                options.success({
                    token: _fb.accessToken,
                    user: user
                });
                return;
            }
            options.error(e);
        });
        _fb.authorize();
        return;
    },
    renewCredentials: function(options) {},
    logout: function() {
        _fb.logout();
        var client = Ti.Network.createHTTPClient();
        client.clearCookies("https://login.facebook.com");
        if (_fb.api) {
            _fb.api.accessToken = null;
            _fb.api.uid = null;
        }
    },
    user: function() {
        var deferred = Q.defer();
        _fb.requestWithGraphPath("me", {}, "GET", function(e) {
            if (e.success) {
                var user;
                user = "string" == typeof e.result ? JSON.parse(e.result) : e.result;
                user.firstName = user.firstName && user["first_name"];
                deferred.resolve({
                    token: _fb.accessToken,
                    user: user
                });
            } else {
                LOGGER.error("Failed to fetch user data from FB...");
                deferred.reject(e);
            }
        });
        return deferred.promise;
    },
    publish: function(action, type, data) {
        return _createOpenGraphObject(type, data).then(function(ogId) {
            return _publishGraphObject(action, ogId);
        });
    }
};