"use strict";

var Q = require("vendor/q");

var purl = require("/utils/purl");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var _handleRedirect = function(xhr, deferred) {
    var locationHeader = xhr.getResponseHeader("Location").replace("#", "?");
    var location = purl.purl(locationHeader);
    deferred.resolve({
        token: location.param("access_token"),
        expiresIn: location.param("expires_in")
    });
};

var _authenticate = function(token) {
    var deferred = Q.defer();
    var url = _.template(require("carmalib/url").USER_ID_REQUEST_WITH_FB, {
        redirectURL: require("bootstrap/env").getWebURL() + "/rtr-mobile-web",
        scope: "rtr"
    });
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            LOGGER.info("Fetched the application token successfully...");
            if ("302" === this.status.toString()) {
                _handleRedirect(xhr, deferred);
                return;
            }
            deferred.reject({
                status: this.status,
                response: this.responseText
            });
        },
        onerror: function(e) {
            LOGGER.error("Failed to fetch the application token: " + JSON.stringify(e));
            if ("302" === this.status.toString()) {
                _handleRedirect(xhr, deferred);
                return;
            }
            var error = this.responseText;
            try {
                error = JSON.parse(error);
            } catch (e) {}
            deferred.reject({
                status: this.status.toString(),
                response: error
            });
        },
        timeout: 6e4
    });
    xhr.open("POST", url);
    LoggingService.log("network", url);
    xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    xhr.autoRedirect = false;
    xhr.send("facebookToken: " + token);
    return deferred.promise;
};

module.exports = {
    authenticate: function(fbToken) {
        require("services/resource_owner_service").singleton().destroy();
        return _authenticate(fbToken).then(function(token) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.save({
                token: token
            });
            return Alloy.createModel("user", {
                userId: "SELF"
            }).fetch().then(function(data) {
                var countries = require("utils/countries");
                var countryInfo = _.findWhere(countries.countries, {
                    cca3: data.country
                });
                countryInfo && (data.country2chars = countryInfo.cca2);
                resourceOwner.save(data);
            });
        });
    }
};