"use strict";

var env = require("bootstrap/env");

var initialised = false;

var running = false;

var intervalId;

var userId = -1;

var currentToggleData;

var currentToggles;

var initListeners = function() {
    var eventManagerModule = require("/ui/common/eventManager");
    eventManagerModule.setEventDomain("carma:feature.toggles", process.backgroundDomain);
    Ti.App.addAppEventListener("carma:life.cycle.launch", function() {
        console.log("CARMABOOT - feature toggle init listener");
        module.exports.start();
    });
    Ti.App.addAppEventListener("carma:life.cycle.resume", function() {
        module.exports.start(true);
    });
    Ti.App.addAppEventListener("carma:life.cycle.pause", function() {
        module.exports.stop();
    });
};

var processToggles = function(toggleData) {
    LOGGER.info("[FEATURETOGGLE] recieved " + toggleData);
    if (toggleData && toggleData != currentToggleData) {
        LOGGER.debug(new Date() + " Feature toggles updated");
        currentToggleData = toggleData;
        currentToggles = JSON.parse(toggleData).featureToggle || [];
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId") || -1;
        LOGGER.info("[FEATURETOGGLE] saved:" + userId + " data:" + toggleData);
        Ti.App.Properties.setObject("featuretoggles-" + userId, currentToggles);
        Ti.App.fireEvent("carma:feature.toggles", {
            data: currentToggles
        });
    }
};

var getFeatureToggles = function() {
    var resourceOwner = require("services/resource_owner_service").singleton();
    var userId = resourceOwner.get("userId") || -1;
    var token = resourceOwner.get("token") || {};
    var url = env.getAPI2() + userId + "/featureToggles";
    if (-1 === userId) {
        var deviceId = Ti.Platform.getId();
        deviceId && (url = url + "?deviceId=" + deviceId);
    }
    var client = Ti.Network.createHTTPClient({
        onload: function(e) {
            processToggles(client.responseText);
        },
        onerror: function(e) {
            LOGGER.error("[FEATURETOGGLE] " + url + " : " + JSON.stringify(e));
        },
        timeout: 3e4
    });
    client.open("GET", url);
    var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");
    LoggingService.log("network", url);
    client.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
    client.setRequestHeader("Accept", "application/json");
    token.token && client.setRequestHeader("Authorization", "Bearer " + token.token);
    client.send();
};

module.exports = {
    init: function() {
        if (!initialised) {
            initialised = true;
            initListeners();
        }
    },
    start: function(dontGetNow) {
        if (!running) {
            running = true;
            dontGetNow || getFeatureToggles();
            intervalId = setInterval(getFeatureToggles, 6e5);
        }
    },
    stop: function() {
        if (running) {
            clearInterval(intervalId);
            running = false;
        }
    },
    ensureStarted: function() {
        if (!initialised) {
            initialised = true;
            initListeners();
        }
        if (!running) {
            running = true;
            intervalId = setInterval(getFeatureToggles, 6e5);
        }
        getFeatureToggles();
    },
    featureToggles: function() {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId") || -1;
        var data = Ti.App.Properties.getObject("featuretoggles-" + userId);
        data || (data = Ti.App.Properties.getObject("featuretoggles--1", {}));
        return data;
    },
    getValue: function(feature) {
        var toggles = this.featureToggles();
        var match = null;
        _.each(toggles, function(toggle) {
            toggle.featureName === feature && (match = toggle.value);
        });
        return match;
    },
    setUserId: function(uid) {
        userId = uid;
    }
};