"use strict";

var utils = require("utils/utils");

module.exports = {
    formatTime: function(d) {
        var preferences = require("services/cached_preferences_service").singleton();
        return d.format("MILITARY" === preferences.getPreferredTimeFormat() ? "H:mm" : "h:mma");
    },
    formatDate: function(d) {
        return d.format("ll");
    },
    formatLongDate: function(d) {
        return d.format("LL");
    },
    formatVeryLongDate: function(d) {
        return d.format("ddd Do MMM YYYY");
    },
    formatDateTime: function(d) {
        var preferences = require("services/cached_preferences_service").singleton();
        return d.format("MILITARY" === preferences.getPreferredTimeFormat() ? "MMM D, H:mm" : "MMM D, h:mma");
    },
    formatTodayDate: function(timestamp) {
        var moment = require("/vendor/moment"), today = moment().startOf("day"), futureDate = moment(timestamp).startOf("day");
        return futureDate.isSame(today, "day") ? L("date_today") : 1 === futureDate.diff(today, "days") ? L("date_tomorrow") : futureDate.format("dddd Do MMM");
    },
    formatDistanceFromMiles: function(distanceInMiles) {
        distanceInMiles || (distanceInMiles = 0);
        var preferences = require("services/cached_preferences_service").singleton();
        var distance = "MILES" === preferences.getPreferredDistanceUnit() ? distanceInMiles : utils.convertMilesToMeters(distanceInMiles) / 1e3;
        return distance.toFixed(distance >= 1e3 ? 0 : 1);
    },
    formatDistanceFromMeters: function(distanceInMeters) {
        distanceInMeters || (distanceInMeters = 0);
        var preferences = require("services/cached_preferences_service").singleton();
        var distance = "MILES" !== preferences.getPreferredDistanceUnit() ? distanceInMeters / 1e3 : utils.convertMetersToMiles(distanceInMeters);
        return distance.toFixed(distance >= 1e3 ? 0 : 1);
    },
    getDistanceUnitDisplayString: function() {
        var preferences = require("services/cached_preferences_service").singleton();
        return L("MILES" === preferences.getPreferredDistanceUnit() ? "widget_rating_miles_label" : "widget_rating_km_label");
    }
};