"use strict";

function canLog(target, level) {
    var targetIndex = LOG_METHODS.indexOf(target);
    var levelIndex = LOG_METHODS.indexOf(level);
    return levelIndex >= targetIndex;
}

function consoleLog(level, args) {
    if (args) {
        var messages = Array.prototype.slice.call(args);
        messages.forEach(function(message) {
            Ti.API.log(level, "LOGGER - " + moment(Date.now()).format("HH:mm:ss.SSS") + " - " + message);
        });
        return;
    }
}

function remoteLog(level, args) {
    consoleLog(level, args);
}

function persist(level, args) {
    var logLevel = Alloy.Globals.LOG_LEVEL;
    if (logLevel && args && canLog(logLevel, level)) {
        var messages = Array.prototype.slice.call(args);
        messages.forEach(function(message) {
            require("services/local_log_service").save({
                level: level,
                message: message
            });
        });
    }
}

var LOG_METHODS = [ "trace", "debug", "info", "warn", "error" ], self = {}, moment = require("alloy/moment");

LOG_METHODS.forEach(function(method) {
    self[method] = function() {
        remoteLog(method, arguments);
        persist(method, arguments);
    };
});

module.exports = self;