"use strict";

var Utils = require("utils/utils"), _ = require("/vendor/underscore"), Q = require("/vendor/q"), Avatar = require("/ui/common/Avatar");

var BoardingDisplayHelper = function(profileService) {
    if (!profileService) throw new Error("ProfileService is required!");
    var screenHeightPixels = Ti.Platform.displayCaps.platformHeight, screenHeightDP = Utils.defaultToDp(screenHeightPixels);
    var badgeMetrics = 540 > screenHeightDP ? {
        badgeWidthPixels: Utils.defaultToDp(130),
        containerHeightDP: "150dp",
        badgeContainerWidthDP: "130dp",
        badgeContainerHeightDP: "130dp",
        badgeImageWidthDP: "110dp",
        badgeImageHeightDP: "110dp",
        badgeImageBorderRadius: Alloy.Globals.Style.Unit.unit55
    } : {
        badgeWidthPixels: Utils.defaultToDp(200),
        containerHeightDP: "210dp",
        badgeContainerWidthDP: "200dp",
        badgeContainerHeightDP: "170dp",
        badgeImageWidthDP: "170dp",
        badgeImageHeightDP: "170dp",
        badgeImageBorderRadius: Alloy.Globals.Style.Unit.unit85
    };
    return {
        getBadgeMetrics: function() {
            return _.clone(badgeMetrics);
        },
        getUserAvatarImage: function(userId) {
            var badgeSizeDefault = Utils.dpToDefault(badgeMetrics.badgeImageWidthDP.replace("dp", ""));
            var profile = profileService.getCachedProfile(userId);
            if (profile) return Avatar.getUserPhoto(profile, false, {
                size: badgeSizeDefault
            });
            var dfd = Q.defer();
            Q().then(function() {
                return profileService.getProfile(userId, profileService.BASIC_DETAIL);
            }).then(function(user) {
                return Avatar.getUserPhoto(user, false, {
                    size: badgeSizeDefault
                });
            }).then(function(image) {
                dfd.resolve(image);
            }).catch(function(e) {
                dfd.reject(e);
            }).done();
            return dfd.promise;
        },
        getCachedUserAvatarImage: function(userId) {
            var badgeSizeDefault = Utils.dpToDefault(badgeMetrics.badgeImageWidthDP.replace("dp", ""));
            var profile = profileService.getCachedProfile(userId);
            if (profile) return Avatar.getLocalPhoto(profile, {
                size: badgeSizeDefault
            });
            return false;
        }
    };
};

module.exports = BoardingDisplayHelper;