"use strict";

var Q = require("/vendor/q"), _ = require("/vendor/underscore");

var SummaryWidgetHelper = function(summaryService, boardingBgService, logger) {
    if (!summaryService) throw new Error("SummaryService is required!");
    if (!boardingBgService) throw new Error("BoardingBgService is required!");
    if (!logger) throw new Error("SummaryService is required!");
    var initSummaryWidget = function() {
        var summaryWidget = Alloy.createWidget("ma.car.app.boardingv6", "summary");
        summaryWidget.setLoading();
        summaryWidget.getView().open();
        return summaryWidget;
    };
    var showSummary = function(summaryId, summaryWidget, isInTransit) {
        var dfd = Q.defer();
        logger.log("UI showSummary: " + summaryId);
        summaryWidget || (summaryWidget = initSummaryWidget());
        Q().then(function() {
            return summaryService.getSummary(summaryId);
        }).then(function(summary) {
            summary.isInTransit = isInTransit;
            summaryWidget.setSummary(summary, function() {
                try {
                    (!summary.isPendingData || summary.isError) && summaryService.deleteSummary(summaryId);
                    dfd.resolve();
                } catch (e) {
                    dfd.reject(e);
                }
            });
        });
        return dfd.promise;
    };
    var self = {
        initSummaryWidget: function() {
            return initSummaryWidget();
        },
        showSummaries: function(widget) {
            var summaries = summaryService.getSummariesReferences(), chain = Q(), lastIndex = summaries.length - 1;
            if (0 === summaries.length) {
                widget && widget.getView().close();
                return Q.reject();
            }
            widget = widget ? widget : initSummaryWidget();
            _.each(summaries, function(summary, index) {
                chain = chain.finally(function() {
                    var isInTransit = index !== lastIndex;
                    widget.resetSummary();
                    return showSummary(summary, widget, isInTransit);
                });
            });
            return chain;
        },
        showDriveSummary: function(driveId, widget) {
            widget = widget ? widget : initSummaryWidget();
            return Q().then(function(e) {
                return boardingBgService.sync();
            }).finally(function() {
                var summaryId = summaryService.getDriveSummaryReference(driveId), isInTransit = true;
                return showSummary(summaryId, widget, isInTransit);
            });
        }
    };
    return self;
};

module.exports = SummaryWidgetHelper;