"use strict";

var Q = require("/vendor/q"), animation = require("/alloy/animation"), utils = require("utils/utils"), SCREEN_HEIGHT = utils.defaultToDp(Ti.Platform.displayCaps.platformHeight), IS_SMALL_SCREEN = 535 > SCREEN_HEIGHT, _INIT_FONT_STYLE = {
    font: {
        fontSize: IS_SMALL_SCREEN ? "180dp" : "200dp",
        fontWeight: "bold"
    },
    color: "#fff",
    opacity: 0,
    transform: Ti.UI.create3DMatrix({
        scale: 1.3
    })
};

var deferred = Q.defer();

var stopped = true;

var _startAnimation = function(deferred, timeDisplay, time) {
    if (stopped) {
        timeDisplay.text = "";
        return;
    }
    if (0 === time) {
        deferred.resolve();
        return;
    }
    timeDisplay.text = time;
    var fadeIn = Ti.UI.createAnimation({
        opacity: 1,
        transform: Ti.UI.create3DMatrix({
            scale: 1
        }),
        duration: 500
    });
    var fadeOut = Ti.UI.createAnimation({
        opacity: 0,
        transform: Ti.UI.create3DMatrix({
            scale: .5
        }),
        delay: 500,
        duration: 500
    });
    animation.chainAnimate(timeDisplay, [ fadeIn, fadeOut ], function() {
        timeDisplay.applyProperties(_INIT_FONT_STYLE);
        _startAnimation(deferred, timeDisplay, --time);
    });
};

module.exports = {
    start: function($container) {
        deferred.reject();
        deferred = Q.defer();
        stopped = false;
        var timeDisplay = Ti.UI.createLabel(_INIT_FONT_STYLE);
        timeDisplay.addEventListener("postlayout", function handler() {
            timeDisplay.removeEventListener("postlayout", handler);
            _startAnimation(deferred, timeDisplay, 3);
        });
        $container.removeAllChildren();
        $container.add(timeDisplay);
        return deferred.promise;
    },
    completeNow: function() {
        stopped = true;
        deferred.resolve();
    },
    stop: function() {
        stopped = true;
        deferred.reject();
    }
};