"use strict";

var Q = require("/vendor/q"), errorCodes = require("services/error_codes");

var Location = {
    getContactLocation: function(trips, isBoarding) {
        trips = trips || [];
        trips instanceof Array || (trips = [ trips ]);
        if (!trips.length) return Q(null);
        var gps = require("gps/gps_manager"), location = gps.lastLocation(), locationValidator = require("/validator/validator_factory").currentLocation(), isValid = locationValidator.isLocationValidForPickupDropoff(location, true);
        if (isValid) return Q(location);
        var dfd = Q.defer();
        if (Ti.Network.online) {
            var ctrl = Alloy.createWidget("ma.car.app.boardingv6", "location", {
                trip: trips,
                type: isBoarding ? errorCodes.ERROR_SOURCE_BOARDING : errorCodes.ERROR_SOURCE_DROPOFF,
                darkTheme: true
            });
            var view = ctrl.getView();
            location = null;
            view.addEventListener("close", function(data) {
                var tripSample = trips[0];
                if (location) {
                    require("/gps/gps_service").injectLocation(location.latitude || location.lat, location.longitude || location.lon, location.accuracy, tripSample.get(isBoarding ? "startTimestamp" : "endTimestamp"));
                    dfd.resolve(location);
                } else dfd.reject(new Error(errorCodes.ERROR_INVALID_LOCATION));
            });
            view.addEventListener("location", function(data) {
                var tripSample = trips[0];
                location = data.location;
                if (!location) return;
                location.accuracy = 0;
                location.timestamp = tripSample.get("startTimestamp");
            });
            view.open();
        } else {
            var dialog = Ti.UI.createAlertDialog({
                cancel: 1,
                buttonNames: [ L("btn_yes"), L("btn_no") ],
                title: L("widget_boarding_no_gps_internet_title"),
                message: L(isBoarding ? "widget_boarding_no_gps_internet_desc" : "widget_boarding_end_no_gps_internet_desc")
            });
            dialog.addEventListener("click", function(e) {
                0 === e.index ? dfd.resolve(null) : dfd.reject(new Error(errorCodes.ERROR_INVALID_LOCATION));
            });
            dialog.show();
        }
        return dfd.promise;
    }
};

module.exports = Location;