"use strict";

var Q = require("/vendor/q"), utils = require("utils/utils"), SCREEN_HEIGHT = utils.defaultToDp(Ti.Platform.displayCaps.platformHeight), IS_SMALL_SCREEN = 535 > SCREEN_HEIGHT, _INIT_FONT_STYLE = {
    font: {
        fontSize: IS_SMALL_SCREEN ? "180dp" : "200dp",
        fontWeight: "bold"
    },
    color: "#fff",
    verticalAlign: Ti.UI.TEXT_VERTICAL_ALIGNMENT_TOP
};

var deferred = Q.defer();

var _animationTimeout = null;

var stopped = true;

var _startAnimation = function(deferred, timeDisplay, time) {
    if (stopped) {
        timeDisplay.text = "";
        return;
    }
    if (0 === time) {
        timeDisplay.text = "";
        deferred.resolve();
        return;
    }
    timeDisplay.text = time;
    _animationTimeout = setTimeout(function() {
        _startAnimation(deferred, timeDisplay, --time);
    }, 1e3);
};

module.exports = {
    start: function($container) {
        deferred.reject();
        deferred = Q.defer();
        stopped = false;
        var timeDisplay = Ti.UI.createLabel(_INIT_FONT_STYLE);
        timeDisplay.addEventListener("postlayout", function handler() {
            timeDisplay.removeEventListener("postlayout", handler);
            _startAnimation(deferred, timeDisplay, 3);
        });
        $container.removeAllChildren();
        $container.add(timeDisplay);
        return deferred.promise;
    },
    completeNow: function() {
        stopped = true;
        clearInterval(_animationTimeout);
        deferred.resolve();
    },
    stop: function() {
        stopped = true;
        clearInterval(_animationTimeout);
        deferred.reject();
    }
};