"use strict";

var Q = require("/vendor/q"), STATES = {
    INVALID: 0,
    IN_TRANSITION: 1,
    DECISION: 2,
    CARMA_CREW: 3,
    RIDE_TIMER: 4,
    RIDING: 5,
    DRIVE_TIMER: 6,
    DRIVING_SOLO: 7,
    DRIVING: 8
}, TRANSITIONS = {
    OPEN_CREW: 101,
    START_RIDE_TIMER: 102,
    TIMER_COMPLETE: 103,
    END_TRIP: 104,
    START_DRIVE_TIMER: 105,
    EXTERNAL_JOIN: 106,
    EXTERNAL_PICKUP: 107,
    BACK: 108,
    ANDROID_BACK: 109
};

var sm = function(widgets) {
    var _currentState = STATES.DECISION, boardingWidget = null;
    var setState = function(newState) {
        _currentState = newState;
        LOGGER.info("################# NEW BOARDING STATE: " + _currentState);
    };
    var initTransition = function() {
        setState(STATES.IN_TRANSITION);
        return Q();
    };
    var changeState = function(newState, transition) {
        if ("undefined" == typeof transition) {
            LOGGER.error("Transition method is undefined!");
            throw new Error("Transition method is undefined!");
        }
        if (transition) return initTransition().then(transition).then(function() {
            setState(newState);
        }).catch(function(e) {
            LOGGER.error("Caught exception in transaction: " + JSON.stringify(e));
            var errorHandler = require("/utils/errorHandler");
            errorHandler.noticeError({
                action: "boarding_transition",
                errorCode: e.message
            });
            _currentState = STATES.DECISION;
            boardingWidget.resetAllState();
        });
        setState(newState);
        return Q();
    };
    boardingWidget = widgets.boardingWidget;
    var stateMachine = {
        _STATES: STATES,
        _TRANSITIONS: TRANSITIONS,
        getCurrentState: function() {
            return _currentState;
        },
        isInTransition: function() {
            return _currentState === STATES.IN_TRANSITION;
        },
        resetAllState: function() {
            _currentState = this._STATES.DECISION;
            LOGGER.info("################# RESETTING SM STATE - CURRENT BOARDING STATE: " + _currentState);
            boardingWidget.resetAllState();
        },
        transition: function(transition) {
            LOGGER.info("################# RECEIVED TRANSITION: " + transition + " - CURRENT BOARDING STATE: " + _currentState);
            switch (_currentState) {
              case this._STATES.DECISION:
                if (transition === TRANSITIONS.OPEN_CREW) return changeState(STATES.CARMA_CREW, boardingWidget.transitionOpenCarmaCrew);
                if (transition === TRANSITIONS.START_DRIVE_TIMER) return changeState(STATES.DRIVE_TIMER, boardingWidget.transitionOpenDriveTimer);
                if (transition === TRANSITIONS.EXTERNAL_JOIN) return changeState(STATES.RIDING, boardingWidget.transitionOnExternalJoinCar);
                if (transition === TRANSITIONS.EXTERNAL_PICKUP) return changeState(STATES.RIDING, boardingWidget.transitionOnExternalPickup);
                if (transition === TRANSITIONS.ANDROID_BACK) {
                    Ti.App.fireEvent("carma:home.tab");
                    return null;
                }
                break;

              case this._STATES.CARMA_CREW:
                if (transition === TRANSITIONS.START_RIDE_TIMER) return changeState(STATES.RIDE_TIMER, boardingWidget.transitionOnDriverSelected);
                if (transition === TRANSITIONS.BACK || transition === TRANSITIONS.ANDROID_BACK) return changeState(STATES.DECISION, boardingWidget.transitionDisplayDecision);
                break;

              case this._STATES.RIDE_TIMER:
                if (transition === TRANSITIONS.TIMER_COMPLETE) return changeState(STATES.RIDING, boardingWidget.transitionStartRiding);
                if (transition === TRANSITIONS.BACK || transition === TRANSITIONS.ANDROID_BACK) return changeState(STATES.CARMA_CREW, boardingWidget.transitionOnJoinCarCancelled);
                break;

              case this._STATES.RIDING:
                if (transition === TRANSITIONS.END_TRIP) return changeState(STATES.DECISION, boardingWidget.transitionDisplayDecision);
                break;

              case this._STATES.DRIVE_TIMER:
                if (transition === TRANSITIONS.TIMER_COMPLETE) return changeState(STATES.DRIVING_SOLO, boardingWidget.transitionStartDriving);
                if (transition === TRANSITIONS.BACK || transition === TRANSITIONS.ANDROID_BACK) return changeState(STATES.DECISION, boardingWidget.transitionDisplayDecision);
                break;

              case this._STATES.DRIVING_SOLO:
                if (transition === TRANSITIONS.END_TRIP) return changeState(STATES.DECISION, boardingWidget.transitionDisplayDecision);
                if (transition === TRANSITIONS.EXTERNAL_JOIN) return changeState(STATES.DRIVING);
                break;

              case this._STATES.DRIVING:
                if (transition === TRANSITIONS.END_TRIP) return changeState(STATES.DECISION, boardingWidget.transitionDisplayDecision);
            }
            LOGGER.info("################# UNHANDLED TRANSITION: " + transition + " - CURRENT BOARDING STATE: " + _currentState);
            return null;
        }
    };
    return stateMachine;
};

module.exports = sm;