"use strict";

var PickerController = function(container, data, options) {
    var picker = null, selected = null, selectedIndex = null, events = [], items = data, self = this;
    var _init = function() {
        options = options || {};
        selectedIndex = options.selectedIndex || 0;
        _sanityCheck();
        _initPickerData();
    };
    var _sanityCheck = function() {
        if (!data) throw "PickerController - Please add some data for the pickerContainer [{title: String, value: X}]";
        if (!container) throw "PickerController - Please add a container in which attach the picker";
    };
    var _initPickerData = function() {
        selected = data[selectedIndex].value;
        var rows = [];
        for (var i = 0; i < data.length; ++i) rows.push(data[i].title);
        picker = Ti.UI.createOptionDialog({
            options: rows,
            selectedIndex: selectedIndex
        });
        picker.addEventListener("click", _handleChangeEvent);
    };
    var _handleChangeEvent = function(e) {
        if (e.index < 0) return;
        selected = items[e.index].value;
        "change" in events && events["change"]({
            value: selected
        });
    };
    this.show = function() {
        picker.show();
    };
    this.addEventListener = function(name, callback) {
        if (!name || !callback) {
            LOGGER.debug("PickerController - addEventListener name and callback must be provided");
            return;
        }
        events[name] = callback;
    };
    this.getValue = function() {
        return selected;
    };
    _init();
};

module.exports = PickerController;