"use strict";

var PickerController = function(container, data, options) {
    var toolbar = null, picker = null, popupView = null, wrapper = null, doneBtn = null, selected = null, selectedIndex = null, events = [], self = this;
    var _init = function() {
        _.bindAll(self, "hide");
        options = options || {};
        selectedIndex = options.selectedIndex || 0;
        _sanityCheck();
        _initToolbar();
        _initPickerData();
        _initPopup();
    };
    var _sanityCheck = function() {
        if (!data) throw "PickerController - Please add some data for the picker [{title: String, value: X}]";
        if (!container) throw "PickerController - Please add a container in which attach the picker";
    };
    var _initToolbar = function() {
        doneBtn = Ti.UI.createButton({
            title: L("btn_done"),
            style: Ti.UI.iPhone.SystemButtonStyle.PLAIN
        });
        var flexSpace = Ti.UI.createButton({
            systemButton: Ti.UI.iPhone.SystemButton.FLEXIBLE_SPACE
        });
        toolbar = Ti.UI.iOS.createToolbar({
            items: [ flexSpace, doneBtn ],
            top: 0,
            borderTop: true,
            borderBottom: true
        });
        doneBtn.addEventListener("click", self.hide);
    };
    var _initPickerData = function() {
        selected = data[selectedIndex].value;
        picker = Ti.UI.createPicker({
            top: 43
        });
        var rows = [];
        for (var i = 0; i < data.length; ++i) {
            var row = Ti.UI.createPickerRow({
                id: data[i].value,
                horizontalWrap: false
            });
            row.add(Ti.UI.createLabel({
                text: data[i].title,
                textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
                font: {
                    fontSize: "18dp"
                },
                width: "100%",
                horizontalWrap: false
            }));
            rows.push(row);
        }
        picker.add(rows);
        picker.selectionIndicator = true;
        picker.addEventListener("change", _handleChangeEvent);
    };
    var _initPopup = function() {
        wrapper = Ti.UI.createView({
            width: "100%",
            height: "100%",
            top: 0,
            left: 0
        });
        wrapper.addEventListener("click", self.hide);
        popupView = Ti.UI.createView({
            width: "100%",
            height: 251,
            bottom: -251
        });
        popupView.add(toolbar);
        popupView.add(picker);
        wrapper.add(popupView);
        container.add(wrapper);
        picker.setSelectedRow(0, selectedIndex, false);
    };
    var _handleChangeEvent = function(e) {
        selected = e.row.id;
        "change" in events && events["change"]({
            value: selected
        });
    };
    this.show = function() {
        if (PickerController.open) return;
        PickerController.open = true;
        wrapper.visible = true;
        popupView.animate({
            duration: 300,
            bottom: 0
        });
    };
    this.hide = function() {
        PickerController.open = false;
        popupView.animate({
            duration: 300,
            bottom: -251
        }, function() {
            wrapper.visible = false;
        });
        this.clean();
    };
    this.clean = function() {
        doneBtn.removeEventListener("click", this.hide);
        wrapper.removeEventListener("click", this.hide);
    };
    this.addEventListener = function(name, callback) {
        if (!name || !callback) {
            LOGGER.debug("PickerController - addEventListener name and callback must be provided");
            return;
        }
        events[name] = callback;
    };
    this.getValue = function() {
        return selected;
    };
    _init();
};

PickerController.prototype.open = false;

module.exports = PickerController;