"use strict";

var PickerDateController = function(container, data, options) {
    var toolbar = null, picker = null, popupView = null, wrapper = null, doneBtn = null, selected = null, selectedIndex = null, events = [], callback = null, self = this;
    var _init = function() {
        _.bindAll(self, "hide");
        options = options || {};
        selectedIndex = options.selectedIndex || [ 0, 0 ];
        callback = options.callback || null;
        _sanityCheck();
        _initToolbar();
        _initPickerData();
        _initPopup();
    };
    var _sanityCheck = function() {
        if (!data) throw "PickerDateController - Please add some data for the picker [{title: String, value: X}]";
        if (!container) throw "PickerDateController - Please add a container in which attach the picker";
    };
    var _initToolbar = function() {
        doneBtn = Ti.UI.createButton({
            title: L("btn_done"),
            style: Ti.UI.iPhone.SystemButtonStyle.PLAIN
        });
        var flexSpace = Ti.UI.createButton({
            systemButton: Ti.UI.iPhone.SystemButton.FLEXIBLE_SPACE
        });
        toolbar = Ti.UI.iOS.createToolbar({
            items: [ flexSpace, doneBtn ],
            top: 0,
            borderTop: true,
            borderBottom: true
        });
        doneBtn.addEventListener("click", self.hide);
    };
    var _initPickerData = function() {
        picker = Ti.UI.createPicker({
            top: 43
        });
        var columns = [], row = null;
        for (var column in data) {
            var tiColumn = Ti.UI.createPickerColumn();
            for (var i = 0; i < data[column].length; ++i) {
                row = Ti.UI.createPickerRow({
                    value: data[column][i],
                    horizontalWrap: false
                });
                row.add(Ti.UI.createLabel({
                    text: data[column][i],
                    textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
                    font: {
                        fontSize: "18dp"
                    },
                    width: "100%",
                    horizontalWrap: false
                }));
                tiColumn.addRow(row);
            }
            columns.push(tiColumn);
        }
        picker.add(columns);
        picker.selectionIndicator = true;
        picker.addEventListener("change", _handleChangeEvent);
    };
    var _initPopup = function() {
        wrapper = Ti.UI.createView({
            width: "100%",
            height: "100%",
            top: 0,
            left: 0,
            zIndex: 999
        });
        wrapper.addEventListener("click", self.hide);
        popupView = Ti.UI.createView({
            width: "100%",
            height: 251,
            bottom: -251
        });
        popupView.add(toolbar);
        popupView.add(picker);
        wrapper.add(popupView);
        container.add(wrapper);
        picker.setSelectedRow(0, selectedIndex[0], false);
        picker.setSelectedRow(1, selectedIndex[1], false);
    };
    var _handleChangeEvent = function(e) {
        selected = {
            0: e.selectedValue[0].value,
            1: e.selectedValue[1].value
        };
        "change" in events && events["change"]({
            value: selected
        });
    };
    this.show = function() {
        if (PickerDateController.open) return;
        PickerDateController.open = true;
        wrapper.visible = true;
        popupView.animate({
            duration: 300,
            bottom: 0
        });
        Ti.App.fireEvent("picker.show");
    };
    this.hide = function() {
        PickerDateController.open = false;
        popupView.animate({
            duration: 300,
            bottom: -251
        }, function() {
            wrapper.visible = false;
        });
        callback && callback();
        this.clean();
        Ti.App.fireEvent("picker.hide");
    };
    this.clean = function() {
        callback = null;
        doneBtn.removeEventListener("click", this.hide);
        wrapper.removeEventListener("click", this.hide);
    };
    this.addEventListener = function(name, callback) {
        if (!name || !callback) {
            LOGGER.debug("PickerDateController - addEventListener name and callback must be provided");
            return;
        }
        events[name] = callback;
    };
    this.getValue = function() {
        return selected;
    };
    _init();
};

PickerDateController.prototype.open = false;

module.exports = PickerDateController;