"use strict";

var LoadingBar = function() {
    var loadingBar = null, isStopped = true, progress = 0, maxIncrement = 30, incrementThreshold = 90, incrementDuration = 500;
    var self = {
        getView: function() {
            return loadingBar;
        },
        start: function() {
            loadingBar.width = "1dp";
            if (isStopped) {
                calculateProgress();
                setTimeout(function() {
                    self.end();
                }, 18e4);
            }
        },
        end: function() {
            progress = 100;
        }
    };
    var calculateProgress = function() {
        isStopped = false;
        if (progress >= 100) {
            incrementProgress(Ti.Platform.displayCaps.platformWidth, hideProgressBar);
            return;
        }
        if (progress >= incrementThreshold) {
            setTimeout(calculateProgress, 500);
            return;
        }
        progress += Math.random() * maxIncrement + 1;
        progress = Math.min(progress, incrementThreshold);
        incrementProgress(progress / 100 * Ti.Platform.displayCaps.platformWidth, calculateProgress);
    };
    var incrementProgress = function(width, callback) {
        loadingBar.animate({
            width: width,
            height: "1dp",
            duration: incrementDuration
        }, callback);
    };
    var hideProgressBar = function() {
        loadingBar.animate({
            opacity: 0,
            duration: incrementDuration
        }, reset);
    };
    var reset = function() {
        progress = 0;
        loadingBar.width = 0;
        isStopped = true;
    };
    var initLoadingBar = function() {
        loadingBar = Ti.UI.createView({
            backgroundColor: Alloy.Globals.Style.Color.Orange,
            top: 0,
            left: 0,
            width: 0,
            height: "1dp",
            zIndex: 100
        });
    };
    initLoadingBar();
    return self;
};

module.exports = LoadingBar;