"use strict";

var PullToRefresh = function($) {
    var options = {
        msgPull: L("pull_to_refresh_pull", "Pull to refresh..."),
        msgRelease: L("pull_to_refresh_release", "Release to refresh..."),
        msgUpdating: L("pull_to_refresh_refreshing", "Refreshing..."),
        top: 0
    };
    var height = 50, attached = false, hidden = true, pulling = false, pulled = false, loading = false, offset = 0;
    var show = function(message) {
        if (!attached || pulled) return false;
        pulled = true;
        hidden = false;
        $.message.text = message || options.msgUpdating;
        $.icon.opacity = 0;
        $.loading.show();
        $.messageContainer.show();
        var os = require("/platform/os"), dimension = require("/platform/dimension");
        $.webviewContainer.animate({
            top: os.isIOS7Plus() ? dimension.statusBarHeightInSystemUnit() : 0
        });
        return true;
    };
    var hide = function() {
        if (!attached || !pulled) return false;
        $.loading.hide();
        $.icon.transform = Ti.UI.create2DMatrix();
        $.icon.opacity = 1;
        $.message.text = options.msgPull;
        setTimeout(function() {
            $.messageContainer.hide();
        }, 250);
        hidden = true;
        pulled = false;
        loading = false;
        return true;
    };
    var refresh = function() {
        if (!attached || loading) return false;
        loading = true;
        show();
        $.trigger("carma:widget.webview.refresh");
        return true;
    };
    var scrollListener = function(e) {
        if (pulled) return;
        offset = e.y;
        if (offset >= 0 - options.top) {
            if (!hidden) {
                $.messageContainer.hide();
                hidden = true;
            }
        } else if (hidden) {
            $.messageContainer.show();
            hidden = false;
        }
        if (pulling && !loading && offset > 0 - options.top - height) {
            pulling = false;
            var unrotate = Ti.UI.create2DMatrix();
            $.icon.animate({
                transform: unrotate,
                duration: 180
            });
            $.message.text = options.msgPull;
        } else if (!pulling && !loading && offset <= 0 - options.top - height) {
            pulling = true;
            var rotate = Ti.UI.create2DMatrix().rotate(180);
            $.icon.animate({
                transform: rotate,
                duration: 180
            });
            $.message.text = options.msgRelease;
        }
        return;
    };
    var dragEndListener = function() {
        if (!pulled && pulling && !loading && offset <= 0 - options.top - height) {
            pulling = false;
            refresh();
        }
        return;
    };
    var init = function() {
        $.webviewContainer.addEventListener("scroll", scrollListener);
        attached = true;
        pulling = false;
        pulled = false;
        loading = false;
        offset = 0;
        $.webviewContainer.addEventListener("dragend", dragEndListener);
        $.message.text = options.msgPull;
        return;
    };
    init();
    return {
        show: show,
        hide: hide,
        refresh: refresh
    };
};

module.exports = PullToRefresh;