"use strict";

!function() {
    var deviceToken = null, appKey = require("utils/utils").getApnsAppKey();
    var fireTokenUpdatedEvent = function(token) {
        Ti.App.fireEvent("carma:push.notification.token.updated", {
            params: [ token, "APNS", appKey, {
                installedAvego: true,
                appkey: appKey
            } ]
        });
    };
    var tokenSuccess = function(token) {
        deviceToken = token;
        LOGGER.info("PN: Registration successful: " + deviceToken);
        fireTokenUpdatedEvent(deviceToken);
    };
    var tokenError = function(e) {
        LOGGER.error("PN: Error during registration: " + JSON.stringify(e));
    };
    var tokenCallback = function(e) {
        LOGGER.info("PN: Received a push notificationPayload");
        Ti.App.fireEvent("carma:push.notification.open", e);
    };
    var tokenCB = function(data) {
        LOGGER.info("PN: Received a push notification token: " + JSON.stringify(data));
        "success" === data.state && tokenSuccess(data.token);
        "error" === data.state && tokenError(data.error);
        "callback" === data.state && tokenCallback(data.callback);
    };
    var registerForIOS = function() {
        if (deviceToken) {
            LOGGER.info("PN: Already got notification token: " + deviceToken);
            fireTokenUpdatedEvent(deviceToken);
            return;
        }
        Ti.App.addAppEventListener("carma:shell.push.token", tokenCB);
        if ("undefined" == typeof shadowMode) {
            LOGGER.info("PN: Pearl calling registerForPushNotifications (No Shadow mode)");
            Ti.Network.registerForPushNotifications({
                types: [ Ti.Network.NOTIFICATION_TYPE_BADGE, Ti.Network.NOTIFICATION_TYPE_ALERT, Ti.Network.NOTIFICATION_TYPE_SOUND ],
                success: function(e) {
                    var token = e.token || e.deviceToken;
                    token ? tokenSuccess(token) : tokenError({
                        error: null
                    });
                },
                error: function(e) {
                    tokenError(e.error);
                },
                callback: function(e) {
                    tokenCallback(e);
                }
            });
        } else {
            LOGGER.info("PN: Pearl asking CarmaOne to registerForPushNotifications (Shadow mode)");
            Ti.App.fireEvent("carma:shell.register.token");
        }
    };
    module.exports = {
        register: function() {
            registerForIOS();
        },
        deleteToken: function() {
            if (deviceToken) {
                Ti.App.removeAppEventListener("carma:shell.push.token", tokenCB);
                deviceToken = null;
            }
        },
        getDeviceToken: function() {
            return deviceToken;
        }
    };
}();