"use strict";

var pushType = {
    POKE_RECEIVED: 1,
    MESSAGE_RECEIVED: 2,
    USER_ONLINE: 3,
    TRIP_CREATED_MATCH: 4,
    RIDER_LEAVES_CAR: 6,
    RIDER_DROP_OFF: 7,
    RIDER_PICKED_UP: 9,
    RIDER_JOINS_CAR: 10,
    RIDE_REQUEST: 12,
    DRIVER_BOOKING_AUTO_CANCELLED: 16,
    DRIVER_BOOKING_REMINDER: 18,
    RIDE_OFFER: 19,
    BOOKING_ACCEPTED_BY_DRIVER: 20,
    BOOKING_CANCELLED_BY_DRIVER: 21,
    RIDER_BOOKING_AUTO_CANCELLED: 23,
    RIDER_BOOKING_REMINDER: 25,
    USER_FAVOURITED: 41,
    DRIVER_RATE: 44,
    RIDER_RATE: 45,
    INVITER_REWARD: 83,
    AFR_MATCH: 48,
    AFR_BOOKING_CONFIRMED: 49,
    DRIVER_RIDE_OFFER_ACCEPTED: 13,
    DRIVER_RIDE_OFFER_DECLINED: 14,
    DRIVER_RIDE_REQUEST_CANCELLED: 17,
    RIDE_BOOKING_DEVIATION_APPLIED: 46,
    DRIVER_RIDE_BOOKING_CANCELLED: 15,
    RIDER_RIDE_BOOKING_CANCELLED: 22,
    RIDER_RIDE_OFFER_CANCELLED: 24,
    TOPUP_TIMEOUT_SUCCESS: 42,
    TOPUP_TIMEOUT_FAIL: 85
};

!function() {
    var deepLinker = require("/bootstrap/deepLinker"), trackingEvent = require("/tracking/tracking_event"), globalEvents = require("/v6/common/GlobalEvents"), appSound = require("/utils/appSound"), samanthaService = require("/services/samantha_service");
    var decodeType = function(encodedType) {
        var decode = Ti.Utils.base64decode(encodedType);
        return decode.getText();
    };
    var trackPushTokenClick = function(token) {
        require("/tracking/tracking_manager").track(trackingEvent.PUSH_NOTIFICATION_OPEN, {
            token: token
        });
    };
    var handleDeeplinkUrl = function(params, isLaunch) {
        if (params && params.data && params.data.p) {
            var url = "avego://rtr-mobile-web/" + params.data.p;
            params.ignoreRedirection || (isLaunch ? Alloy.Globals.loadUrl = url : !deepLinker.handleUrl(url) && params.noActivityCallback && "function" == typeof params.noActivityCallback && params.noActivityCallback.call(this));
        }
    };
    var pushNotificationParamsHandler = function(params, isLaunch) {
        var type = null;
        if (params.data) {
            if (params.data.t) {
                type = parseInt(decodeType(params.data.t), 10);
                var typeId = 255 & type;
                if (params.data.tt && "a" === params.data.tt.toString()) {
                    var allowSamantha = require("featuretoggles/featuretoggles").getValue("samantha");
                    allowSamantha || require("featuretoggles/featuretoggles").ensureStarted();
                }
                switch (typeId) {
                  case pushType.POKE_RECEIVED:
                  case pushType.MESSAGE_RECEIVED:
                    Ti.App.fireEvent("carma:messages.count.change", {
                        source: "notification"
                    });
                    (false || Ti.App.Properties.getBool("com.activate.gcm.app.inforeground", false)) && appSound.play("snuffle");
                    break;

                  case pushType.RIDER_LEAVES_CAR:
                  case pushType.RIDER_DROP_OFF:
                  case pushType.RIDER_PICKED_UP:
                  case pushType.RIDER_JOINS_CAR:
                    Ti.App.fireEvent("carma:trips.changed", {
                        isDropoff: 9 > typeId
                    });
                    break;

                  case pushType.RIDE_REQUEST:
                  case pushType.RIDE_OFFER:
                    if (params.data.tt && "m" === params.data.tt.toString()) {
                        Ti.App.fireEvent("carma:notifications.change", {
                            source: isLaunch ? "notification_open" : "notification"
                        });
                        params.data.p = "showbookingrequest";
                        Ti.App.fireEvent("carma:open.conversation");
                    } else Ti.App.fireEvent("carma:home.tab");
                    break;

                  case pushType.DRIVER_RATE:
                  case pushType.RIDER_RATE:
                    Ti.App.Properties.setBool("carma:messages.display", false);
                    Ti.App.fireEvent("carma:notifications.change", {
                        source: isLaunch ? "notification_open" : "notification"
                    });
                    break;

                  case pushType.DRIVER_BOOKING_REMINDER:
                    params.data.p = "bookingreminder:driver";
                    break;

                  case pushType.RIDER_BOOKING_REMINDER:
                    params.data.p = "bookingreminder:rider";
                    break;

                  case pushType.USER_FAVOURITED:
                    (true && !params.ignoreRedirection || false) && Ti.App.fireEvent("carma:profile.open", {
                        userId: params.data.uid
                    });
                    break;

                  case pushType.INVITER_REWARD:
                    require("services/user_service").findOwn().then(function(data) {
                        Ti.App.fireEvent("app.carma.profile.change", data);
                    });
                    var json = _.extend({}, {
                        invitee: params.data.uid,
                        text: params.data.text,
                        currency: params.data.c,
                        reward: params.data.a
                    });
                    params.data.p = "inviter_reward/" + json.invitee + "/" + json.currency + "/" + json.reward;
                    break;

                  case pushType.AFR_MATCH:
                    require("/services/samantha_service").enablePushNotification();
                    break;

                  case pushType.AFR_BOOKING_CONFIRMED:
                    require("/services/samantha_service").enablePushNotification();
                    break;

                  case pushType.DRIVER_RIDE_BOOKING_CANCELLED:
                  case pushType.DRIVER_RIDE_REQUEST_CANCELLED:
                  case pushType.RIDER_RIDE_BOOKING_CANCELLED:
                  case pushType.DRIVER_RIDE_OFFER_DECLINED:
                  case pushType.BOOKING_CANCELLED_BY_DRIVER:
                  case pushType.RIDER_RIDE_OFFER_CANCELLED:
                    require("/services/samantha_service").enablePushNotification();
                    break;

                  case pushType.RIDE_BOOKING_DEVIATION_APPLIED:
                    break;

                  case pushType.DRIVER_RIDE_OFFER_ACCEPTED:
                  case pushType.BOOKING_ACCEPTED_BY_DRIVER:
                    params.data.tt && "m" === params.data.tt.toString() && Ti.App.fireEvent("carma:open.conversation");
                    break;

                  case pushType.TOPUP_TIMEOUT_SUCCESS:
                    globalEvents.fireTopupTimeoutResponse({
                        success: true,
                        amount: params.data.a
                    });
                    break;

                  case pushType.TOPUP_TIMEOUT_FAIL:
                    globalEvents.fireTopupTimeoutResponse({
                        success: false
                    });
                    break;

                  default:
                    LOGGER.error("[push notification] - type: " + type + ", t: " + params.data.t + ", id: " + typeId + ", " + JSON.stringify(params));
                    LOGGER.error("[push notification] - no action registered for previous notification");
                }
                if (256 === (256 & type)) {
                    var Globals = require("/v6/infrastructure/Globals"), bookingService = Globals.IOC.get("bookingService");
                    bookingService.emit(bookingService.BOOKING_UPDATED);
                    Ti.App.fireEvent("carma:booking.changed");
                }
            }
            params.data.p ? handleDeeplinkUrl(params, isLaunch) : true && params.noActivityCallback && "function" == typeof params.noActivityCallback && params.noActivityCallback.call(this);
            params.data.pt && trackPushTokenClick(params.data.pt);
            params.data.tId && Ti.App.fireEvent("carma:messages.change", {
                source: isLaunch ? "notification_open" : "notification",
                threadId: params.data.tId
            });
        }
    };
    var androidNewIntent = function(intent) {
        var params = {
            inBackground: true
        };
        intent.hasExtra("payload") && (params = JSON.parse(intent.getStringExtra("payload")));
        params.noActivityCallback = function() {
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.hasLogin() || Ti.App.fireEvent("carma:logout");
        };
        LOGGER.info("Android new intent received");
        Ti.App.fireEvent("carma:push.notification.open", params);
    };
    module.exports = {
        handle: pushNotificationParamsHandler,
        handleAndroidNewIntent: androidNewIntent,
        registerNotificationHandler: function() {
            Ti.App.addAppEventListener("carma:push.notification.open", pushNotificationParamsHandler);
        }
    };
}();