"use strict";

var Q = require("vendor/q"), moment = require("alloy/moment");

var _currentPage = 0, _pageSize = 10, _rides = null, _newUsers = null;

var activityId = function(activity) {
    if (!activity.length) return "";
    var ids = [];
    var length = activity.length;
    for (var i = 0; length > i; ++i) ids.push(activity[i].id);
    return ids.sort().toString();
};

var _filterLastDay = function(rides) {
    var lastDay = [];
    var oneday = moment().subtract("days", 1).valueOf();
    for (var i = 0; i < rides.length; i++) rides[i].endTimestamp >= oneday && lastDay.push(rides[i]);
    return lastDay;
};

var _filterLastDaySignups = function(users) {
    var lastDay = [];
    var oneday = moment().subtract("days", 1).valueOf();
    for (var i = 0; i < users.length; i++) users[i].lastKnownLocation.timestamp >= oneday && lastDay.push(users[i]);
    return lastDay;
};

var _mergeLists = function(rides, users) {
    var merged = [];
    LOGGER.debug("_mergeLists Users : " + users.length);
    merged = _.union(rides, users);
    var i = 0;
    merged.sort(function(a, b) {
        var tsA, tsB;
        tsA = a.endTimestamp ? a.endTimestamp : a.lastKnownLocation.timestamp;
        tsB = b.endTimestamp ? b.endTimestamp : b.lastKnownLocation.timestamp;
        if (tsB > tsA) return 1;
        if (tsA > tsB) return -1;
        return 0;
    });
    return merged;
};

var _persistUsersToCache = function(users) {
    TM.push(function() {
        var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
        _.each(users, function(user) {
            console.log("SAVE : " + JSON.stringify(user));
            ProfileService.saveMinimalProfile(user);
        });
    }, process.backgroundDomain);
};

var _cleanupActivityData = function(data) {
    var rides = JSON.parse(data.apiCallResponses[0].body);
    var users = JSON.parse(data.apiCallResponses[1].body);
    rides = _.toArray(rides.recentTrips);
    users = _.toArray(users.users);
    var dataset = {};
    dataset.rides = _.toArray(rides);
    dataset.users = _.toArray(users);
    _persistUsersToCache(dataset.users);
    if (dataset.rides && dataset.rides.length > 0) {
        var users = [];
        _.each(dataset.rides, function(ride) {
            users.push(ride.user);
        });
        _persistUsersToCache(users);
    }
    dataset.merged = _mergeLists(dataset.rides, dataset.users);
    dataset.recentActivity = dataset.rides;
    dataset.recentSignups = dataset.users;
    var resourceOwner = require("services/resource_owner_service").singleton();
    var userId = resourceOwner.get("userId");
    var cache = Ti.App.Properties.getObject("activityFeed_" + userId);
    Ti.App.Properties.setObject("activityFeed_" + userId, dataset);
    return dataset;
};

module.exports = {
    rides: null,
    findAll: function() {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        var self = this;
        var lastWeek = moment().subtract("days", 30).valueOf();
        var lastLocation = require("/gps/gps_manager").lastLocation();
        var radius = 25e3;
        var activity = Alloy.createModel("batch_api", {
            apiCallRequests: [ {
                method: "GET",
                url: "/rtr/v1.0/object/recentTrips/location?centerLat=" + lastLocation.lat + "&centerLon=" + lastLocation.lon + "&radius=" + radius + "&photoRequired=true&startTime=" + lastWeek + "&includeUnlikelyRides=false"
            }, {
                method: "GET",
                url: "/rtr/v1.0/object/users/recentSignups?centerLat=" + lastLocation.lat + "&centerLon=" + lastLocation.lon + "&radius=" + radius + "&userFields=ALIAS,PHOTO_URL,LAST_KNOWN_LOCATION_ADDRESS,WELCOMED,NUM_WELCOMES,GENDER"
            } ]
        });
        return activity.save().then(_cleanupActivityData).catch(function() {
            LOGGER.debug("ACTIVITY_ERROR Retuning cached....");
            return Ti.App.Properties.getObject("activityFeed_" + userId);
        });
    },
    getCached: function() {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        return Ti.App.Properties.getObject("activityFeed_" + userId);
    },
    restart: function() {},
    hasNext: function() {
        return false;
    }
};