"use strict";

var i18n = require("i18n/i18n"), Q = require("vendor/q");

module.exports = {
    bookingsRecord: function() {
        var rideBookingRecord = Alloy.createModel("ride_booking_record");
        rideBookingRecord.fetch();
        return rideBookingRecord;
    },
    findAll: function(onlyConfirmed) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        return Alloy.createModel("ride_booking").findAll(userId, onlyConfirmed);
    },
    findAllStates: function(tripId, baseEffectiveTime) {
        var userId = require("services/resource_owner_service").singleton().get("userId"), states = "REQUESTED,ACCEPTED,COMPLETED_RIDER_PICKUP,COMPLETED_DRIVER_PICKUP,DECLINED,CANCELLED,CANCELLED_TOO_OLD";
        return Alloy.createModel("ride_booking").findByStates(userId, tripId, baseEffectiveTime, states, true);
    },
    findByTrip: function(tripId, onlyConfirmed) {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), statesQuery = "ACCEPTED";
        onlyConfirmed || (statesQuery += ",REQUESTED");
        return Alloy.createModel("trip_ride_booking", {
            userId: userId,
            tripId: tripId,
            statesQuery: statesQuery
        }).fetch();
    },
    rideRequest: function(trip, groupingKey) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        var riderId, driverId, requestNote;
        var name = resourceOwner.get("alias");
        var destination = trip.destination.address;
        var dateTime = i18n.formatDateTime(moment(trip.effectiveTime));
        riderId = userId;
        driverId = trip.user.userId;
        return Alloy.createModel("ride_booking", {
            urlParams: {
                userId: userId
            },
            tripId: trip.id,
            riderId: riderId,
            driverId: driverId,
            bookingDate: trip.effectiveTime,
            groupingKey: groupingKey
        }).save();
    },
    rideOffer: function(trip, groupingKey) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        var riderId, driverId, requestNote;
        var name = resourceOwner.get("alias");
        var destination = trip.destination.address;
        var dateTime = i18n.formatTime(moment(trip.effectiveTime));
        riderId = trip.user.userId;
        driverId = userId;
        return Alloy.createModel("ride_booking", {
            urlParams: {
                userId: userId
            },
            tripId: trip.id,
            riderId: riderId,
            driverId: driverId,
            bookingDate: trip.effectiveTime,
            groupingKey: groupingKey
        }).save();
    },
    accept: function(bookingId, requestorName) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        var msg = String.format(L("widget_msg_booking_accept_system"), resourceOwner.get("alias"));
        return Alloy.createModel("ride_booking").accept(userId, bookingId, msg).then(function() {
            require("services/notification_service").updateCache(bookingId, "rideBookingRequests");
            return Q.fcall(function() {
                return true;
            });
        }).catch(function(e) {
            return Q.fcall(function() {
                return false;
            });
        });
    },
    decline: function(bookingId, requestorName) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        var msg = String.format(L("widget_msg_booking_decline_system"), resourceOwner.get("alias"));
        return Alloy.createModel("ride_booking").decline(userId, bookingId, msg).then(function() {
            require("services/notification_service").updateCache(bookingId, "rideBookingRequests");
            return Q.fcall(function() {
                return true;
            });
        }).catch(function(e) {
            return Q.fcall(function() {
                return false;
            });
        });
    }
};