"use strict";

var _ = require("/vendor/underscore"), EventEmitter = require("/v6/infrastructure/EventEmitter/Extend");

var Service = function() {};

Service.prototype = new EventEmitter();

Service.prototype.constructor = Service;

_.extend(Service.prototype, {
    INFINITE_PAGE_SIZE: 1e6,
    EVENT_FAVOURITE_USERS_COUNT_CHANGED: "favourite_users_count_changed",
    createCollection: function() {
        return Alloy.createCollection("favourite_user");
    },
    fetch: function(count) {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), collection = this.createCollection(), self = this;
        var defer = collection.fetchFavouriteUsers(userId, {
            pageSize: count
        });
        defer.then(function(favorites) {
            self._updateCacheFavoriteUsers(userId, favorites.length);
            self._setCacheFavoriteCount(userId, favorites.length);
        });
        return defer;
    },
    fetchAll: function() {
        return this.fetch(this.INFINITE_PAGE_SIZE);
    },
    getCache: function() {
        return this.createCollection().fetch({
            localOnly: true
        });
    },
    addFavouriteUser: function(user, callback) {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), self = this;
        var defer = Alloy.createModel("favourite_user_add", {
            urlParams: {
                userId: userId,
                favouriteUserId: user.userId
            }
        }).save();
        callback && defer.then(callback);
        defer.then(function() {
            var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
            ProfileService.saveProfileToCache(user);
            ProfileService.toggleFavourite(userId, user.userId, true);
            var collection = self.createCollection();
            collection.fetch({
                localOnly: true
            }).then(function(cachedFavorites) {
                cachedFavorites.push(user);
                collection.save(cachedFavorites, {
                    localOnly: true
                }).done();
            });
        });
        return defer;
    },
    removeFavouriteUser: function(favouriteUserId) {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), self = this;
        var defer = Alloy.createModel("favourite_user_remove", {
            urlParams: {
                userId: userId,
                favouriteUserId: favouriteUserId
            }
        }).save();
        defer.then(function() {
            var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
            ProfileService.toggleFavourite(userId, favouriteUserId, false);
            var collection = self.createCollection();
            collection.fetch({
                localOnly: true
            }).then(function(cachedFavorites) {
                var removedItem = self._removeItemByUserId(cachedFavorites, favouriteUserId);
                removedItem && collection.save(cachedFavorites, {
                    localOnly: true
                }).done();
            });
        });
        return defer;
    },
    compares: function(oldFavouritesList, newFavouritesList) {
        var _oldFavouritesList = oldFavouritesList ? oldFavouritesList : [];
        var _newFavouritesList = newFavouritesList ? newFavouritesList : [];
        var result = {
            added: [],
            updated: [],
            deleted: []
        };
        for (var i = 0; i < _newFavouritesList.length; i++) {
            var newItem = _newFavouritesList[i];
            var oldItem = this._findItemByUserId(_oldFavouritesList, newItem.userId);
            if (!oldItem) {
                result.added.push(newItem);
                continue;
            }
            (oldItem.alias != newItem.alias || oldItem.photoUrl != newItem.photoUrl || oldItem.gender != newItem.gender) && result.updated.push(newItem);
        }
        for (var i = 0; i < _oldFavouritesList.length; i++) {
            var oldItem = _oldFavouritesList[i];
            this._findItemByUserId(_newFavouritesList, oldItem.userId) || result.deleted.push(oldItem);
        }
        return result;
    },
    removeByUserId: function(favouritesList, userId) {
        return this._removeItemByUserId(favouritesList, userId);
    },
    _findItemByUserId: function(favouritesList, userId) {
        var index = this._findIndexByUserId(favouritesList, userId);
        return -1 != index ? favouritesList[index] : null;
    },
    _removeItemByUserId: function(favouritesList, userId) {
        var index = this._findIndexByUserId(favouritesList, userId), item = -1 !== index ? favouritesList[index] : null;
        -1 !== index && favouritesList.splice(index, 1);
        return item;
    },
    _findIndexByUserId: function(favouritesList, userId) {
        if (!userId && 0 !== userId) return -1;
        var _favouritesList = favouritesList ? favouritesList : [];
        for (var i = 0; i < _favouritesList.length; i++) {
            var item = _favouritesList[i];
            if (item.userId === userId) return i;
        }
        return -1;
    },
    _updateCacheFavoriteUsers: function(userId, count) {
        Alloy.createCollection("favourite_user").fetchFavouriteUsers(userId, {
            pageSize: count
        });
    },
    _setCacheFavoriteUsers: function(data) {
        var favoriteCollection = Alloy.createCollection("favourite_user");
        favoriteCollection.fetch({
            localOnly: true
        }).then(function(cache) {
            favoriteCollection.save(data, {
                localOnly: true
            });
        });
    },
    _setCacheFavoriteCount: function(userId, count) {
        var key = "user-count-favourites-" + userId;
        Ti.App.Properties.setInt(key, count);
    }
});

module.exports = Service;