"use strict";

var _ = require("/vendor/underscore"), EventEmitter = require("/v6/infrastructure/EventEmitter/Extend");

var Service = function() {};

Service.prototype = new EventEmitter();

Service.prototype.constructor = Service;

_.extend(Service.prototype, {
    getCache: function() {
        return this.createFreeRiderCollection().getCache();
    },
    getFreeRiders: function(options) {
        return this.createFreeRiderCollection().findAll(options);
    },
    addFreeRider: function(user) {
        var collection = this.createFreeRiderCollection(), promise = collection.addFreeRider(user), self = this;
        promise.then(function() {
            self._addUserToCache(user);
        });
        return promise;
    },
    removeFreeRider: function(user) {
        var collection = this.createFreeRiderCollection(), promise = collection.deleteFreeRider(user), self = this;
        promise.then(function() {
            self._removeUserFromCache(user);
        });
        return promise;
    },
    compares: function(oldUserList, newUserList) {
        var _oldUserList = oldUserList ? oldUserList : [];
        var _newUserList = newUserList ? newUserList : [];
        var result = {
            added: [],
            updated: [],
            deleted: []
        };
        for (var i = 0; i < _newUserList.length; i++) {
            var newItem = _newUserList[i];
            var oldItem = this._findItemByUserId(_oldUserList, newItem.userId);
            if (!oldItem) {
                result.added.push(newItem);
                continue;
            }
            (oldItem.alias != newItem.alias || oldItem.photoUrl != newItem.photoUrl || oldItem.gender != newItem.gender) && result.updated.push(newItem);
        }
        for (var i = 0; i < _oldUserList.length; i++) {
            var oldItem = _oldUserList[i];
            this._findItemByUserId(_newUserList, oldItem.userId) || result.deleted.push(oldItem);
        }
        return result;
    },
    removeByUserId: function(userList, id) {
        var index = this._findIndexByUserId(userList, id), item = -1 !== index ? userList[index] : null;
        -1 !== index && userList.splice(index, 1);
        return item;
    },
    _addUserToCache: function(user) {
        var collection = this.createFreeRiderCollection(), self = this;
        collection.getCache().then(function(cachedUsers) {
            var index = self._findIndexByUserId(cachedUsers, user.userId);
            if (-1 !== index) throw new Error("User[" + user.userId + "] has been a free rider of user[" + self.getSelfId() + "].");
            cachedUsers.push({
                userId: user.userId,
                photoUrl: user.photoUrl,
                alias: user.alias,
                gender: user.gender
            });
            collection.updateCache(cachedUsers).done();
            var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
            ProfileService.saveProfileToCache(user);
        }).done();
    },
    _removeUserFromCache: function(user) {
        var self = this;
        this.getCache().then(function(cachedUsers) {
            var index = self._findIndexByUserId(cachedUsers, user.userId);
            if (-1 === index) throw new Error("User[" + user.userId + "] is not a free rider of user[" + self.getSelfId() + "].");
            cachedUsers.splice(index, 1);
            self._updateCacheOnly(cachedUsers);
        }).catch(function(e) {
            LOGGER.error("Unkown exception on removeUserFromCache : " + JSON.stringify(e));
        }).done();
    },
    _findIndexByUserId: function(users, id) {
        if (!id || !users || 0 === users.length) return -1;
        for (var i = 0; i < users.length; i++) if (users[i].userId === id) return i;
        return -1;
    },
    _findItemByUserId: function(users, id) {
        var index = this._findIndexByUserId(users, id);
        return -1 !== index ? users[index] : null;
    },
    _updateCacheOnly: function(users) {
        return this.createFreeRiderCollection().updateCache(users);
    },
    getSelfId: function() {
        return require("services/resource_owner_service").singleton().get("userId");
    },
    createFreeRiderCollection: function() {
        return Alloy.createCollection("free_riders");
    }
});

module.exports = Service;