"use strict";

var chatModel = null, conversationCollection = null;

module.exports = {
    unreadCount: function() {
        var resourceOwner = require("services/resource_owner_service").singleton();
        return Alloy.createModel("user_messages", {
            urlParams: {
                userId: resourceOwner.get("userId")
            }
        }).countUnread();
    },
    markThreadRead: function(options) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        return Alloy.createModel("user_messages").markThreadRead({
            userId: resourceOwner.get("userId"),
            threadId: options.threadId
        });
    },
    countThreadUnread: function(options) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        return Alloy.createModel("user_messages").countThreadUnread({
            userId: resourceOwner.get("userId"),
            threadId: options.threadId
        });
    },
    send: function(userId, message) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var senderId = resourceOwner.get("userId");
        return Alloy.createModel("user_messages", {
            urlParams: {
                userId: userId
            },
            messageType: "USER_TEXT",
            sendingUser: senderId,
            body: message,
            recipientUser: userId
        }).save();
    },
    sendGroup: function(groupId, message) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var senderId = resourceOwner.get("userId");
        return Alloy.createModel("user_messages").sendGroupMessage({
            groupId: groupId,
            messageType: "USER_TEXT",
            sendingUser: senderId,
            body: message
        });
    },
    deleteChat: function(options) {
        options = options || {};
        options.userId = require("services/resource_owner_service").singleton().get("userId");
        return Alloy.createModel("user_messages", {
            id: options.threadId
        }).deleteChat(options);
    },
    poke: function(attr) {
        attr = attr || {};
        attr.sendingUser = require("services/resource_owner_service").singleton().get("userId");
        attr.timestamp = Date.now();
        attr.messageType = "USER_POKE";
        return Alloy.createModel("user_messages").poke(attr);
    },
    getConversations: function() {
        conversationCollection = Alloy.createCollection("user_conversations");
        return module.exports.getCached().then(function(cache) {
            var timestampsCache = [];
            cache && cache.length > 0 && cache.forEach(function(item) {
                timestampsCache.push(item.messages.notifications[0].timestamp);
            });
            return conversationCollection.fetch().then(function(data) {
                var timestamps = [];
                data && data.length > 0 && data.forEach(function(item) {
                    timestamps.push(item.messages.notifications[0].timestamp);
                });
                if (timestampsCache.length && timestampsCache.toString() === timestamps.toString()) return -1;
                return data;
            });
        });
    },
    getCached: function() {
        conversationCollection = Alloy.createCollection("user_conversations");
        return conversationCollection.fetch({
            localOnly: true
        });
    },
    getNextConversationPage: function() {
        return conversationCollection.next();
    },
    conversationHasNextPage: function() {
        return conversationCollection.hasNext();
    },
    getChatMessages: function(options) {
        chatModel = Alloy.createModel("user_chat", {
            id: options.threadId
        });
        return chatModel.fetch(options);
    },
    getNextChatMessagesPage: function(page) {
        return chatModel.next();
    },
    chatHasNextPage: function() {
        return chatModel.hasNext();
    },
    getNewChatMessages: function(options) {
        return chatModel.fetch(options);
    },
    updateConversationsCache: function(threadId) {
        return Alloy.createCollection("user_conversations").fetch();
    }
};