"use strict";

var Q = require("vendor/q");

var _checkTripCount = function(actionRecord, trips, difference) {
    var target = trips + difference;
    var message, variation;
    if ((10 === target || 100 === target || 250 === target || target >= 500 && target % 500 === 0) && !actionRecord.get("upcoming_trips_at_" + target)) {
        actionRecord.set("upcoming_trips_at_" + target, true);
        if (10 === target || 250 === target) {
            message = String.format(L("widget_milestone_trips_message"), "" + difference);
            variation = 1;
        } else {
            message = String.format(L("widget_milestone_trips_alternative_message"), "" + difference);
            variation = 2;
        }
        return {
            message: message,
            type: "TRIP",
            variation: variation
        };
    }
    return null;
};

var _checkUserCount = function(actionRecord, users, difference) {
    var target = users + difference;
    var message, variation;
    if ((5 === target || 10 === target || 15 === target || target >= 20 && target % 20 === 0) && !actionRecord.get("upcoming_users_at_" + target)) {
        actionRecord.set("upcoming_users_at_" + target, true);
        if (10 === target || target > 20) {
            message = String.format(L("widget_milestone_users_alternative_message"), "" + difference, "" + target);
            variation = 2;
        } else {
            message = String.format(L("widget_milestone_users_message"), "" + difference);
            variation = 1;
        }
        return {
            message: message,
            type: "USERS",
            variation: variation
        };
    }
    return null;
};

var _checkDistanceRecord = function(actionRecord, currentDistance) {
    var distanceUnit = require("services/cached_preferences_service").singleton().getPreferredDistanceUnit();
    var distanceUnitShort = L("MILES" === this._distanceUnit ? "unit_miles" : "unit_kilometers");
    var targets = [ 100, 500, 1e3, 2500, 5e3, 1e4 ];
    var distance = 0;
    distance = "MILES" === distanceUnit ? require("i18n/i18n").formatDistanceFromMeters(currentDistance) : currentDistance / 1e3;
    distance = Math.round(distance);
    for (var i = 0; i < targets.length; i++) if (distance < targets[i] && distance > targets[i] - 20 && !actionRecord.get("upcoming_distance_at_" + targets[i] + "_" + distanceUnitShort)) {
        actionRecord.set("upcoming_distance_at_" + targets[i] + "_" + distanceUnitShort, true);
        var difference = targets[i] - distance;
        var message = String.format(L("widget_milestone_distance_message"), "" + difference, distanceUnitShort);
        var variation = 1;
        return {
            message: message,
            type: "DISTANCE",
            variation: variation
        };
    }
    return null;
};

var _chooseMilestoneAlert = function(milestoneData) {
    var actionRecord = require("services/action_record_service");
    var difference = 1;
    var userMilestone = _checkUserCount(actionRecord, milestoneData.users, difference);
    if (null !== userMilestone) return userMilestone;
    difference = 3;
    var tripMilestone = _checkTripCount(actionRecord, milestoneData.rides + milestoneData.drives, difference);
    if (null !== tripMilestone) return tripMilestone;
    var distanceMilestone = _checkDistanceRecord(actionRecord, milestoneData.distance);
    if (null !== distanceMilestone) return distanceMilestone;
    return null;
};

var _presentDialog = function(milestone) {
    Ti.App.fireEvent("carma:coachmark", {
        name: "milestone",
        params: milestone
    });
};

module.exports = {
    determineMilestones: function() {
        var userId = require("services/resource_owner_service").singleton().get("userId");
        require("services/user_service").getRideStats(userId).then(function(data) {
            var alert = _chooseMilestoneAlert(data);
            null !== alert && _presentDialog(alert);
        });
    }
};