"use strict";

var Q = require("vendor/q");

var notificationDeletionQueue = [], isDeleting = false, _currentPage = 0, _pageSize = 4;

var notificationsId = function(notifications) {
    if (!notifications.length) return "";
    var ids = [];
    var length = notifications.length;
    for (var i = 0; length > i; ++i) ids.push(notifications[i].id);
    return ids.sort().toString();
};

var _getCurrentPageData = function(data) {
    if (!data) return {
        count: 0
    };
    var bookings = data.rideBookingRequests, dropoffs = data.dropoffNotifications, dataSet = {};
    dataSet.rideBookingRequests = _.toArray(bookings).splice(_currentPage * _pageSize, _pageSize);
    dataSet.dropoffNotifications = _.toArray(dropoffs).splice(_currentPage * _pageSize - bookings.length, _pageSize - dataSet.rideBookingRequests.length);
    dataSet.count = bookings.length + dropoffs.length;
    return dataSet;
};

module.exports = {
    unreadCount: function() {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        return Alloy.createModel("notification").count(userId).then(function(data) {
            return data.response ? parseInt(data.response) : 0;
        });
    },
    findAll: function() {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        var self = this;
        return Alloy.createModel("notification").findAll(userId).then(function(data) {
            var rideBookingRequests = [];
            _.each(data.rideBookingRequests, function(booking) {
                var initiator = booking.initiator.toLowerCase();
                var initiatedByMe = booking[initiator].userId === userId;
                initiatedByMe || "REQUESTED" !== booking.state || rideBookingRequests.push(booking);
            });
            data.rideBookingRequests = rideBookingRequests;
            var dropoffNotifications = [];
            _.each(data.dropoffNotifications, function(rating) {
                var targetType = rating.driver ? "driver" : "rider";
                var targetUser = rating[targetType];
                targetUser.userId > 0 && dropoffNotifications.push(rating);
            });
            data.dropoffNotifications = dropoffNotifications;
            data.count = data.rideBookingRequests.length + data.dropoffNotifications.length;
            var cache = Ti.App.Properties.getObject("notifications_" + userId);
            if (cache) {
                var cacheIds = notificationsId(cache.dropoffNotifications) + "," + notificationsId(cache.rideBookingRequests);
                var dataIds = notificationsId(data.dropoffNotifications) + "," + notificationsId(data.rideBookingRequests);
                if (cacheIds === dataIds) return -1;
            }
            Ti.App.Properties.setObject("notifications_" + userId, data);
            return _getCurrentPageData(data);
        }).catch(function() {
            return _getCurrentPageData(Ti.App.Properties.getObject("notifications_" + userId));
        });
    },
    getCached: function(paged) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        return paged ? _getCurrentPageData(Ti.App.Properties.getObject("notifications_" + userId)) : Ti.App.Properties.getObject("notifications_" + userId);
    },
    deleteDropOff: function(notificationId, callback) {
        notificationId && notificationDeletionQueue.push({
            id: notificationId,
            callback: callback
        });
        if (0 === notificationDeletionQueue.length) return;
        if (isDeleting) return;
        isDeleting = true;
        var notification = notificationDeletionQueue.shift(), resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), self = this;
        Alloy.createModel("notification", {
            id: notification.id
        }).deleteDropOff(userId, notification.id).finally(function() {
            self.updateCache(notification.id, "dropoffNotifications");
            notification.callback && TM.push(function() {
                notification.callback();
            }, process.backgroundDomain);
            isDeleting = false;
            module.exports.deleteDropOff();
        });
    },
    deleteDropOffNoQueue: function(notificationId) {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), self = this;
        return Alloy.createModel("notification", {
            id: notificationId
        }).deleteDropOff(userId, notificationId).then(function() {
            self.updateCache(notificationId, "dropoffNotifications");
            return Q.fcall(function() {
                return true;
            });
        }).catch(function() {
            return Q.fcall(function() {
                throw new Error();
            });
        });
    },
    updateCache: function(id, type) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        var cache = Ti.App.Properties.getObject("notifications_" + userId) || {};
        var notifications = cache[type];
        if (!notifications) return;
        var reduced = _.reject(notifications, function(notification) {
            return notification.id === id;
        });
        cache[type] = reduced;
        cache.count = cache.rideBookingRequests.length + cache.dropoffNotifications.length;
        Ti.App.Properties.setObject("notifications_" + userId, cache);
    },
    hasNext: function() {
        var cache = this.getCached() || {
            count: 0
        }, totalPages = Math.ceil(cache.count / _pageSize);
        return totalPages > _currentPage + 1;
    },
    next: function() {
        if (!this.hasNext()) return Q.fcall(function() {
            return false;
        });
        ++_currentPage;
        var self = this;
        return Q.fcall(function() {
            return self.getCached(true);
        });
    },
    restart: function() {
        _currentPage = 0;
    }
};