"use strict";

var Q = require("vendor/q");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var _upload = function(url, token, media) {
    var deferred = Q.defer();
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            LOGGER.info("Image sent to server successfully...");
            deferred.resolve({
                url: url,
                media: media
            });
        },
        onerror: function(e) {
            LOGGER.debug(xhr.responseText);
            LOGGER.error("Failed to send image to server: " + e.error);
            deferred.reject({
                status: xhr.status,
                error: e.error
            });
        },
        timeout: 12e4
    });
    xhr.open("POST", url);
    LoggingService.log("network", url);
    xhr.setRequestHeader("Content-Type", "image/jpeg");
    xhr.setRequestHeader("Accept", "application/json");
    token && xhr.setRequestHeader("Authorization", "Bearer " + token.token);
    xhr.send(media);
    return deferred.promise;
};

module.exports = {
    saveUserImage: function(media) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var url = require("bootstrap/env").getAPI1() + resourceOwner.get("userId") + "/photo";
        var token = resourceOwner.get("token");
        return _upload(url, token, media);
    },
    saveGroupImage: function(groupId, media) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var url = require("bootstrap/env").getAPI2().replace("users", "groups") + groupId + "/photo";
        var token = resourceOwner.get("token");
        return _upload(url, token, media);
    }
};