"use strict";

var booking_service = require("/services/booking_service"), Q = require("vendor/q");

module.exports = {
    findBookings: function(onlyConfirmed) {
        return booking_service.findAll(onlyConfirmed);
    },
    acceptBooking: function(bookingId, requestorName) {
        return booking_service.accept(bookingId, requestorName);
    },
    declineBooking: function(bookingId, requestorName) {
        return booking_service.decline(bookingId, requestorName);
    },
    getRideRequests: function(rideRequestId) {
        return Alloy.createModel("auto_ride_request").fetch({
            rideRequestId: rideRequestId
        });
    },
    bookingsRecord: function() {
        return booking_service.bookingsRecord();
    },
    getSamanthaState: function(tripId, timestamp) {
        var deferred = Q.defer();
        Alloy.createModel("trip").getTrip(tripId, {
            data: {
                relativeTime: timestamp
            }
        }).then(function(result) {
            var state = "";
            state = result.baseTrip.autoRideMatchStatus ? result.baseTrip.autoRideMatchStatus : "NO_TRIGGERRED";
            deferred.resolve(state);
        });
        return deferred.promise;
    },
    getNextTrip: function() {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), tripsUrl = "/api/rtr/v2/users/" + userId + "/trips", nextTripUrl = "/api/rtr/v2/users/" + userId + "/trips/next?excludeNotGoingTrips=false&tripFields=LOCATIONS,LOCATION_ADDRESSES,LOCATION_ALIASES,ALIAS,USER_ROLE,AUTO_FIND_RIDE_STATUS,BOOKINGS,HAS_MATCHES,CONFIRMATION,SCHEDULE,EFFECTIVE_TIME", apiCallRequests = [];
        apiCallRequests.push({
            method: "GET",
            url: tripsUrl
        });
        apiCallRequests.push({
            method: "GET",
            url: nextTripUrl
        });
        var nextTrip = Alloy.createModel("batch_api", {
            apiCallRequests: apiCallRequests
        });
        return nextTrip.save();
    },
    getNext2Trips: function() {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), time = Date.now() - 54e5, tripsUrl = "/api/rtr/v2/users/" + userId + "/trips", nextTripUrl = "/api/rtr/v2/users/" + userId + "/trips/nextn?numTrips=2&relativeTime=" + time + "&excludeNotGoingTrips=false&tripFields=LOCATIONS,LOCATION_ADDRESSES,LOCATION_ALIASES,ALIAS,USER_ROLE,AUTO_FIND_RIDE_STATUS,BOOKINGS,HAS_MATCHES,CONFIRMATION,SCHEDULE,EFFECTIVE_TIME", apiCallRequests = [];
        apiCallRequests.push({
            method: "GET",
            url: tripsUrl
        });
        apiCallRequests.push({
            method: "GET",
            url: nextTripUrl
        });
        var nextTrips = Alloy.createModel("batch_api", {
            apiCallRequests: apiCallRequests
        });
        return nextTrips.save();
    },
    markBookingAsRead: function(bookingId, timestamp) {
        return booking_service.bookingsRecord().markAsRead(bookingId, timestamp);
    },
    isBookingRead: function(bookingId) {
        return booking_service.bookingsRecord().isRead(bookingId);
    },
    markBookingAsPop: function(bookingId, timestamp) {
        return booking_service.bookingsRecord().markAsPop(bookingId, timestamp);
    },
    isBookingPop: function(bookingId) {
        return booking_service.bookingsRecord().isPop(bookingId);
    },
    enablePushNotification: function() {
        return Alloy.createModel("auto_ride_enable_push").enable();
    }
};