"use strict";

var q = require("vendor/q");

module.exports = {
    findOwn: function() {
        return Alloy.createModel("user", {
            userId: "SELF"
        }).fetch().then(function(data) {
            if (data) {
                var resourceOwner = require("services/resource_owner_service").singleton();
                var countries = require("utils/countries");
                var countryInfo = _.findWhere(countries.countries, {
                    cca3: data.country
                });
                countryInfo && (data.country2chars = countryInfo.cca2);
                resourceOwner.save(data);
            }
            return data;
        });
    },
    findOne: function(userId) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        if (userId === resourceOwner.get("userId")) return this.findOwn();
        return Alloy.createModel("public_profile", {
            userId: userId
        }).fetch();
    },
    cachedCount: function(key, userId) {
        var key = "user-count-" + key + "-" + userId;
        return Ti.App.Properties.getInt(key, null);
    },
    countTrips: function(userId) {
        var trips = Alloy.createModel("trips", {
            urlParams: {
                userId: userId
            }
        });
        return trips.fetch().then(function(data) {
            var count = data.paginator.totalResults;
            var key = "user-count-trips-" + userId;
            Ti.App.Properties.setInt(key, count);
            return count;
        });
    },
    countGroups: function(userId) {
        var groups = Alloy.createModel("groups", {
            urlParams: {
                userId: userId
            }
        });
        return groups.fetch().then(function(data) {
            var count = data && data.groups ? data.groups.length : 0;
            var key = "user-count-groups-" + userId;
            Ti.App.Properties.setInt(key, count);
            return count;
        });
    },
    countFavourites: function(userId) {
        var favourites = Alloy.createModel("favourite_user_count", {
            urlParams: {
                userId: userId
            }
        });
        return favourites.fetch().then(function(data) {
            var count = data.count;
            var key = "user-count-favourites-" + userId;
            Ti.App.Properties.setInt(key, count);
            return count;
        });
    },
    countRides: function(userId) {
        var deferred = q.defer();
        var rides = Alloy.createModel("ride_history_count", {
            urlParams: {
                userId: userId
            }
        });
        return rides.fetch().then(function(data) {
            var count = data.count;
            var key = "user-count-rides-" + userId;
            Ti.App.Properties.setInt(key, count);
            return count;
        });
    },
    getRideStats: function(userId) {
        var deferred = q.defer();
        var rides = Alloy.createModel("ride_history_count", {
            urlParams: {
                userId: userId
            }
        });
        return rides.fetch().then(function(statistics) {
            return statistics;
        });
    },
    getReferralCode: function() {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId");
        return Alloy.createModel("referral_code", {
            urlParams: {
                userId: userId
            }
        }).save();
    },
    cachReferralCode: function(referralCode) {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId");
        var key = "user-referral-code-" + userId;
        Ti.App.Properties.setString(key, referralCode);
    },
    getCachReferralCode: function() {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId");
        var key = "user-referral-code-" + userId;
        return 0 == Ti.App.Properties.getString(key, null) ? null : Ti.App.Properties.getString(key, null);
    },
    createInviteCode: function() {
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId");
        return Alloy.createModel("invite_code", {
            urlParams: {
                userId: userId
            }
        }).save();
    },
    getInformationCards: function(userId, lat, lon) {
        var cards = Alloy.createModel("user_information_cards");
        return cards.getCards({
            userId: userId,
            lat: lat,
            lon: lon
        });
    }
};