"use strict";

!function() {
    var KEY = "TRACK_USER_EVENT", isFlushing = false, flushTimer = null, FLUSH_TIMEOUT = 1e4, FLUSH_THRESHOLD = 50;
    var _push = function(event) {
        var items = Ti.App.Properties.getList(KEY, []);
        items.push(event);
        Ti.App.Properties.setList(KEY, items);
    };
    var _pushFront = function(events) {
        if (!events || 0 === events.length) return;
        var items = Ti.App.Properties.getList(KEY, []);
        items = events.concat(items);
        Ti.App.Properties.setList(KEY, items);
    };
    var _pop = function(numEvents) {
        var events = Ti.App.Properties.getList(KEY, []);
        if (events.length > numEvents) {
            var ret = events.splice(0, numEvents);
            Ti.App.Properties.setList(KEY, events);
            return ret;
        }
        Ti.App.Properties.setList(KEY, []);
        return events;
    };
    var _hasReachFlushThreshold = function() {
        var events = Ti.App.Properties.getList(KEY, []);
        if (events.length < FLUSH_THRESHOLD) return false;
        return true;
    };
    var _flush = function() {
        if (!require("bootstrap/network").isConnectionGood()) {
            LOGGER.warn("[TRACKING] - bad connection, wait for next turn");
            return;
        }
        LOGGER.debug("[TRACKING] - start flushing...");
        var events = _pop(FLUSH_THRESHOLD);
        if (0 === events.length) {
            LOGGER.debug("[TRACKING] - no events left, nothing to flush");
            return;
        }
        isFlushing = true;
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = -1;
        resourceOwner.hasLogin() && (userId = resourceOwner.get("userId"));
        var model = Alloy.createModel("tracking", {
            urlParams: {
                userId: userId
            },
            events: events
        });
        model.save().then(function() {
            LOGGER.info("[TRACKING] - Successfully flushed the queue for activity");
        }).catch(function() {
            LOGGER.info("[TRACKING] - Failed to flush the queue for activity, put them back to queue");
            _pushFront(events);
        }).finally(function() {
            isFlushing = false;
            LOGGER.info("[TRACKING] - after flushing, re-check if we need to flush again...");
            _prepareFlush();
        }).done();
    };
    var _resetTimer = function() {
        if (!flushTimer) return;
        clearTimeout(flushTimer);
        flushTimer = null;
    };
    var _startFlushCountDown = function() {
        if (flushTimer) return;
        flushTimer = setTimeout(function() {
            _resetTimer();
            TM.push(function() {
                _flush();
            }, process.backgroundDomain);
        }, FLUSH_TIMEOUT);
    };
    var _prepareFlush = function() {
        if (isFlushing) {
            LOGGER.debug("[TRACKING] - flushing in progress, leave it alone");
            return;
        }
        if (_hasReachFlushThreshold()) {
            LOGGER.debug("[TRACKING] - force flush or reach flush threshold");
            _resetTimer();
            TM.push(function() {
                _flush();
            }, process.backgroundDomain);
            return;
        }
        LOGGER.debug("[TRACKING] - start flush count down");
        _startFlushCountDown();
    };
    module.exports = {
        track: function(event, properties) {
            LOGGER.debug("[TRACKING] - activity track: " + event);
            _push({
                event: event,
                timestamp: Date.now(),
                properties: properties
            });
            _prepareFlush();
        },
        flush: _prepareFlush
    };
}();