"use strict";

var activity = require("tracking/activity"), trackingConfig = require("services/tracking_config_service"), pushNotification = require("tracking/push_notification");

var _normalizeTrackingData = function(properties) {
    var data = {};
    for (var key in properties) void 0 !== properties[key] && null !== properties[key] && (data[key] = properties[key]);
    return data;
};

var _injectDeviceId = function(properties) {
    if (properties.hasOwnProperty("device_id")) return;
    var deviceId = "unknown";
    try {
        deviceId = Ti.Platform.macaddress;
    } catch (e) {}
    properties.device_id = deviceId;
};

var _injectUserId = function(properties) {
    if (properties.hasOwnProperty("user_id")) return;
    var resourceOwner = require("services/resource_owner_service").singleton();
    resourceOwner.hasLogin() && (properties.user_id = resourceOwner.get("userId"));
};

var _rollDice = function(event) {
    var config = trackingConfig.getTrackingConfig();
    if (!config.hasOwnProperty(event)) return true;
    var random = Math.random();
    return random <= config[event].sampleRate ? true : false;
};

module.exports = {
    init: function() {
        LOGGER.debug("[TRACKING] - init tracking manager");
        Ti.App.addAppEventListener("carma:network.change", require("/bootstrap/patch").debounce(function() {
            activity.flush();
            pushNotification.flush();
        }, 3e3));
    },
    track: function(event, properties) {
        if (!event) {
            LOGGER.warn("[TRACKING] - track: The event name must be given");
            return;
        }
        if (!_rollDice(event)) return;
        properties = properties || {};
        var data = _normalizeTrackingData(properties);
        _injectDeviceId(data);
        _injectUserId(data);
        "push-notification-open" === event ? pushNotification.track(event, data) : activity.track(event, data);
    }
};