"use strict";

var CACHE_FOLDER = Ti.Filesystem.getFile(Ti.Filesystem.applicationCacheDirectory, "profile-cache"), Q = require("vendor/q");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

CACHE_FOLDER.exists() || CACHE_FOLDER.createDirectory();

module.exports = {
    _ONLINE_THRESHOLD: 18e5,
    _RECENT_ONLINE_THRESHOLD: 72e5,
    _COLOR_OFFLINE: "#bebebe",
    _COLOR_RECENT_ONLINE: "#f4dd5e",
    _COLOR_ONLINE: "#b1ec11",
    getStatusColor: function(lastSeen) {
        if (!lastSeen) return this._COLOR_OFFLINE;
        var diff = new Date().getTime() - lastSeen;
        if (diff < this._ONLINE_THRESHOLD) return this._COLOR_ONLINE;
        if (diff < this._RECENT_ONLINE_THRESHOLD) return this._COLOR_RECENT_ONLINE;
        return this._COLOR_OFFLINE;
    },
    getLocalPhoto: function(user, options) {
        if (!user || !user.photoUrl) return false;
        options = options || {};
        var url = require("utils/photoURL").toURL(user.photoUrl, {
            width: options.size || 50,
            height: options.size || 50,
            circular: options.circular
        });
        var file = Ti.Filesystem.getFile(CACHE_FOLDER.resolve(), this._localFileName(url));
        if (file.exists()) return file.getNativePath();
        return false;
    },
    getUserPhoto: function(user, isGroup, options) {
        var deferred = Q.defer();
        options = options || {};
        options.width = options.size || 50;
        options.height = options.size || 50;
        if (!user || !user.photoUrl) {
            deferred.resolve(isGroup ? this.getDefaultGroupPhoto() : this.getDefaultPhoto(user));
            return deferred.promise;
        }
        var url = require("utils/photoURL").toURL(user.photoUrl, {
            width: options.width,
            height: options.height,
            circular: options.circular
        });
        return this.loadPhoto(url, deferred);
    },
    loadPhoto: function(url, deferred) {
        deferred = deferred ? deferred : Q.defer();
        var file = Ti.Filesystem.getFile(CACHE_FOLDER.resolve(), this._localFileName(url));
        if (file.exists()) {
            deferred.resolve(file.getNativePath());
            return deferred.promise;
        }
        if (!url) {
            deferred.reject();
            return deferred.promise;
        }
        var client = Ti.Network.createHTTPClient({
            onload: function(e) {
                file.write(this.responseData);
                deferred.resolve(file.getNativePath());
            },
            onerror: function() {
                deferred.reject();
            },
            timeout: 5e3
        });
        TM.push(function() {
            client.open("GET", url);
            LoggingService.log("network", url);
            client.send();
        });
        return deferred.promise;
    },
    getDefaultPhoto: function(user) {
        var gender = (user || {}).gender;
        return this.getDefaultPhotoByGender("FEMALE" === gender || "f" === gender);
    },
    getDefaultPhotoByGender: function(isFemale) {
        return isFemale ? "/images/default_photo_female-rounded.png" : "/images/default_photo_male-rounded.png";
    },
    getDefaultGroupPhoto: function() {
        return "/images/default_group_picture-rounded.png";
    },
    _localFileName: function(imageURL) {
        return Ti.Utils.md5HexDigest(imageURL + "");
    }
};