"use strict";

var dimension = require("/platform/dimension");

var AnimatedBottomView = function() {
    var _options = null, _bottomWhenViewOnTop = null, _container = null, _triggers = null, _timer = null;
    var _registerAnimationTriggers = function(triggers) {
        var singular = !_.isArray(triggers);
        _triggers = singular ? [ triggers ] : triggers;
        _triggers.forEach(function(trigger) {
            _registerAnimationTrigger(trigger);
        });
    };
    var _registerAnimationTrigger = function(trigger) {
        trigger.isFocus = false;
        trigger.addEventListener("focus", _focus);
        trigger.addEventListener("blur", _blur);
    };
    var _focus = function(e) {
        e.source.isFocus = true;
        _moveViewToTop();
        _startTimer();
    };
    var _blur = function(e) {
        e.source.isFocus = false;
    };
    var _startTimer = function() {
        if (_timer) return;
        _timer = setInterval(_checkFocusedTriggers, 100);
    };
    var _checkFocusedTriggers = function() {
        var trigger = _getFocusedTrigger();
        if (trigger) return;
        _moveViewToBottom();
        clearInterval(_timer);
        _timer = null;
    };
    var _calcBottomWhenViewOnTop = function() {
        if (_options.hasOwnProperty("bottomWhenViewOnTop")) return _options.bottomWhenViewOnTop;
        var windowHeight = dimension.windowHeightInSystemUnit(), navBarHeight = dimension.navBarHeightInSystemUnit();
        if (_container.size.height > 0) return windowHeight - navBarHeight - _container.size.height;
    };
    var _moveViewToTop = function() {
        var trigger = _getFocusedTrigger();
        if (!trigger) return;
        _bottomWhenViewOnTop || (_bottomWhenViewOnTop = _calcBottomWhenViewOnTop());
        var callbackResult = true;
        _options.hasOwnProperty("beforeMoveToTop") && (callbackResult = _options.beforeMoveToTop());
        if (false === callbackResult) return;
        _container.animate({
            bottom: _bottomWhenViewOnTop,
            duration: 200
        }, function() {
            _container.bottom = _bottomWhenViewOnTop;
        });
    };
    var _moveViewToBottom = function() {
        var callbackResult = true;
        _options.hasOwnProperty("beforeMoveToBottom") && (callbackResult = _options.beforeMoveToBottom());
        if (false === callbackResult) return;
        _container.animate({
            bottom: _options.bottom || 0,
            duration: 200
        }, function() {
            _container.bottom = _options.bottom || 0;
        });
    };
    var _getFocusedTrigger = function() {
        for (var i = 0; i < _triggers.length; i++) {
            var trigger = _triggers[i];
            if (trigger.isFocus) return trigger;
        }
        return null;
    };
    return {
        setup: function(options) {
            _options = options || {};
            if (!_options.container || !_options.triggers) return;
            _container = _options.container;
            _registerAnimationTriggers(_options.triggers);
        },
        forceMoveViewToBottom: _moveViewToBottom
    };
};

module.exports = AnimatedBottomView;