"use strict";

function checkAskButtonCoachMark(button) {
    if (!require("services/action_record_service").get("askShown")) {
        if (button.declined) return false;
        var buttons = [ L("btn_cancel"), L("widget_future_best_match_btn_ask") ], cancelIndex = 0;
        buttons = [ L("widget_future_best_match_btn_ask"), L("btn_cancel") ];
        cancelIndex = 1;
        var dialog = Ti.UI.createAlertDialog({
            cancel: cancelIndex,
            buttonNames: buttons,
            title: L("widget_future_ask_help_title"),
            message: L("widget_future_ask_help_message")
        });
        dialog.addEventListener("click", function(e) {
            e.index !== cancelIndex && button.fireEvent("click", {});
        });
        dialog.show();
        require("services/action_record_service").set("askShown", true);
        return true;
    }
    return false;
}

function checkDeclineState(button) {
    if (button.declined) {
        var buttons = [ L("btn_ok") ], cancelIndex = 0;
        var dialog = Ti.UI.createAlertDialog({
            cancel: cancelIndex,
            buttonNames: buttons,
            title: String.format(L("widget_future_ask_declined_title"), L("RIDE" === button.trip.type ? "mode_rider" : "mode_driver")),
            message: L("widget_future_ask_declined_body")
        });
        dialog.show();
        return true;
    }
    return false;
}

var Globals = require("/v6/infrastructure/Globals"), bookingService = Globals.IOC.get("bookingService");

var buttonStyle = {
    width: "50dp",
    height: "44dp",
    normalColor: "transparent",
    selectedColor: "#70333333",
    backgroundImage: "none",
    borderRadius: Alloy.Globals.Style.Unit.unit3
};

var activityIndicatorStyle = {
    width: "20dp",
    height: "20dp",
    style: Ti.UI.ActivityIndicatorStyle.PLAIN,
    visible: false,
    zIndex: 3
};

var askButtonStyle = {
    width: Ti.UI.FILL,
    height: Ti.UI.FILL,
    title: L("widget_future_best_match_btn_ask"),
    font: {
        fontSize: "15dp"
    },
    color: Alloy.Globals.Style.Color.DarkGray,
    normalColor: "#00333333",
    selectedColor: "#70333333",
    backgroundImage: "none",
    zIndex: 2
};

var askedButtonStyle = {
    width: Ti.UI.FILL,
    height: Ti.UI.FILL,
    title: "c",
    font: {
        fontFamily: "splinico",
        fontSize: "16dp"
    },
    backgroundColor: Alloy.Globals.Style.Color.LightBlue,
    color: "#ffffff",
    normalColor: "#00333333",
    selectedColor: "#70333333",
    backgroundImage: "none",
    zIndex: 1
};

var declinedButtonStyle = {
    width: Ti.UI.FILL,
    height: Ti.UI.FILL,
    title: "1",
    font: {
        fontFamily: "splinico",
        fontSize: "16dp"
    },
    backgroundColor: Alloy.Globals.Style.Color.Grey,
    color: "#ffffff",
    normalColor: Alloy.Globals.Style.Color.Grey,
    selectedColor: Alloy.Globals.Style.Color.Grey,
    backgroundImage: "none",
    zIndex: 1
};

var askButtonBordersStyle = {
    borderColor: Alloy.Globals.Style.Color.DarkGray,
    borderWidth: Alloy.Globals.Style.Unit.unit2
};

var askedButtonBorderStyle = {
    borderWidth: 0,
    borderColor: "#fff"
};

var CustomButton = function(args) {
    args = _.extend(args, buttonStyle, askButtonBordersStyle);
    args.indicatorStyle && (activityIndicatorStyle.style = args.indicatorStyle);
    var wrapper = Ti.UI.createView(args), indicator = Ti.UI.createActivityIndicator(activityIndicatorStyle), button = Ti.UI.createButton(askButtonStyle);
    wrapper.add(indicator);
    wrapper.add(button);
    button.enabled = false != args.enabled && "false" != args.enabled;
    wrapper.opacity = button.enabled ? 1 : .5;
    var _markAsked = function() {
        button.asked = true;
        button.applyProperties(askedButtonStyle);
        wrapper.applyProperties(askedButtonBorderStyle);
    };
    var _markUnAsked = function() {
        button.asked = false;
        button.applyProperties(askButtonStyle);
        wrapper.applyProperties(askButtonBordersStyle);
    };
    var _markDeclined = function() {
        button.asked = true;
        button.declined = true;
        button.applyProperties(declinedButtonStyle);
        wrapper.applyProperties(askedButtonBorderStyle);
    };
    var press = function() {
        button.enabled && (wrapper.backgroundColor = wrapper.selectedColor);
    };
    var release = function() {
        if (button.enabled) {
            wrapper.backgroundColor = wrapper.normalColor;
            return;
        }
    };
    wrapper.addEventListener("touchstart", press);
    wrapper.addEventListener("touchend", release);
    wrapper.addEventListener("touchcancel", release);
    wrapper.addEventListener("touchmove", release);
    button.addEventListener("click", function() {
        if (checkAskButtonCoachMark(button)) return;
        if (checkDeclineState(button)) return;
        if (!button.asked) {
            var executor = require("utils/executor");
            var funcs = [];
            funcs.push(function() {
                button.animate({
                    opacity: 0,
                    duration: 300
                }, function() {
                    indicator.visible = true;
                });
            });
            funcs.push(function() {
                var service = require("services/booking_service");
                if (!button.bookedByUser && button.booking && "REQUESTED" === button.booking.state) return service.accept(button.booking.id, button.initiator.firstName).then(function() {
                    Ti.App.fireEvent("carma:booking.changed", button.trip);
                    bookingService.emit(bookingService.BOOKING_UPDATED);
                });
                var askeeType = button.trip.type.toUpperCase();
                var isAskeeRider = "RIDE" === askeeType || "RIDER" === askeeType;
                LOGGER.info("[ASK] : Making " + (isAskeeRider ? "ride offer" : "ride request"));
                return isAskeeRider ? service.rideOffer(button.trip, button.groupingKey) : service.rideRequest(button.trip, button.groupingKey);
            });
            funcs.push(function() {
                _markAsked();
                button.handleBooked && button.handleBooked();
            });
            executor(funcs).withProgress(false).error(function(e) {
                var errorMessage;
                if (e && "already_cancelled" === e.errorCode) {
                    errorMessage = L("widget_future_ask_already_cancelled");
                    _markDeclined();
                } else if (e && "request_already_exists" === e.errorCode) _markAsked(); else {
                    errorMessage = L(e && "invalid_booking_date" === e.errorCode ? "widget_future_ask_error_past" : "widget_future_ask_error");
                    _markUnAsked();
                }
                if (!errorMessage) return;
                Carma.UI.createAlertWindow({
                    title: L("error_title"),
                    message: errorMessage
                }).show();
            }).endWith(function() {
                if (true && button.asked) {
                    Ti.App.fireEvent("carma:booking.changed", button.trip);
                    bookingService.emit(bookingService.BOOKING_UPDATED);
                }
                button.animate({
                    opacity: 1,
                    duration: 300
                }, function() {
                    indicator.visible = false;
                });
                var event = require("/tracking/tracking_event");
                require("/tracking/tracking_manager").track(event.ASK_BY_BOOKING, {
                    targetUserId: button.trip.user.userId
                });
            }).run();
        }
    });
    wrapper.setData = function(data, fetchBooking) {
        fetchBooking = void 0 !== fetchBooking ? fetchBooking : true;
        var assert = require("utils/assert");
        assert.required(data.trip, [ "id", "origin", "destination", "type", "effectiveTime", "user" ]);
        button.asked = false;
        button.trip = data.trip;
        button.groupingKey = data.groupingKey;
        button.handleBooked = data.handleBooked;
        button.enabled = false != data.enabled && "false" != data.enabled;
        wrapper.opacity = button.enabled ? 1 : .5;
        if (wrapper.setBooking(data.booking)) return;
        if (!fetchBooking) {
            button.opacity = 1;
            return;
        }
        var service = require("services/booking_service");
        service.findByTrip(button.trip.id, false).then(function(data) {
            data && data.rideBookings && _.each(data.rideBookings, function(booking) {
                "REQUESTED" === booking.state && button.trip.effectiveTime === booking.bookingDate && (button.enabled || "RIDE" !== data.trip.type) && _markAsked();
            });
            button.animate({
                opacity: 1,
                duration: 150
            }, function() {
                button.opacity = 1;
            });
        });
    };
    wrapper.setBooking = function(booking) {
        if (!booking || !booking.sourceTrip || !booking.trip) return false;
        var userId = require("services/resource_owner_service").singleton().get("userId"), sourceTrip = booking.sourceTrip.baseTrip, sourceTripGroupingKey = sourceTrip.id + "-" + sourceTrip.schedule.baseEffectiveTime, trip = booking.trip.baseTrip, tripGroupingKey = trip.id + "-" + trip.schedule.baseEffectiveTime;
        button.booking = booking;
        button.initiator = booking[booking.initiator.toLowerCase()];
        button.bookedByUser = userId === button.initiator.userId;
        if (button.groupingKey === sourceTripGroupingKey || button.groupingKey === tripGroupingKey) {
            if ("DECLINED" === booking.state) {
                _markDeclined();
                return true;
            }
            if (button.bookedByUser) {
                if ("REQUESTED" === booking.state || "ACCEPTED" === booking.state) {
                    _markAsked();
                    return true;
                }
            } else if ("CANCELLED" === booking.state) {
                button.booking = null;
                return true;
            }
        }
        return false;
    };
    return wrapper;
};

exports.createView = function(args) {
    return new CustomButton(args);
};