"use strict";

var CustomButton = function(args) {
    var self = this;
    args = args || {};
    args.backgroundColor = args.normalColor;
    self.normalColor = args.normalColor;
    self.selectedColor = args.selectedColor;
    self.disableAnimation = args.disableAnimation;
    self.debounceTimeMS = args.debounceTimeMS || 1500;
    self.preventAutoCleanup = args.preventAutoCleanup || false;
    var eventsToRemove = [];
    var addEventListener = function(view, event, cb) {
        if (!view) return;
        eventsToRemove.push({
            view: view,
            event: event,
            cb: cb
        });
        view.addEventListener(event, cb);
    };
    var removeAllEventListeners = function() {
        eventsToRemove.forEach(function(ev) {
            ev.view.removeEventListener(ev.event, ev.cb);
        });
        eventsToRemove = [];
    };
    var view = Ti.UI.createView(args);
    view.enabled = false != view.enabled && "false" != view.enabled;
    var press = function() {
        if (!view) return;
        false !== view.enabled && false !== view.touchEnabled && (view.backgroundColor = self.selectedColor);
    };
    var release = function() {
        if (!view) return;
        if (false !== view.enabled && false !== view.touchEnabled) {
            if (self.disableAnimation) {
                view.backgroundColor = view.normalColor || self.normalColor;
                return;
            }
            view.animate({
                backgroundColor: view.normalColor || self.normalColor,
                duration: 300
            });
            setTimeout(function() {
                if (!view) return;
                view.backgroundColor = view.normalColor || self.normalColor;
            }, 300);
        }
    };
    addEventListener(view, "touchstart", press);
    addEventListener(view, "touchend", release);
    addEventListener(view, "touchcancel", release);
    addEventListener(view, "touchmove", release);
    view.setEnabled = function(flag) {
        if (!view) return;
        view.enabled = flag;
        view.opacity = flag ? 1 : .4;
        if (view.enabled) {
            if (self.disableAnimation) {
                view.backgroundColor = view.normalColor || self.normalColor;
                return;
            }
            view.animate({
                backgroundColor: view.normalColor || self.normalColor,
                duration: 300
            });
            setTimeout(function() {
                if (!view) return;
                view.backgroundColor = view.normalColor || self.normalColor;
            }, 300);
        }
    };
    var toggleChildren = function(view, enable) {
        if (!view) return;
        view.touchEnabled = enable;
        var children = view.getChildren(), childrenLength = view.getChildren().length;
        if (childrenLength) for (var i = 0; childrenLength > i; ++i) toggleChildren(children[i], enable);
    };
    view.toggleClick = function(enable) {
        if (!view) return;
        var children = view.getChildren();
        children.length ? toggleChildren(view, enable) : view.touchEnabled = enable;
    };
    view.setEnhancer = function(enchancer) {
        if (!view) return;
        addEventListener(enchancer, "touchstart", press);
        addEventListener(enchancer, "touchend", release);
        addEventListener(enchancer, "touchcancel", release);
        addEventListener(enchancer, "touchmove", release);
    };
    var label = null;
    if (args.title || args.caption) {
        label = Ti.UI.createLabel({
            text: "SIZE" === args.width || "size" === args.width ? " " + (args.title ? args.title : args.caption) + " " : args.title ? args.title : args.caption,
            textAlign: args.textAlign,
            font: args.font,
            color: args.color,
            width: Ti.UI.SIZE
        });
        args.padding && label.applyProperties({
            left: args.padding,
            right: args.padding
        });
        view.add(label);
        view.setTitle = view.setCaption = function(text) {
            if (!view) return;
            label.text = text;
        };
        view.setFont = function(font) {
            if (!view) return;
            label.font = font;
        };
        view.setColor = function(color) {
            if (!view) return;
            label.color = color;
        };
    }
    if (args.image) {
        var img = Ti.UI.createImageView({
            image: args.image,
            width: args.imageWidth,
            height: args.imageHeight
        });
        view.add(img);
        img = null;
    }
    var _indicator = null;
    if (args.indicator) {
        var size = parseInt(view.height), indicatorStyle = Ti.UI.ActivityIndicatorStyle.PLAIN;
        args.indicatorStyle && (indicatorStyle = args.indicatorStyle);
        _indicator = Ti.UI.createActivityIndicator({
            width: size,
            height: size,
            style: indicatorStyle
        });
        view.add(_indicator);
        var hidedSubviews = [];
        var showIndicator = function(indicator) {
            if (!view) return;
            args.hideSubviewsWhenShowIndicator && view && view.children && _.each(view.children, function(subview) {
                if (-1 === subview.toString().indexOf("TiUIActivityIndicator") && (void 0 === subview.visible || "false" !== subview.visible.toString())) {
                    subview.hide();
                    hidedSubviews.push(subview);
                }
            });
            indicator.show();
        };
        var hideIndicator = function(indicator) {
            if (!view) return;
            if (args.hideSubviewsWhenShowIndicator && view && hidedSubviews && hidedSubviews.length > 0) {
                _.each(hidedSubviews, function(subview) {
                    -1 === subview.toString().indexOf("TiUIActivityIndicator") && subview.show();
                });
                hidedSubviews = [];
            }
            indicator.hide();
        };
        addEventListener(view, "click", function() {
            if (!view) return;
            if (view.enabled) {
                view.toggleClick(false);
                -1 !== self.debounceTimeMS && setTimeout(function() {
                    if (!view) return;
                    view.setEnabled(true);
                    view.toggleClick(true);
                }, self.debounceTimeMS);
                label ? label.animate({
                    opacity: 0,
                    duration: 300
                }, function() {
                    if (!view) return;
                    showIndicator(_indicator);
                }) : showIndicator(_indicator);
            }
        });
        view.stopIndicator = function() {
            if (!view) return;
            label ? label.animate({
                opacity: 1,
                duration: 300
            }, function() {
                if (!view) return;
                hideIndicator(_indicator);
            }) : hideIndicator(_indicator);
        };
    } else addEventListener(view, "click", function() {
        if (!view) return;
        view.toggleClick(false);
        -1 !== self.debounceTimeMS && setTimeout(function() {
            if (!view) return;
            view.setEnabled(true);
            view.toggleClick(true);
        }, self.debounceTimeMS);
    });
    view.setNormalColor = function(normalColor) {
        if (!view) return;
        self.normalColor = normalColor;
    };
    view.setSelectedColor = function(selectedColor) {
        if (!view) return;
        self.selectedColor = selectedColor;
    };
    view.setDisableAnimation = function(disableAnimation) {
        if (!view) return;
        self.disableAnimation = disableAnimation;
    };
    view.setStyle = function(properties) {
        if (!view) return;
        for (var key in properties) {
            var funcKey = "set" + key[0].toUpperCase() + key.slice(1), func = view[funcKey];
            func && _.isFunction(func) && func.call(view, properties[key]);
            view[key] = properties[key];
        }
    };
    view.cleanup = function() {
        removeAllEventListeners();
        view = null;
        label = null;
        _indicator = null;
    };
    var parentWindow = null;
    var onWindowClose = function() {
        parentWindow && parentWindow.removeEventListener("close", onWindowClose);
        removeAllEventListeners();
        view = null;
        label = null;
        _indicator = null;
        parentWindow = null;
    };
    self.preventAutoCleanup || view.addEventListener("postlayout", function onPostLayout() {
        view.removeEventListener("postlayout", onPostLayout);
        parentWindow = require("/utils/ui").getWindow(view);
        if (!parentWindow) {
            LOGGER.error("custom_button could not find window, will probably leak events");
            return;
        }
        parentWindow.addEventListener("close", onWindowClose);
    });
    view.buttonCleanUp = function() {
        parentWindow && parentWindow.removeEventListener("close", onWindowClose);
        removeAllEventListeners();
        view = null;
        label = null;
        _indicator = null;
    };
    return view;
};

exports.createButton = function(args) {
    return new CustomButton(args);
};