"use strict";

var CustomScrollView = function(args) {
    var _ptrView = null, _scrollView = null;
    var _pulling = false, _loading = false, _pulledTimestamp = 0, _pulledThreadhold = 50, _androidTouchStartY = null, _androidMinTouchDistance = require("/platform/units").dipToSystemUnit(125), _androidProgress = 0, _offset = 0;
    var _init = function() {
        _ptrView = _createPtrView();
        _scrollView = Ti.UI.createScrollView(_.extend(args, {
            layout: "vertical"
        }));
        _scrollView.overScrollMode = Ti.UI.Android.OVER_SCROLL_NEVER;
        _scrollView.add(_ptrView);
    };
    var _initEventListener = function() {
        var _scrollHandler = _androidScroll;
        _scrollView.addEventListener("scroll", _scrollHandler);
        _scrollView.addEventListener("touchmove", _androidTouchMove);
        _scrollView.addEventListener("touchend", _androidTouchEnd);
        _scrollView.addEventListener("touchcancel", _androidTouchEnd);
    };
    var _exports = function() {
        _scrollView.refresh = function() {
            _loading = true;
            _scrollView.load && _scrollView.load();
        };
        _scrollView.loaded = function() {
            _loading = false;
            _ptrView.visible = true;
            setTimeout(function() {
                _ptrView.showLoading(false);
            }, 100);
        };
    };
    var _iosScroll = function(e) {
        if (e.y > 0) return;
        if (_loading) return;
        _offset = e.y;
        _pulling = 0 > _offset && _offset > -50;
        if (_pulling && _offset > -50) _ptrView.unrotateArrow(); else if (!_pulling && -50 >= _offset) {
            _ptrView.rotateArrow();
            _pulledTimestamp = Date.now();
        }
    };
    var _iosDragEnd = function() {
        if (-50 >= _offset && Date.now() - _pulledTimestamp > _pulledThreadhold) {
            _ptrView.showLoading(true);
            _scrollView.setContentOffset({
                x: 0,
                y: -50
            });
            _scrollView.refresh();
        }
    };
    var _androidScroll = function(e) {
        if (_loading) return;
        _offset = e.y;
    };
    var _androidTouchMove = function(e) {
        if (_loading) return;
        if (_offset > 0) {
            _ptrView.clearProgress();
            return;
        }
        if (e.source !== _scrollView) return;
        if (null === _androidTouchStartY) {
            _androidTouchStartY = e.y;
            return;
        }
        var offset = e.y - _androidTouchStartY;
        if (offset > 0) {
            _androidProgress = Math.min(offset / _androidMinTouchDistance * 100, 100);
            _ptrView.showProgress(_androidProgress + "%");
        }
    };
    var _androidTouchEnd = function() {
        if (_loading) return;
        _androidTouchStartY = null;
        if (_offset > 0 || 100 > _androidProgress) {
            _ptrView.clearProgress();
            return;
        }
        _ptrView.showLoading(true);
        _scrollView.refresh();
    };
    var _createPtrView = function() {
        var container = null;
        var ptrArrow;
        var ptrIndicator;
        container = Ti.UI.createView({
            top: 0,
            height: Ti.UI.SIZE,
            width: Ti.UI.FILL,
            layout: "vertical",
            visible: false
        });
        var bar = Ti.UI.createView({
            top: 0,
            height: "3dp",
            width: 0,
            backgroundColor: args.loadingIndicatorColor
        });
        var loadingContainer = Ti.UI.createView({
            top: 0,
            height: 0,
            width: Ti.UI.FILL
        });
        var loadingIndicator = Ti.UI.createActivityIndicator({
            style: Ti.UI.ActivityIndicatorStyle.DARK
        });
        loadingContainer.add(loadingIndicator);
        container.add(bar);
        container.add(loadingContainer);
        container.showProgress = function(width) {
            bar.width = width;
        };
        container.clearProgress = function() {
            _androidProgress = 0;
            bar.width = 0;
        };
        container.showLoading = function(isLoading) {
            if (isLoading) {
                container.clearProgress();
                loadingContainer.height = "50dp";
                loadingIndicator.show();
            } else {
                loadingIndicator.hide();
                loadingContainer.height = 0;
            }
        };
        return container;
    };
    _init();
    _initEventListener();
    _exports();
    return _scrollView;
};

exports.createScrollView = function(args) {
    return new CustomScrollView(args);
};