"use strict";

var eventDomains = {};

exports.setEventDomain = function(event, domain) {
    eventDomains[event] = domain;
};

exports.createEventManager = function() {
    return {
        manageEvents: function(context, theObject) {
            LOGGER.debug("[EH] Managing events for " + context);
            var appEventListeners = [];
            theObject.addAppEventListener = function(event, callback) {
                LOGGER.debug("[EH] Adding " + context + " Event Handler: " + event);
                var domain = process.domain, options = options || {}, debounceWait = options.debounceWait || 100, willDebounce = !!options.debounce;
                var wrappedCallback = function(data) {
                    eventDomains[event] && (domain = eventDomains[event]);
                    LOGGER.debug("[EH] Calling " + context + " " + event + " event handler in domain " + domain.getName());
                    TM.push(function() {
                        willDebounce ? _.debounce(callback, debounceWait)(data) : callback(data);
                    }, domain);
                };
                Ti.App.addEventListener(event, wrappedCallback);
                appEventListeners.push({
                    event: event,
                    callback: wrappedCallback,
                    rawCallback: callback
                });
                LOGGER.debug("[EH] " + context + " Event Handlers: " + appEventListeners.length);
            };
            theObject.removeAppEventListener = function(event, callback) {
                var item = _.findWhere(appEventListeners, {
                    event: event,
                    rawCallback: callback
                });
                if (item) {
                    LOGGER.debug("[EH] Unregistering " + context + " " + event + " event handler");
                    Ti.App.removeEventListener(item.event, item.callback);
                    appEventListeners = _.without(appEventListeners, item);
                    LOGGER.debug("[EH] " + context + " Event Handlers: " + appEventListeners.length);
                } else {
                    LOGGER.debug("[EH] Could not find " + context + " " + event + " event handler");
                    LOGGER.debug("[EH] " + context + " Event Handlers: " + appEventListeners.length);
                }
            };
            theObject.removeAllAppEventListeners = function() {
                _.each(appEventListeners, function(item) {
                    LOGGER.debug("[EH] Unregistering " + context + " " + item.event + " event handler");
                    Ti.App.removeEventListener(item.event, item.callback);
                });
                appEventListeners = [];
                LOGGER.debug("[EH] " + context + " Event Handlers: " + appEventListeners.length);
            };
            theObject = null;
        }
    };
};