"use strict";

var _askForFeedback = function() {
    var trackingEvent = require("/tracking/tracking_event"), analytics = require("/tracking/tracking_manager");
    analytics.track(trackingEvent.FEEDBACK_REQUEST_SHOWN);
    var alertDialog = Titanium.UI.createAlertDialog({
        title: L("widget_appstore_feedback_title"),
        message: L("widget_appstore_feedback_question"),
        buttonNames: [ L("btn_no"), L("btn_yes") ],
        cancel: 1
    });
    alertDialog.addEventListener("click", function(evt) {
        switch (evt.index) {
          case 1:
            _runAppStoreReview();
            break;

          case 0:
            _askForEmailFeedback();
        }
    });
    alertDialog.show();
};

var _askForEmailFeedback = function() {
    var alertDialog = Titanium.UI.createAlertDialog({
        title: L("widget_appstore_negative_feedback_title"),
        message: L("widget_appstore_negative_feedback_question"),
        buttonNames: [ L("widget_appstore_btn_no_thanks"), L("widget_appstore_btn_sure") ],
        cancel: 1
    });
    Ti.App.Properties.setString("RemindToRate", Number.MAX_VALUE);
    Ti.App.Properties.getBool("reminderSet", false);
    var lastVersionRated = Ti.App.Properties.getInt("carma.lastVersionRated", 0);
    var currentVersion = Ti.App.Properties.getInt("carma.revision");
    Ti.App.Properties.setInt("carma.lastVersionRated", currentVersion);
    alertDialog.addEventListener("click", function(evt) {
        switch (evt.index) {
          case 1:
            _showEmail();
            break;

          case 0:        }
    });
    alertDialog.show();
};

var _showEmail = function() {
    var trackingEvent = require("/tracking/tracking_event"), analytics = require("/tracking/tracking_manager");
    var emailDialog = Ti.UI.createEmailDialog();
    emailDialog.html = true;
    emailDialog.subject = L("widget_appstore_feedback_email_subject");
    emailDialog.toRecipients = [ "feedback@car.ma" ];
    emailDialog.messageBody = L("widget_appstore_feedback_email_content");
    emailDialog.open();
    emailDialog.addEventListener("complete", function(e) {
        Ti.UI.Android.hideSoftKeyboard();
        e.result === emailDialog.CANCELLED && TM.push(function() {
            analytics.track(event.FEEDBACK_CANCELLED, {
                source: "email"
            });
        });
        e.result === emailDialog.SENT && TM.push(function() {
            analytics.track(event.FEEDBACK_SENT, {
                source: "email"
            });
        });
    });
};

var _runAppStoreReview = function() {
    var trackingEvent = require("/tracking/tracking_event"), analytics = require("/tracking/tracking_manager");
    analytics.track(trackingEvent.APP_STORE_REVIEW_SHOWN);
    var alertDialog = Titanium.UI.createAlertDialog({
        title: L("widget_appstore_review_dialog_title"),
        message: L("widget_appstore_review_dialog_message"),
        buttonNames: [ L("widget_appstore_review_you_got_it"), L("widget_appstore_review_remind"), L("widget_appstore_review_no") ],
        cancel: 0
    });
    alertDialog.addEventListener("click", function(evt) {
        switch (evt.index) {
          case 0:
            analytics.track(trackingEvent.APP_STORE_REVIEW_OK);
            Ti.App.Properties.setString("RemindToRate", Number.MAX_VALUE);
            Ti.App.Properties.getBool("reminderSet", false);
            Ti.Platform.openURL("https://play.google.com/store/apps/details?id=com.avego.avegodriver");
            var lastVersionRated = Ti.App.Properties.getInt("carma.lastVersionRated", 0);
            var currentVersion = Ti.App.Properties.getInt("carma.revision");
            Ti.App.Properties.setInt("carma.lastVersionRated", currentVersion);
            break;

          case 1:
            analytics.track(trackingEvent.APP_STORE_REVIEW_REMIND);
            Ti.App.Properties.setBool("reminderSet", true);
            var now = new Date().getTime();
            Ti.App.Properties.setString("RemindToRate", now + 6048e5);
            break;

          case 2:
            Ti.App.Properties.setString("RemindToRate", Number.MAX_VALUE);
            Ti.App.Properties.getBool("reminderSet", false);
            analytics.track(trackingEvent.APP_STORE_REVIEW_NEVER);
            var lastVersionRated = Ti.App.Properties.getInt("carma.lastVersionRated", 0);
            var currentVersion = Ti.App.Properties.getInt("carma.revision");
            Ti.App.Properties.setInt("carma.lastVersionRated", currentVersion);
        }
    });
    alertDialog.show();
};

var _showReviewOptions = function() {
    var now = new Date().getTime();
    var remindToRate = Ti.App.Properties.getString("RemindToRate");
    var ask = false;
    if (remindToRate) now > remindToRate && (ask = true); else {
        Ti.App.Properties.setString("RemindToRate", now);
        ask = true;
    }
    ask = true;
    ask && (Ti.App.Properties.getBool("reminderSet") ? _runAppStoreReview() : _askForFeedback());
};

module.exports = {
    checkAndPresentRating: _showReviewOptions
};