"use strict";

var Q = require("/vendor/q");

var styler = function($pointer, options) {
    options = options || {};
    this.$ = $pointer;
    this.waitUntilUpdated = !!options.waitUntilUpdated;
    this.strictCheck = !!options.strictCheck;
    this.elementNeedToUpdateCount = 0;
};

_.extend(styler.prototype, {
    $: null,
    waitUntilUpdated: false,
    strictCheck: false,
    deferred: null,
    elementNeedToUpdateCount: 0,
    checkAllUpdated: function() {
        this.elementNeedToUpdateCount < 1 && this.deferred.resolve();
    },
    startSetStyles: function(css) {
        this.startSetStylesTask(css);
        if (this.waitUntilUpdated) {
            this.deferred = Q.defer();
            this.checkAllUpdated();
            return this.deferred.promise;
        }
    },
    startSetStylesTask: function(css) {
        if (!css) return;
        for (var key in css) this.isId(key) && this.startSetStylesForIdkey(css[key], key);
    },
    startSetStylesForIdkey: function(root, idkey) {
        if (!root && !id) return;
        var pros = {}, classes = "";
        if (this.isNested(root)) for (var key in root) this.isId(key) ? this.startSetStylesForIdkey(root[key], key) : this.isClasses(key) ? classes = classes.length > 0 ? classes + " " + root[key] : root[key] : pros[key] = root[key]; else _.isString(root) && (classes = root);
        var obj = this.$[this.getId(idkey)];
        obj && this.applyProperties(obj, this.createStyle(obj.getApiName(), classes, pros));
    },
    createStyle: function(apiName, classes, pros) {
        pros = pros || {};
        return this.$.createStyle(_.extend({
            classes: classes,
            apiName: apiName
        }, pros));
    },
    strictChecker: function(obj, pros) {
        var pass = true;
        for (var key in pros) if (obj[key] !== pros[key] && "classes" !== key && "apiName" !== key) {
            pass = false;
            break;
        }
        return pass;
    },
    watchOverUpdate: function(obj, pros) {
        if (obj && obj.addEventListener && !this.strictChecker(obj, pros)) {
            this.elementNeedToUpdateCount++;
            var self = this;
            obj.addEventListener("postlayout", function postlayoutWatcher() {
                if (!self.strictCheck || self.strictChecker(obj, pros)) {
                    obj.removeEventListener("postlayout", postlayoutWatcher);
                    self.elementNeedToUpdateCount--;
                    self.checkAllUpdated();
                }
            });
        }
    },
    applyProperties: function(obj, pros) {
        if (obj && obj.applyProperties && _.isFunction(obj.applyProperties)) {
            this.waitUntilUpdated && this.watchOverUpdate(obj, pros);
            obj.applyProperties(pros);
        }
    },
    getId: function(key) {
        return key.substring(1);
    },
    getApiName: function(key) {
        return key.substring(key.indexOf("[") + 1, key.indexOf("]"));
    },
    isId: function(obj) {
        return obj && _.isString(obj) && /^#.+$/.test(obj);
    },
    isClasses: function(obj) {
        return obj && _.isString(obj) && "classes" === obj;
    },
    isNested: function(obj) {
        return obj && _.isObject(obj);
    }
});

module.exports.setStyle = function($pointer, css, options) {
    if (!$pointer || !css) {
        LOGGER.error("[CSS] - missing required parameters!");
        return;
    }
    var _styler = new styler($pointer, options);
    return _styler.startSetStyles(css);
};