"use strict";

function addLinkText(label, linkText) {
    var lbLink = Ti.UI.createLabel({
        text: L("widget_messenger_link_mark") + " " + linkText,
        color: "#ccc",
        bottom: "5dp",
        left: "5dp",
        right: "5dp",
        font: {
            fontSize: "12dp"
        },
        width: Ti.UI.FILL,
        height: "16dp",
        touchEnabled: false
    });
    _labelContainer && _labelContainer.add(lbLink);
}

function openLink(e) {
    if (e && e.source && e.source.link) {
        var linkUrl = e.source.link;
        Ti.Platform.openURL(require("alloy/string").trim(linkUrl));
    }
}

function buildLinkAttributedString(text, linkText) {
    var attr = null, color = "blue";
    text && linkText && (attr = text.replace(linkText, '<font color="' + color + '">' + linkText + "</font>"));
    return attr;
}

var getText = function(attachTo) {
    return attachTo.text || attachTo.value;
};

var expURL = "^|s((https?://)[w-]+(.[w-]+)+.?(:d+)?(/S*)?)";

var expInlineURL = "(https?://[^S]+)";

var expHref = '<a href="([^"]+)"[^<]*>([^</]*)</a>';

var _labelContainer = null;

exports.attachLink = function(attachTo, container) {
    if (!attachTo || !getText(attachTo)) return;
    _labelContainer = container;
    if (handleHref(attachTo) || handleURL(attachTo)) {
        _labelContainer.link = attachTo.link;
        _labelContainer.addEventListener("click", openLink);
    }
};

exports.clearLinkText = function(label) {
    if (!label || !getText(label)) return;
    handleURL(label, true);
    handleHref(label, true);
};

var handleURL = function(attachTo, noEvent) {
    var text = getText(attachTo), links = text.match(new RegExp(expURL));
    if (!links) return false;
    if (!noEvent) {
        var link = links[0];
        attachTo.link = require("alloy/string").trim(link);
        addLinkText(attachTo, link);
    }
    return true;
};

exports.convertHrefToText = function(text) {
    if (!text) return text;
    var links = text.match(new RegExp(expHref));
    if (!links) return text;
    return text.replace(new RegExp(expHref, "g"), links[2]);
};

exports.getLink = function(text) {
    var result = {
        href: "",
        text: ""
    }, elements = text.match(new RegExp(expHref)) || [];
    if (3 == elements.length) {
        result.href = require("alloy/string").trim(elements[1]);
        result.text = elements[2].trim();
    } else {
        elements = text.match(new RegExp(expInlineURL)) || [];
        2 == elements.length && (result.href = require("alloy/string").trim(elements[1]));
    }
    return result;
};

var handleHref = function(attachTo, noEvent) {
    var text = getText(attachTo), links = text.match(new RegExp(expHref));
    if (!links) return false;
    var linkHref = links[1], linkText = links[2];
    attachTo.text ? attachTo.text = text.replace(new RegExp(expHref, "g"), linkText) : attachTo.value = text.replace(new RegExp(expHref, "g"), linkText);
    if (!noEvent) {
        attachTo.link = require("alloy/string").trim(linkHref);
        addLinkText(attachTo, linkText);
    }
    return true;
};

exports.makeUnderlinedLink = function(label) {
    Alloy.Globals.setUnderline(label, true);
};