"use strict";

var _needStartService = true, enabledGeofences = false;

var LocationAccuracyLevel = {
    LocationAccuracyLevelLow: 0,
    LocationAccuracyLevelMedium: 1,
    LocationAccuracyLevelHigh: 2
};

var helper = {
    gotLastKnownLocation: function() {
        _needStartService && this._startGeofencesService();
    },
    startGeofencesBaseOnTrips: function() {
        var gps, resourceOwner, myUserId, myToken;
    },
    startGeofences: function() {
        _needStartService && this._startGeofencesService();
    },
    _startGeofencesService: function() {
        try {
            enabledGeofences = require("featuretoggles/featuretoggles").getValue("enableGeofences");
            if (!enabledGeofences) return;
            var locationService = require("ma.car.ti.module.gps"), resourceOwner = require("/services/resource_owner_service").singleton(), myUserId = resourceOwner.get("userId"), myToken = resourceOwner.get("token");
            if (!locationService) return;
            enabledGeofences ? Alloy.createModel("geofences").fetch().then(function(data) {
                _needStartService && locationService.monitorZones({
                    userId: myUserId,
                    userToken: myToken.token,
                    apiUrl: require("bootstrap/env").getApiURL(),
                    geofences: data,
                    debug: enabledGeofences,
                    success: function() {
                        alert("monitorZones started!");
                        _needStartService = false;
                    }
                });
            }).catch(function(e) {
                LOGGER.error("[geofences] - geofences error: " + JSON.stringify(e));
            }) : this.shutdownGeofences();
        } catch (e) {
            LOGGER.error("Error initialising geofences: " + JSON.stringify(e));
        }
    },
    startLocationMonitoring: function(uniqueId) {
        if (enabledGeofences) try {
            var locationService = require("ma.car.ti.module.gps");
            locationService && locationService.startLocationMonitoring({
                accuracyLevel: LocationAccuracyLevel.LocationAccuracyLevelHigh
            });
        } catch (e) {
            LOGGER.error("[geofences helper] startLocationMonitoring error: " + JSON.stringify(e));
        }
    },
    stopLocationMonitoring: function(uniqueId) {
        try {
            var locationService = require("ma.car.ti.module.gps");
            locationService && locationService.stopLocationMonitoring({});
        } catch (e) {
            LOGGER.error("[geofences helper] stopLocationMonitoring error: " + JSON.stringify(e));
        }
    },
    shutdownGeofences: function() {
        try {
            var locationService = require("ma.car.ti.module.gps");
            if (!locationService) return;
            locationService.stopMonitoringAllZones();
            _needStartService = true;
        } catch (e) {
            LOGGER.error("Error shutting down geofences: " + JSON.stringify(e));
        }
    }
};

module.exports = helper;