"use strict";

var beaconManager = null;

var BeaconRideSyncingState = {
    RideSyncingStateUnknown: 0,
    RideSyncingStateBeaconNotFound: 1,
    RideSyncingStateCorruptData: 2,
    RideSyncingStateSuccess: 3,
    RideSyncingStateFailed: 4
};

var helper = {
    boardingWidget: null,
    boot: function(boardingWidget) {
        if (this._isFeatureEnabled()) {
            var myBeacons = null, self = this;
            this.boardingWidget = boardingWidget;
            require("/services/beacons_service").list().then(function(beacons) {
                myBeacons = beacons.beacons;
            }).catch(function(e) {}).finally(function() {
                self._startBeaconMonitoring(myBeacons);
            });
        }
    },
    _startBeaconMonitoring: function(beacons) {
        if (!beaconManager) return;
        var myBeacons = beacons || [], resourceOwner = require("/services/resource_owner_service").singleton(), myUserId = resourceOwner.get("userId"), myToken = resourceOwner.get("token"), self = this;
        beaconManager.startService({
            apiURL: require("bootstrap/env").getApiURL(),
            userId: myUserId,
            userToken: "Bearer " + myToken.token,
            beacons: this._formatBeacons(beacons),
            debug: require("featuretoggles/featuretoggles").getValue("enableProximityBoardingLogs", 0),
            boarding: function(beacon) {
                var beaconId = self._composeBeaconId(beacon), isSynced = beacon.rideSyncingState === BeaconRideSyncingState.RideSyncingStateSuccess;
                beaconId && (self._isMyBeacon(myBeacons, beaconId) ? self.boardingWidget.startSoloDriveByBeacon(beaconId, beacon.location, isSynced) : self.boardingWidget.joinCarByBeacon(beaconId, beacon.location, isSynced));
            },
            unboarding: function(beacon) {
                var beaconId = self._composeBeaconId(beacon), isSynced = beacon.rideSyncingState === BeaconRideSyncingState.RideSyncingStateSuccess;
                beaconId && (self._isMyBeacon(myBeacons, beaconId) ? self.boardingWidget.endSoloDriveByBeacon(beaconId, beacon.location, isSynced) : self.boardingWidget.leaveCarByBeacon(beaconId, beacon.location, isSynced));
            },
            connecting: function(beacon) {
                var beaconId = self._composeBeaconId(beacon);
                alert("connecting beacon: " + beaconId);
            },
            fail: function(error) {
                LOGGER.error("[beacon] - error: " + JSON.stringify(error));
            },
            JoinCarPrompt: L("module_beacon_proximity_join_car"),
            LeaveCarPrompt: L("module_beacon_proximity_leave_car"),
            StartSoloDrivePrompt: L("module_beacon_proximity_start_solo_trip"),
            EndSoloDrivePrompt: L("module_beacon_proximity_end_solo_trip"),
            LowCreditPrompt: L("module_beacon_proximity_low_credit"),
            DriverNotAvailablePrompt: L("module_beacon_proximity_driver_not_available")
        });
    },
    _isFeatureEnabled: function() {
        return require("featuretoggles/featuretoggles").getValue("enableProximityBoarding", 0) && false;
    },
    _formatBeacons: function(beacons) {
        var _beacons = [];
        if (beacons) for (var i = 0; i < beacons.length; i++) {
            var beacon = beacons[i];
            _beacons.push(beacon.id + "");
        }
        return _beacons;
    },
    _isMyBeacon: function(myBeacons, beaconId) {
        if (!myBeacons) return false;
        var myBeacon = _.find(myBeacons, function(beacon) {
            return beacon.id === beaconId;
        });
        return !!myBeacon;
    },
    _composeBeaconId: function(beacon) {
        if (beacon && beacon.proximityUUID && beacon.major && beacon.minor) return beacon.proximityUUID + "-" + beacon.major + "-" + beacon.minor;
        return null;
    },
    setupActiveTrip: function(param, isSoloDrive) {
        if (!beaconManager || !param || !this._isFeatureEnabled()) return;
        var self = this, _beaconManager = beaconManager;
        var setup = function() {
            param.beaconIds && param.beaconIds.length > 0 && !isSoloDrive && _beaconManager.setupActiveRide(_.extend({
                isSoloDrive: !!isSoloDrive,
                beaconIds: param.beaconIds
            }, param));
            param = null;
            _beaconManager = null;
        };
        if (!param.beaconIds && param.driverId) {
            var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
            ProfileService.getProfile(param.driverId, ProfileService.FULL_DETAIL).then(function(profile) {
                param.beaconIds = self._formatBeacons(profile.beacons);
            }).catch(function(e) {}).finally(function() {
                setup();
            });
        } else setup();
    },
    stopService: function() {
        beaconManager && beaconManager.stopService();
    }
};

module.exports = helper;