"use strict";

var q = require("vendor/q");

var _handle404 = function(e) {
    if (e && e.status && "404" === e.status.toString()) {
        LOGGER.error("[Executor] 404: " + e);
        Carma.UI.createAlertWindow({
            title: L("error_title"),
            message: L("generic_error_message")
        }).show();
        return true;
    }
    return false;
};

module.exports = function(funcs) {
    funcs = _.isArray(funcs) ? funcs : [].concat(funcs);
    return {
        _withProgress: true,
        run: function() {
            var self = this;
            var promise = q.fcall(function() {
                self._withProgress && Ti.App.fireEvent("carma:progress.start");
            });
            funcs.forEach(function(f) {
                promise = promise.then(f);
            });
            promise = promise.catch(function(e) {
                LOGGER.warn(e);
                if (_handle404(e)) {
                    self._catchFunc && self._catchFunc(e);
                    return;
                }
                throw e;
            });
            self._catchFunc && (promise = promise.catch(self._catchFunc));
            promise = promise.catch(function(e) {
                LOGGER.error(e);
                Carma.UI.createAlertWindow({
                    title: L("error_title"),
                    message: self._errorMessage ? self._errorMessage : "Something went wrong..."
                }).show();
            });
            promise.finally(function() {
                self._endFunc && self._endFunc();
                self._withProgress && Ti.App.fireEvent("carma:progress.stop");
            });
        },
        error: function(func) {
            this._catchFunc = func;
            return this;
        },
        endWith: function(func) {
            this._endFunc = func;
            return this;
        },
        errorMessage: function(errorMessage) {
            this._errorMessage = errorMessage;
            return this;
        },
        withProgress: function(flag) {
            this._withProgress = flag;
            return this;
        }
    };
};