"use strict";

function encodeNumber(num) {
    var encodeString = "";
    while (num >= 32) {
        encodeString += String.fromCharCode((32 | 31 & num) + 63);
        num >>= 5;
    }
    encodeString += String.fromCharCode(num + 63);
    return encodeString;
}

function encodeSignedNumber(num) {
    var sgn_num = num << 1;
    0 > num && (sgn_num = ~sgn_num);
    return encodeNumber(sgn_num);
}

function encodePoint(plat, plng, lat, lng) {
    var late5 = Math.round(1e5 * lat);
    var plate5 = Math.round(1e5 * plat);
    var lnge5 = Math.round(1e5 * lng);
    var plnge5 = Math.round(1e5 * plng);
    var dlng = lnge5 - plnge5;
    var dlat = late5 - plate5;
    return encodeSignedNumber(dlat) + encodeSignedNumber(dlng);
}

function createEncodings(coords) {
    var i = 0;
    var plat = 0;
    var plng = 0;
    var encoded_points = "";
    for (i = 0; i < coords.length; ++i) {
        var lat = coords[i].latitude;
        var lng = coords[i].longitude;
        encoded_points += encodePoint(plat, plng, lat, lng);
        plat = lat;
        plng = lng;
    }
    return encoded_points;
}

var decodePolylines = function(encoded) {
    var len = encoded.length;
    var index = 0;
    var array = [];
    var lat = 0;
    var lng = 0;
    while (len > index) {
        var b;
        var shift = 0;
        var result = 0;
        do {
            b = encoded.charCodeAt(index++) - 63;
            result |= (31 & b) << shift;
            shift += 5;
        } while (b >= 32);
        var dlat = 1 & result ? ~(result >> 1) : result >> 1;
        lat += dlat;
        shift = 0;
        result = 0;
        do {
            b = encoded.charCodeAt(index++) - 63;
            result |= (31 & b) << shift;
            shift += 5;
        } while (b >= 32);
        var dlng = 1 & result ? ~(result >> 1) : result >> 1;
        lng += dlng;
        array.push([ 1e-5 * lat, 1e-5 * lng ]);
    }
    return array;
};

module.exports = {
    encode: createEncodings,
    decode: decodePolylines
};