"use strict";

var gps = require("gps/gps_manager"), locationValidator = require("/validator/validator_factory").currentLocation();

var checkLocation = function(NWPoint, SEPoint, ignoreTimestamp) {
    var lastLocation = gps.lastLocation();
    var isValid = locationValidator.isGPSPositionValid(lastLocation, ignoreTimestamp);
    if (!isValid) return;
    return lastLocation.lat <= NWPoint.lat && lastLocation.lat >= SEPoint.lat && lastLocation.lon >= NWPoint.lon && lastLocation.lon <= SEPoint.lon;
};

module.exports = {
    AUSTIN: "Austin",
    BERGEN: "Bergen",
    SFBAY: "SFBay",
    CORK: "Cork",
    checkPilotArea: function() {
        var lastLocation = gps.lastLocation();
        var isValid = locationValidator.isGPSPositionValid(lastLocation, true);
        if (!isValid) return;
        var sfbay = {
            name: this.SFBAY,
            NWPoint: {
                lat: 39.451,
                lon: -123.811
            },
            SEPoint: {
                lat: 36.837,
                lon: -120.74
            }
        };
        var austin = {
            name: this.AUSTIN,
            NWPoint: {
                lat: 31.065,
                lon: -98.53
            },
            SEPoint: {
                lat: 29.662,
                lon: -96.95
            }
        };
        var bergen = {
            name: this.BERGEN,
            NWPoint: {
                lat: 60.46,
                lon: 4.952236
            },
            SEPoint: {
                lat: 60.304,
                lon: 5.588757
            }
        };
        var cork = {
            name: this.CORK,
            NWPoint: {
                lat: 51.94,
                lon: -8.575
            },
            SEPoint: {
                lat: 51.676,
                lon: -8.297
            }
        };
        var pilots = [ sfbay, austin, bergen, cork ];
        for (var i = 0; i < pilots.length; i++) {
            var pilotCity = pilots[i];
            if (lastLocation.lat <= pilotCity.NWPoint.lat && lastLocation.lat >= pilotCity.SEPoint.lat && lastLocation.lon >= pilotCity.NWPoint.lon && lastLocation.lon <= pilotCity.SEPoint.lon) return pilotCity.name;
        }
        return null;
    },
    checkWithinSFBay: function() {
        var NWPoint = {
            lat: 38.999989,
            lon: -123.920754
        };
        var SEPoint = {
            lat: 36.595697,
            lon: -120.987405
        };
        var lastLocation = gps.lastLocation();
        var isValid = locationValidator.isGPSPositionValid(lastLocation);
        if (!isValid) return;
        return lastLocation.lat <= NWPoint.lat && lastLocation.lat >= SEPoint.lat && lastLocation.lon >= NWPoint.lon && lastLocation.lon <= SEPoint.lon;
    },
    checkWithinAustin: function() {
        var NWPoint = {
            lat: 31.065,
            lon: -98.53
        };
        var SEPoint = {
            lat: 29.662,
            lon: -96.95
        };
        var lastLocation = gps.lastLocation();
        var isValid = locationValidator.isGPSPositionValid(lastLocation);
        if (!isValid) return;
        return lastLocation.lat <= NWPoint.lat && lastLocation.lat >= SEPoint.lat && lastLocation.lon >= NWPoint.lon && lastLocation.lon <= SEPoint.lon;
    },
    checkWithinCork: function() {
        var NWPoint = {
            lat: 51.94,
            lon: -8.575
        };
        var SEPoint = {
            lat: 51.676,
            lon: -8.297
        };
        var lastLocation = gps.lastLocation();
        var isValid = locationValidator.isGPSPositionValid(lastLocation);
        if (!isValid) return;
        return lastLocation.lat <= NWPoint.lat && lastLocation.lat >= SEPoint.lat && lastLocation.lon >= NWPoint.lon && lastLocation.lon <= SEPoint.lon;
    }
};