"use strict";

var Q = require("vendor/q");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var CACHE_FOLDER = Ti.Filesystem.getFile(Ti.Filesystem.applicationCacheDirectory, "profile-cache");

CACHE_FOLDER.exists() || CACHE_FOLDER.createDirectory();

var _localFileName = function(imageURL) {
    return Ti.Utils.md5HexDigest(imageURL + "");
};

module.exports = {
    getCached: function(url) {
        if (!url) return null;
        var file = Ti.Filesystem.getFile(CACHE_FOLDER.resolve(), _localFileName(url));
        if (file.exists()) return file.getNativePath();
        return null;
    },
    downloadAvatar: function(url) {
        var file = Ti.Filesystem.getFile(CACHE_FOLDER.resolve(), _localFileName(url)), deferred = Q.defer();
        var client = Ti.Network.createHTTPClient({
            onload: function() {
                file.write(this.responseData);
                deferred.resolve(this.responseData);
            },
            onerror: function(e) {
                deferred.reject(e);
            },
            timeout: 5e3
        });
        client.open("GET", url);
        LoggingService.log("network", url);
        client.send();
        return deferred.promise;
    },
    getRemoteFile: function(url) {
        "object" == typeof url && url.response && (url = url.response);
        var file = Ti.Filesystem.getFile(CACHE_FOLDER.resolve(), _localFileName(url)), deferred = Q.defer();
        if (file.exists()) deferred.resolve(file.getNativePath()); else {
            var client = Ti.Network.createHTTPClient({
                onload: function() {
                    var index = this.responseText.substring(0, 8).indexOf("PNG");
                    if (this.responseData && (0 !== this.responseData.height && 0 !== this.responseData.width || 0 === index || 1 === index)) {
                        file.write(this.responseData);
                        deferred.resolve(this.responseData);
                    } else deferred.reject({
                        reason: "ResponseData isn't an image"
                    });
                },
                onerror: function(e) {
                    deferred.reject(e);
                },
                timeout: 6e4
            });
            client.open("GET", url);
            client.send();
        }
        return deferred.promise;
    }
};