"use strict";

var Q = require("/vendor/q");

var Internal = {};

Internal.Now = function() {
    return new Date().getTime().toString();
};

Internal.execute = function(fn, args, options) {
    var dfd = Q.defer();
    var result = null, error = null;
    try {
        result = fn.apply(options.scope, args);
    } catch (e) {
        error = e;
    }
    error ? result = Q.reject(error) : Q.isPromise(result) || (result = Q(result));
    Q.isPromise(result) || (result = Q(result));
    result.catch(function(e) {
        dfd.reject(e);
    }).then(function(r) {
        dfd.resolve(r);
    });
    return options.timeout ? dfd.promise.timeout(options.timeout) : dfd.promise;
};

var QueueEntry = function(options) {
    options = options || {};
    this.name = options.name || Internal.Now();
    this.runtimeName = this.name + "_" + (options.runtimeName ? options.runtimeName : new Date().getTime() * Math.random());
    this.fn = options.fn || function() {};
    this.args = options.args || [];
    this.scope = options.scope || {};
    this.defer = Q.defer();
};

QueueEntry.prototype.execute = function(timeout) {
    var self = this, dfd = Q.defer();
    return Internal.execute(this.fn, this.args, {
        scope: this.scope,
        timeout: timeout
    }).then(function(e) {
        self.defer.reject(e);
    }).then(function(r) {
        self.defer.resolve(r);
    });
};

module.exports = QueueEntry;