"use strict";

var _buildParams = function(prefix, obj, add) {
    if (_.isArray(obj)) _.each(obj, function(element, index) {
        _buildParams(prefix + "[" + ("object" == typeof element ? index : "") + "]", element, add);
    }); else if (_.isObject(obj)) {
        var name;
        for (name in obj) _buildParams(prefix + "[" + name + "]", obj[name], add);
    } else add(prefix, obj);
};

var _S4 = function() {
    return (0 | 65536 * (1 + Math.random())).toString(16).substring(1);
};

var _loadRoute = function(origin, destination) {
    return Alloy.createModel("points_route").route({
        originLat: origin.latitude,
        originLon: origin.longitude,
        destinationLat: destination.latitude,
        destinationLon: destination.longitude
    });
};

var _shortenPath = function(points, factor) {
    return _.filter(points, function(vertex, index) {
        if (index % factor === 0) return vertex;
    });
};

var _shortenEncodedPath = function(routePoint) {
    return require("/vendor/q").fcall(function() {
        var shortRoute = _shortenPath(routePoint.route, 10);
        while (shortRoute.length > 100) shortRoute = _shortenPath(shortRoute, 10);
        var encodedRoute = require("/utils/gmap").encode(shortRoute);
        return encodeURIComponent(encodedRoute);
    });
};

var MemoryPoolObj = function() {
    var _window;
    this.init = function() {
        _window = Ti.UI.createWindow();
        _window.hide();
        _window.open();
    };
    this.clean = function(obj) {
        this.init();
        if (obj instanceof Array) {
            var arLen = obj.length;
            for (var i = 0, len = arLen; len > i; ++i) _window.add(obj[i]);
        } else _window.add(obj);
        LOGGER.info("Cleaning MemoryPool.");
        _window.close();
        _window = null;
    };
};

module.exports = {
    param: function(obj) {
        var prefix, s = [], add = function(key, value) {
            value = _.isFunction(value) ? value() : null === value ? "" : value;
            s[s.length] = encodeURIComponent(key) + "=" + encodeURIComponent(value);
        };
        if (_.isArray(obj) || !_.isObject(obj)) _.each(obj, function() {
            add(this.name, this.value);
        }); else for (prefix in obj) _buildParams(prefix, obj[prefix], add);
        return s.join("&").replace(/%20/g, "+");
    },
    defaultToDp: function(defaultUnit) {
        return this.pixelToDp(defaultUnit);
    },
    pixelToDp: function(pixel) {
        var dpi = Ti.Platform.displayCaps.dpi;
        return pixel / (dpi / 160);
    },
    dpToDefault: function(dp) {
        return this.dpToPixel(dp);
    },
    dpToPixel: function(dp) {
        var dpi = Ti.Platform.displayCaps.dpi;
        return dp * (dpi / 160);
    },
    getResourceFile: function(path) {
        var fs = Ti.Filesystem, filepath = path;
        if (fs.getFile(filepath).exists()) return filepath;
        filepath = Ti.Filesystem.resourcesDirectory + filepath;
        if (fs.getFile(filepath).exists()) return filepath;
        throw new Error("Resource file doesn't exist " + path);
    },
    convertMetersToMiles: function(meters) {
        return Math.round(621371e-9 * meters * 10) / 10;
    },
    convertMilesToMeters: function(miles) {
        return Math.round(1609.34 * miles * 10) / 10;
    },
    createMapUrl: function(trip, mapWidth, mapHeight) {
        var units = require("/platform/units");
        var cacheKey = "mapurl: " + trip.origin.latitude + "," + trip.origin.longitude + "," + trip.destination.latitude + "," + trip.destination.longitude + "," + mapWidth + "x" + mapHeight;
        var cachedUrl = Ti.App.Properties.getObject(cacheKey, null);
        if (cachedUrl) return require("/vendor/q").fcall(function() {
            return cachedUrl;
        });
        var _loadStaticMapUrl = function(route) {
            var promise = Alloy.createModel("static_map").save({
                size: {
                    width: Math.round(units.dipToPixel(mapWidth)),
                    height: Math.round(units.dipToPixel(mapHeight))
                },
                origin: {
                    lat: trip.origin.latitude,
                    lon: trip.origin.longitude
                },
                destination: {
                    lat: trip.destination.latitude,
                    lon: trip.destination.longitude
                },
                path: route
            });
            return promise;
        };
        var _cacheMapUrl = function(url) {
            Ti.App.Properties.setObject(cacheKey, url);
            return require("/vendor/q").fcall(function() {
                return url;
            });
        };
        return _loadRoute(trip.origin, trip.destination).then(_shortenEncodedPath).then(_loadStaticMapUrl).then(_cacheMapUrl);
    },
    guid: function() {
        return _S4() + _S4() + "-" + _S4() + "-" + _S4() + "-" + _S4() + "-" + _S4() + _S4() + _S4();
    },
    getApnsAppKey: function() {
        var appId = Ti.App.getId(), appKey = "avego";
        "ma.car.CarmaEnterprise" === appId && (appKey = "carma-enterprise");
        return appKey;
    },
    clean: function(e) {
        null != e && e.children && e.removeAllChildren();
    },
    cleanMemory: function(viewObj) {
        var MemoryPool = new MemoryPoolObj();
        MemoryPool.clean(viewObj);
    }
};