"use strict";

var _ = require("/vendor/underscore"), LocalServerStatusStorage = require("./storage/LocalServerStatusStorage"), APICall = require("/v6/network/entity/APICall"), APICallType = require("/v6/network/entity/APICallType");

var CurrentStatusService = function(networkService) {
    var getCurrentDrive = function() {
        var apiCall = new APICall(APICallType.ActiveDrive, Date.now(), {
            localServerStatus: LocalServerStatusStorage.get()
        });
        return networkService.request(apiCall).then(function(data) {
            LocalServerStatusStorage.save({
                updatedOn: data.updatedOn
            });
            return {
                apiCall: apiCall,
                current: data
            };
        });
    };
    var service = {
        reset: function() {
            var defaultTime = new Date(1985, 11, 5).getTime();
            LocalServerStatusStorage.save({
                updatedOn: defaultTime
            });
        },
        lastUpdatedOn: function() {
            var localServerStatus = LocalServerStatusStorage.get();
            return localServerStatus.updatedOn;
        },
        getServerStatus: function() {
            var lastServerStatusDate = LocalServerStatusStorage.get();
            var apiCall = new APICall(APICallType.Status, Date.now(), {
                localServerStatus: lastServerStatusDate
            });
            return networkService.request(apiCall).then(function(data) {
                LocalServerStatusStorage.save({
                    updatedOn: data.updatedOn
                });
                return data.hasStartedRides || data.hasFinishedRides || data.hasStartedDrives || data.hasFinishedDrives ? getCurrentDrive() : null;
            });
        }
    };
    return service;
};

module.exports = CurrentStatusService;