"use strict";

var APICall = require("/v6/network/entity/APICall"), PendingDataAPICall = require("/v6/network/entity/PendingDataAPICall"), APICallType = require("/v6/network/entity/APICallType");

var BoardingApiCallFactory = function(currentUser) {
    return {
        createStartTripApiCall: function(drive) {
            var operations = [];
            if (drive.driverId != currentUser.id) return operations;
            if (drive.state.done) return operations;
            if (!drive.driveId) {
                operations.push(drive.startTimestamp && drive.startLat && drive.startLon && drive.startAccuracy >= 0 ? new APICall(APICallType.StartTrip, drive.startTimestamp, {
                    drive: drive
                }) : new PendingDataAPICall(APICallType.StartTrip, drive.startTimestamp, {
                    drive: drive
                }));
                operations.push(new PendingDataAPICall(APICallType.EndTrip, drive.startTimestamp, {
                    drive: drive
                }));
            }
            return operations;
        },
        createEndDriveApiCall: function(drive) {
            var operations = [];
            if (drive.driverId != currentUser.id) return operations;
            if (drive.state.done) return operations;
            drive.driveId && (drive.endTimestamp && drive.endLat && drive.endLon && drive.endAccuracy >= 0 ? operations.push(new APICall(APICallType.EndTrip, drive.endTimestamp, {
                drive: drive
            })) : drive.endTimestamp && operations.push(new PendingDataAPICall(APICallType.EndTrip, drive.startTimestamp, {
                drive: drive
            })));
            return operations;
        },
        createJoinCarApiCall: function(drive, ride) {
            var operations = [];
            if (drive.state.done || ride.state.done) return operations;
            if (!ride.rideId) {
                var apiCallType = APICallType.JoinCar;
                ride.driverPhoneNumber ? apiCallType = APICallType.JoinCarByPhoneNumber : ride.driverBeaconId && (apiCallType = APICallType.JoinCarByBeacon);
                operations.push(ride.startTimestamp && ride.startLat && ride.startLon && ride.startAccuracy >= 0 ? new APICall(apiCallType, ride.startTimestamp, {
                    drive: drive,
                    ride: ride
                }) : new PendingDataAPICall(apiCallType, ride.startTimestamp, {
                    drive: drive,
                    ride: ride
                }));
                operations.push(new PendingDataAPICall(APICallType.LeaveCar, ride.startTimestamp, {
                    drive: drive,
                    ride: ride
                }));
            }
            return operations;
        },
        createLeaveCarApiCall: function(drive, ride) {
            var operations = [];
            if (drive.state.done) return operations;
            if (ride.state.done) return operations;
            ride.rideId && (ride.endTimestamp && ride.endLat && ride.endLon && ride.endAccuracy >= 0 ? operations.push(new APICall(APICallType.LeaveCar, ride.endTimestamp, {
                drive: drive,
                ride: ride
            })) : ride.endTimestamp && operations.push(new PendingDataAPICall(APICallType.LeaveCar, drive.endTimestamp, {
                drive: drive,
                ride: ride
            })));
            return operations;
        }
    };
};

module.exports = BoardingApiCallFactory;