"use strict";

var _ = require("/vendor/underscore");

var BoardingLogMessages = {
    driveStatusLogMessage: function(drive) {
        drive = _.extend({
            id: null,
            driveId: null,
            driverPhoneNumber: null,
            driverBeaconId: null,
            startTimestamp: null,
            startLat: null,
            startLon: null,
            startAccuracy: null,
            endTimestamp: null,
            endLat: null,
            endLon: null,
            endAccuracy: null,
            state: {}
        }, drive);
        var message = "\n	" + _.template("Drive id=<%= id %>, driveId=<%= driveId %>, driverPhoneNumber=<%= driverPhoneNumber %>, driverBeaconId=<%= driverBeaconId %>", {
            id: drive.id,
            driveId: drive.driveId,
            driverPhoneNumber: drive.driverPhoneNumber,
            driverBeaconId: drive.driverBeaconId
        }) + "\n	" + _.template("state=<%= state %>, error=<%= error %>, errorCode=<%= code %>, errorMessage=<%= message %>", {
            state: drive.state.done,
            error: drive.state.error,
            code: drive.state.errorCode,
            message: drive.state.errorMessage
        }) + "\n	" + _.template("startTimestamp=<%= startTimestamp %>, startLat=<%= startLat %>, startLon=<%= startLon %>, startAccuracy=<%= startAccuracy %>", drive) + "\n	" + _.template("endTimestamp=<%= endTimestamp %>, endLat=<%= endLat %>, endLon=<%= endLon %>, endAccuracy=<%= endAccuracy %>", drive);
        return message;
    },
    fullDriveStatusLogMessage: function(drive) {
        drive = _.extend({
            id: null,
            driveId: null,
            driverPhoneNumber: null,
            driverBeaconId: null,
            startTimestamp: null,
            startLat: null,
            startLon: null,
            startAccuracy: null,
            endTimestamp: null,
            endLat: null,
            endLon: null,
            endAccuracy: null,
            state: {}
        }, drive);
        var message = "\n	" + _.template("Drive id=<%= id %>, driveId=<%= driveId %>, driverPhoneNumber=<%= driverPhoneNumber %>, driverBeaconId=<%= driverBeaconId %>", {
            id: drive.id,
            driveId: drive.driveId,
            driverPhoneNumber: drive.driverPhoneNumber,
            driverBeaconId: drive.driverBeaconId
        }) + "\n	" + _.template("done=<%= state %>, error=<%= error %>, errorCode=<%= code %>, errorMessage=<%= message %>", {
            state: drive.state.done,
            error: drive.state.error,
            code: drive.state.errorCode,
            message: drive.state.errorMessage
        }) + "\n	" + _.template("startTimestamp=<%= startTimestamp %>, startLat=<%= startLat %>, startLon=<%= startLon %>, startAccuracy=<%= startAccuracy %>", drive) + "\n	" + _.template("endTimestamp=<%= endTimestamp %>, endLat=<%= endLat %>, endLon=<%= endLon %>, endAccuracy=<%= endAccuracy %>", drive);
        _.each(drive.rides, function(ride) {
            ride = _.extend({
                id: null,
                rideId: null,
                startLat: null,
                startLon: null,
                startAccuracy: null,
                endTimestamp: null,
                endLat: null,
                endLon: null,
                endAccuracy: null,
                state: {}
            }, ride);
            message = message + "\n		" + _.template("Ride id=<%= id %>, rideId=<%= rideId %>, riderId=<%= riderId %>", ride) + "\n		" + _.template("done=<%= state %>, error=<%= error %>", {
                state: ride.state.done,
                error: ride.state.error
            }) + "\n		" + _.template("startTimestamp=<%= startTimestamp %>, startLat=<%= startLat %>, startLon=<%= startLon %>, startAccuracy=<%= startAccuracy %>", ride) + "\n		" + _.template("endTimestamp=<%= endTimestamp %>, endLat=<%= endLat %>, endLon=<%= endLon %>, endAccuracy=<%= endAccuracy %>", ride);
        });
        return message;
    },
    fullSummaryLogMessage: function(summary) {
        summary = _.extend({
            id: null,
            distance: null,
            isError: null,
            isPendingData: null,
            startTimestamp: null,
            startAddress: null,
            endTimestamp: null,
            endAddress: null,
            unrealisticCause: null,
            achievements: [],
            rewards: [],
            drive: {}
        }, summary);
        var message = "\n	" + _.template("Summary id=<%= id %>, driveId=<%= driveId %>", {
            id: summary.id,
            driveId: summary.drive.driveId
        }) + "\n	" + _.template("isError=<%= isError %>, isPendingData=<%= isPendingData %>, unrealisticCause=<%= unrealisticCause %>", summary) + "\n	" + _.template("distance=<%= distance %>, nachievements=<%= nachievements %>, nrewards=<%= nrewards %>", {
            distance: summary.distance,
            nachievements: summary.achievements.length,
            nrewards: summary.rewards.length
        }) + "\n	" + _.template("startTimestamp=<%= startTimestamp %>, startAddress=<%= startAddress %>", summary) + "\n	" + _.template("endTimestamp=<%= endTimestamp %>, endAddress=<%= endAddress %>", summary);
        _.each(summary.rides, function(ride) {
            ride = _.extend({
                userId: null,
                duration: null
            }, ride);
            message = message + "\n		" + _.template("Rider riderId=<%= riderId %>, duration=<%= duration %>", ride);
        });
        return message;
    }
};

module.exports = BoardingLogMessages;