"use strict";

var _ = require("/vendor/underscore");

var Validation = {
    required: function(value, name) {
        name = name || "value";
        if (null == value) throw new Error(_.template("<%= name %> is required", {
            name: name
        }));
    },
    cannotBeZero: function(value, name) {
        name = name || "value";
        if (0 === value) throw new Error(_.template("<%= name %> cannot be zero", {
            name: name
        }));
    },
    cannotBeNegative: function(value, name) {
        name = name || "value";
        if (0 > value) throw new Error(_.template("<%= name %> cannot be negative", {
            name: name
        }));
    },
    cannotBeRedefined: function(value, update, name) {
        name = name || "value";
        if (value == update) return;
        if (null == value) return;
        throw new Error(_.template("<%= name %> was already defined.", {
            name: name
        }));
    },
    allAreOk: function(values, name) {
        name = name || "values";
        var valid = true;
        _.each(values, function(value) {
            value || (valid = false);
            if (!valid) return false;
        });
        if (!valid) throw new Error(_.template("All the <%= name %> must be Ok", {
            name: name
        }));
    },
    oneIsOk: function(values, name) {
        name = name || "values";
        var valid = false;
        _.each(values, function(value) {
            if (!!value) {
                valid = true;
                return false;
            }
        });
        if (!valid) throw new Error(_.template("At least one of <%= name %> should be Ok", {
            name: name
        }));
    },
    allAreDefined: function(values, name) {
        name = name || "values";
        var valid = true;
        _.each(values, function(value) {
            null == value && (valid = false);
            if (!valid) return false;
        });
        if (!valid) throw new Error(_.template("All the <%= name %> must be defined", {
            name: name
        }));
    },
    allAreDefinedOrUndefined: function(values, name) {
        name = name || "values";
        var valid = true;
        _.each(values, function(value, index) {
            if (0 === index) return;
            var previous = values[index - 1];
            (null != value && null == previous || null == value && null != previous) && (valid = false);
            if (!valid) return false;
        });
        if (!valid) throw new Error(_.template("All the <%= name %> must be defined or undefined", {
            name: name
        }));
    },
    isValidGPSPosition: function(gps) {
        if (("number" == typeof gps || "string" == typeof gps) && (gps >= 0 || 0 > gps)) return true;
        return false;
    },
    isValidGPSAccuracy: function(accuracy) {
        if (("number" == typeof accuracy || "string" == typeof accuracy) && accuracy >= 0) return true;
        return false;
    },
    isValidTimestamp: function(timestamp) {
        if (("number" == typeof timestamp || "string" == typeof timestamp) && timestamp > 0) return true;
        return false;
    }
};

module.exports = Validation;