"use strict";

var _ = require("/vendor/underscore"), Backbone = require("/v6/infrastructure/backbone/BackboneExtend"), Validation = require("../helpers/Validation"), RideCollection = require("./RideCollection"), Mixins = require("./Mixins");

var Drive = Backbone.ExtendedModel.extend({
    config: {
        sync: require("/v6/infrastructure/backbone/wORM/Sync")
    },
    wORM: {
        module: "boarding",
        name: "Drive",
        fields: {
            state: {
                module: "boarding",
                name: "SyncState",
                create: true,
                cascadeDelete: true
            }
        }
    },
    initialize: function(attr) {
        attr = attr || {};
        var rides = new RideCollection();
        this.set("rides", rides, {
            silent: true
        });
        attr.rides && rides.add(attr.rides);
    },
    id: function() {
        return this.get("id") || "";
    },
    driveId: function() {
        return this.get("driveId") || 0;
    },
    driverId: function() {
        return this.get("driverId") || 0;
    },
    driverPhoneNumber: function() {
        return this.get("driverPhoneNumber") || null;
    },
    driverBeaconId: function() {
        return this.get("driverBeaconId") || null;
    },
    startTimestamp: function() {
        return this.get("startTimestamp") || 0;
    },
    startLon: function() {
        return this.get("startLon") || 0;
    },
    startLat: function() {
        return this.get("startLat") || 0;
    },
    startAccuracy: function() {
        return this.get("startAccuracy") || 0;
    },
    endTimestamp: function() {
        return this.get("endTimestamp") || 0;
    },
    endLon: function() {
        return this.get("endLon") || 0;
    },
    endLat: function() {
        return this.get("endLat") || 0;
    },
    endAccuracy: function() {
        return this.get("endAccuracy") || 0;
    },
    rides: function() {
        return this.get("rides");
    },
    state: function() {
        return this.get("state");
    },
    validate: function() {
        Validation.oneIsOk([ this.get("driverId"), this.get("driverPhoneNumber"), this.get("driverBeaconId") ], "Driver information");
        this.validateStarEndTimestamp();
        this.validateGPS();
    }
});

_.extend(Drive.prototype, Mixins.StarEndTimestamp);

_.extend(Drive.prototype, Mixins.GPS);

module.exports = Drive;