"use strict";

var _ = require("/vendor/underscore"), Validation = require("../helpers/Validation");

var Mixins = {
    StarEndTimestamp: {
        hasEnded: function() {
            return !!this.get("endTimestamp");
        },
        validateStarEndTimestamp: function() {
            var self = this, notRedifined = [ "startTimestamp", "endTimestamp" ];
            Validation.required(this.get("startTimestamp"), "startTimestamp");
            _.each(notRedifined, function(fieldName) {
                Validation.cannotBeRedefined(self.previous(fieldName), self.get(fieldName), fieldName);
            });
        }
    },
    GPS: {
        needStartGPS: function() {
            var validGPS = Validation.isValidGPSPosition(this.get("startLat")) && Validation.isValidGPSPosition(this.get("startLon")) && Validation.isValidGPSAccuracy(this.get("startAccuracy"));
            return !validGPS;
        },
        needEndGPS: function() {
            if (!Validation.isValidTimestamp(this.get("endTimestamp"))) return false;
            var validGPS = Validation.isValidGPSPosition(this.get("endLat")) && Validation.isValidGPSPosition(this.get("endLon")) && Validation.isValidGPSAccuracy(this.get("endAccuracy"));
            return !validGPS;
        },
        needsGPS: function() {
            return this.needStartGPS() || this.needEndGPS();
        },
        validateGPS: function() {
            var self = this;
            var notRedifined = [ "startLon", "startLat", "startAccuracy", "endLon", "endLat", "endAccuracy" ];
            _.each(notRedifined, function(fieldName) {
                Validation.cannotBeRedefined(self.previous(fieldName), self.get(fieldName), fieldName);
            });
            Validation.allAreDefinedOrUndefined([ this.get("startLon"), this.get("startLat"), this.get("startAccuracy") ], "Start GPS coordinates");
            Validation.allAreDefinedOrUndefined([ this.get("endLon"), this.get("endLat"), this.get("endAccuracy") ], "End GPS coordinates");
            var notZeroFields = [ "startTimestamp", "startLon", "startLat", "endTimestamp", "endLon", "endLat" ];
            _.each(notZeroFields, function(fieldName) {
                Validation.cannotBeZero(self.get(fieldName), fieldName);
            });
            Validation.cannotBeNegative(self.get("startAccuracy"), "startAccuracy");
            Validation.cannotBeNegative(self.get("endAccuracy"), "endAccuracy");
        }
    }
};

module.exports = Mixins;