"use strict";

var _ = require("/vendor/underscore"), Backbone = require("/v6/infrastructure/backbone/BackboneExtend"), Validation = require("../helpers/Validation"), Mixins = require("./Mixins");

var Ride = Backbone.ExtendedModel.extend({
    config: {
        sync: require("/v6/infrastructure/backbone/wORM/Sync")
    },
    wORM: {
        module: "boarding",
        name: "Ride",
        fields: {
            state: {
                module: "boarding",
                name: "SyncState",
                create: true,
                cascadeDelete: true
            }
        }
    },
    initialize: function(attr) {
        attr = attr || {};
    },
    id: function() {
        return this.get("id") || "";
    },
    rideId: function() {
        return this.get("rideId") || 0;
    },
    driverId: function() {
        return this.get("driverId") || 0;
    },
    driverPhoneNumber: function() {
        return this.get("driverPhoneNumber") || null;
    },
    riderId: function() {
        return this.get("riderId") || 0;
    },
    startTimestamp: function() {
        return this.get("startTimestamp") || 0;
    },
    startLon: function() {
        return this.get("startLon") || 0;
    },
    startLat: function() {
        return this.get("startLat") || 0;
    },
    startAccuracy: function() {
        return this.get("startAccuracy") || 0;
    },
    endTimestamp: function() {
        return this.get("endTimestamp") || 0;
    },
    endLon: function() {
        return this.get("endLon") || 0;
    },
    endLat: function() {
        return this.get("endLat") || 0;
    },
    endAccuracy: function() {
        return this.get("endAccuracy") || 0;
    },
    freeRide: function() {
        return !!this.get("freeRide");
    },
    state: function() {
        return this.get("state");
    },
    validate: function() {
        Validation.oneIsOk([ this.get("driverId"), this.get("driverPhoneNumber"), this.get("driverBeaconId") ], "Rider driver information");
        if (!this.get("riderId")) return new Error("Ride.riderId is undefined");
        this.validateStarEndTimestamp();
        this.validateGPS();
    }
});

_.extend(Ride.prototype, Mixins.StarEndTimestamp);

_.extend(Ride.prototype, Mixins.GPS);

module.exports = Ride;