"use strict";

var _ = require("/vendor/underscore"), Backbone = require("/v6/infrastructure/backbone/BackboneExtend"), Drive = require("./Drive"), SummaryEarning = require("./SummaryEarning"), SummaryAchievementCollection = require("./SummaryAchievementCollection"), SummaryRideCollection = require("./SummaryRideCollection"), SummaryRewardCollection = require("./SummaryRewardCollection"), SummaryRewardTotalCollection = require("./SummaryRewardTotalCollection"), Validation = require("../helpers/Validation");

var Summary = Backbone.ExtendedModel.extend({
    config: {
        sync: require("/v6/infrastructure/backbone/wORM/Sync")
    },
    wORM: {
        module: "boarding",
        name: "Summary",
        drive: {
            module: "boarding",
            name: "Drive",
            create: false,
            cascadeDelete: true
        },
        earning: {
            module: "boarding",
            name: "SummaryEarning",
            create: false,
            cascadeDelete: true
        }
    },
    initialize: function(attr) {
        attr = attr || {};
        attr.earning = attr.earning || [];
        attr.achievements = attr.achievements || [];
        this.set("drive", new Drive(attr.drive));
        this.set("rides", new SummaryRideCollection(attr.rides));
        this.set("achievements", new SummaryAchievementCollection(attr.achievements));
        this.set("rewards", new SummaryRewardCollection(attr.rewards));
        this.set("rewardTotals", new SummaryRewardTotalCollection(attr.rewardTotals));
    },
    id: function() {
        return this.get("id") || "";
    },
    drive: function() {
        return this.get("driveId") || 0;
    },
    distance: function() {
        return this.get("distance") || 0;
    },
    startTimestamp: function() {
        return this.get("startTimestamp") || 0;
    },
    startAddress: function() {
        return this.get("startAddress") || "";
    },
    endTimestamp: function() {
        return this.get("endTimestamp") || 0;
    },
    endAddress: function() {
        return this.get("endAddress") || "";
    },
    isError: function() {
        return this.get("isError") || false;
    },
    unrealisticCause: function() {
        return this.get("unrealisticCause") || "";
    },
    isPendingData: function() {
        return this.get("isPendingData") || false;
    },
    rides: function() {
        return this.get("rides");
    },
    earning: function() {
        return this.get("earning");
    },
    achievements: function() {
        return this.get("achievements");
    },
    rewards: function() {
        return this.get("rewards");
    },
    rewardTotals: function() {
        return this.get("rewardTotals");
    },
    validate: function() {
        var isError = !!this.get("isError"), isPendingData = !!this.get("isPendingData");
        Validation.required(isError, "isError");
        Validation.required(isPendingData, "isPendingData");
        Validation.required(this.get("drive").get("id"), "drive");
        if (!isError && !isPendingData) {
            Validation.required(this.get("distance"), "distance");
            Validation.required(this.get("startTimestamp"), "startTimestamp");
            Validation.required(this.get("startAddress"), "startAddress");
            Validation.required(this.get("endTimestamp"), "endTimestamp");
            Validation.required(this.get("endAddress"), "endAddress");
        }
    }
});

module.exports = Summary;