"use strict";

var Q = require("/vendor/q"), _ = require("/vendor/underscore"), FileLogger = require("./logging/FileLogger");

var TYPES = {
    network: "network",
    boarding: "boarding",
    profile: "profile"
};

var LEVELS = {
    info: "info",
    error: "error"
};

var LoggingService = function(configService) {
    var isDev = configService.isDev(), timeout = null, loggers = {};
    _.each(TYPES, function(type) {
        loggers[type] = new FileLogger(type, {
            baseUrl: configService.logsBaseUrl,
            enabled: isDev
        });
    });
    var periodicFlush = function() {
        clearTimeout(timeout);
        timeout = setTimeout(function() {
            self.flush().finally(periodicFlush);
        }, 3e4);
    };
    isDev && periodicFlush();
    var self = {
        logger: function(type) {
            var logger = loggers[type];
            return logger;
        },
        log: function(type, message) {
            var logger = loggers[type];
            logger && logger.log(message);
        },
        flush: function() {
            clearTimeout(timeout);
            var dfd = Q();
            _.each(loggers, function(logger) {
                dfd = dfd.finally(logger.flush);
            });
            return dfd.finally(function() {
                periodicFlush();
            });
        },
        dispose: function() {
            clearTimeout(timeout);
            loggers = null;
        }
    };
    return self;
};

LoggingService.$inject = [ "ConfigService" ];

LoggingService.$isRevealingModule = true;

module.exports = LoggingService;