"use strict";

var _ = require("/vendor/underscore"), FileLoggerRemoteDispatcher = require("./FileLoggerRemoteDispatcher");

var MAX_LOGFILE_SIZE_BYTES = 5242880;

var getLogFile = function(name) {
    return Ti.Filesystem.getFile(Ti.Filesystem.tempDirectory, name + ".log");
};

var getRemoteLogFile = function(name) {
    return Ti.Filesystem.getFile(Ti.Filesystem.tempDirectory, name + ".log.remote");
};

var tuncateFile = function(name) {
    var file = getLogFile(name);
    file && file.size >= MAX_LOGFILE_SIZE_BYTES && file.write("", false);
};

var getMessage = function(message) {
    var date = new Date();
    message = _.template("[<%= year %>-<%= month %>-<%= day %> <%= hour %>:<%= minute %>:<%= second %>:<%= msec %>] <%= message %> \n", {
        year: date.getFullYear(),
        month: date.getMonth() + 1,
        day: date.getDate(),
        hour: date.getHours(),
        minute: date.getMinutes(),
        second: date.getSeconds(),
        msec: date.getMilliseconds(),
        message: message
    });
    return message;
};

var FileLogger = function(name, options) {
    options = options || {};
    options.enabled = options.enabled || true;
    options.baseUrl = options.baseUrl || "";
    var remoteDispatcher = FileLoggerRemoteDispatcher({
        name: name,
        baseUrl: options.baseUrl,
        filePath: getRemoteLogFile(name).nativePath
    });
    var self = {
        log: function(message) {
            if (!options.enabled) return;
            var msg = message, file = getLogFile(name), remoteFile = getRemoteLogFile(name);
            if ("function" == typeof message) try {
                msg = message();
            } catch (e) {
                msg = e.toString() + "\n" + e.backtrace;
            }
            msg = _.template("<%= msg %>", {
                msg: msg
            });
            file.write(getMessage(msg), true);
            remoteFile.write(getMessage(msg), true);
        },
        logDuration: function(fn, options) {
            if ("function" != typeof fn) return fn;
            options = options || {};
            options.scope = options.scope || {};
            options.name = options.name || "?";
            return function() {
                var start = Date.now(), args = arguments, result = void 0;
                try {
                    result = fn.apply(options.scope, args);
                } finally {
                    var duration = Date.now() - start;
                    duration > 200 && self.log(_.template("WARNING slow function <%= name %>. (<%= duration %>msec).", {
                        duration: duration,
                        name: options.name
                    }));
                }
                return result;
            };
        },
        flush: function() {
            return remoteDispatcher.flush().then(function() {
                tuncateFile(name);
            });
        }
    };
    return self;
};

module.exports = FileLogger;