"use strict";

var Q = require("/vendor/q"), _ = require("/vendor/underscore");

var readFile = function(filePath) {
    var file = Ti.Filesystem.getFile(filePath), size = file.size;
    if (0 == size) return Q();
    var blob = file.read();
    return Q(blob);
};

var send = function(baseUrl, loggerName, blob) {
    if (!blob || !blob.length) return Q();
    var dfd = Q.defer(), user = require("/services/resource_owner_service").singleton(), url = _.template("<%= baseUrl %>/[<%= uuid %>@<%= osname %>]%20[<%= name %>]%20<%= email %>/log", {
        baseUrl: baseUrl,
        uuid: Ti.Platform.id,
        email: user && user.get("email") ? user.get("email") : "anonymous",
        osname: "android",
        name: loggerName
    });
    var client = Ti.Network.createHTTPClient({
        cache: false,
        enableKeepAlive: true,
        timeout: 12e4
    });
    client.onload = function() {
        dfd.resolve();
    };
    client.onerror = function(e) {
        var isOffline = 1 == e.code || 2 == e.code;
        isOffline ? dfd.reject(e) : dfd.resolve();
    };
    client.open("POST", url);
    client.setRequestHeader("Content-Type", "text/plain");
    client.setRequestHeader("Accept", "text/plain");
    client.setRequestHeader("User-Agent", Ti.userAgent);
    client.setRequestHeader("Accept-Encoding", "gzip,deflate");
    client.send(blob);
    return dfd.promise;
};

var FileLoggerRemoteDispatcher = function(options) {
    options = options || {};
    options.name = options.name || "";
    options.baseUrl = options.baseUrl || "";
    options.filePath = options.filePath || "";
    var self = {
        flush: function() {
            return readFile(options.filePath).then(function(blob) {
                return send(options.baseUrl, options.name, blob);
            }).then(function() {
                var file = Ti.Filesystem.getFile(options.filePath);
                file.write("", false);
            }).catch(function(e) {
                var file = Ti.Filesystem.getFile(options.filePath);
                file.write(e.toString(), true);
            });
        }
    };
    return self;
};

module.exports = FileLoggerRemoteDispatcher;