"use strict";

var _ = require("/vendor/underscore"), Q = require("/vendor/q"), QueueEntry = require("/v6/common/queue/QueueEntry");

var Queue = function(options) {
    options = options || {};
    var _queue = [];
    var _qIndex = {};
    var _qTimeout = options.timeout || null;
    var _current = null;
    var _isStopped = false;
    var _execute = function() {
        if (_isStopped) return;
        if (_current) return;
        _current = _queue.shift();
        if (!_current) return;
        delete _qIndex[_current.runtimeName];
        var result = _current.execute(_qTimeout);
        Q.isPromise(result) || (result = Q(result));
        result.catch(function(e) {
            console.log("Error on function " + _current.name);
            console.log(e);
            throw e;
        }).finally(function(val) {
            _current = null;
            _execute();
        });
    };
    var _push = function(entry) {
        if (!entry instanceof QueueEntry) throw new Error("Expected QueueEntry instance");
        if (_qIndex[entry.runtimeName] && _qIndex[entry.runtimeName].name == entry.name) return null;
        _qIndex[entry.runtimeName] = entry;
        _queue.push(entry);
        _execute();
        return entry;
    };
    this.push = function(fn, options) {
        options = options || {};
        options.name = options.name;
        options.runtimeName = options.runtimeName;
        options.args = options.args;
        options.scope = options.scope || {};
        var entry = new QueueEntry({
            fn: fn,
            name: options.name,
            runtimeName: options.runtimeName,
            scope: options.scope,
            args: options.args
        });
        entry = _push(entry);
        if (null == entry) return Q();
        return entry.defer.promise;
    };
    this.bindFn = function(fn, options) {
        options = options || {};
        options.scope = options.scope || {};
        options.unique = !!options.unique;
        options.fnName = options.fnName || "";
        if ("function" != typeof fn) throw new Error("A function was expected!");
        var self = this;
        var runner = function() {
            var runtimeName = null;
            options.unique && options.fnName && (runtimeName = options.fnName);
            self.push(fn, {
                name: options.fnName,
                runtimeName: runtimeName,
                scope: options.scope,
                args: arguments
            });
            return Q();
        };
        return fn;
    };
    this.bind = function(scope, fnName, options) {
        if ("function" != typeof scope[fnName]) throw new Error("A function was expected!");
        var self = this, fn = scope[fnName];
        scope[fnName] = this.bindFn(fn, _.extend({}, options, {
            fnName: fnName,
            scope: scope
        }));
    };
    this.bindAll = function(scope, fns) {
        var self = this;
        _.each(fns, function(fnName) {
            self.bind(scope, fnName);
        });
    };
    this.clear = function() {
        _queue = [];
        _qIndex = {};
    };
    this.all = function() {
        var dfd = Q();
        _queue.length && (dfd = _queue[_queue.length - 1].defer);
        this.start = true;
        return dfd.promise;
    };
    Object.defineProperty(this, "start", {
        get: function() {
            return !this.stop;
        },
        set: function(val) {
            this.stop = !val;
        },
        configurable: false
    });
    Object.defineProperty(this, "stop", {
        get: function() {
            return _isStopped;
        },
        set: function(val) {
            _isStopped = !!val;
            _execute();
        },
        configurable: false
    });
    Object.defineProperty(this, "length", {
        get: function() {
            return _queue.length;
        },
        set: function(val) {
            throw new Error("Invalid Operation");
        },
        configurable: false
    });
    this.clear();
};

module.exports = Queue;