"use strict";

var urls = require("/carmalib/url"), _ = require("/vendor/underscore"), Q = require("/vendor/q"), ExploreServiceBatchApiHelper = require("./ExploreServiceBatchApiHelper");

var ExploreServiceBatchApiFactory = function() {
    var _create = function(method, urlKey, urlParams, extraUrlQueryParams) {
        if (!method || !urlKey) {
            LOGGER.error("[ExploreServiceBatchApiFactory] - Method create, missing some arguments");
            return;
        }
        urlParams = urlParams || {};
        var url = _.template(urls[urlKey], urlParams);
        url = _appendExtraQueryParams(url, extraUrlQueryParams);
        return {
            method: method,
            url: url
        };
    };
    var _appendExtraQueryParams = function(url, extraUrlQueryParams) {
        var firstParam = false;
        if (-1 === url.indexOf("?")) {
            url += "?";
            firstParam = true;
        }
        extraUrlQueryParams = extraUrlQueryParams || {};
        for (var key in extraUrlQueryParams) if (firstParam) {
            url += key + "=" + extraUrlQueryParams[key];
            firstParam = false;
        } else url += "&" + key + "=" + extraUrlQueryParams[key];
        return url;
    };
    var _execute = function(apiCallRequests, apiCallOrder, callback) {
        if (!apiCallRequests || !apiCallOrder) {
            LOGGER.error("[ExploreServiceBatchApiFactory] - No api call requests or no api call order provided");
            return;
        }
        return Q().then(function() {
            return Alloy.createModel("batch_api", {
                apiCallRequests: apiCallRequests
            }).save();
        }).then(function(data) {
            ExploreServiceBatchApiHelper.processResponse(data, apiCallOrder);
        }).then(function() {
            callback && callback();
        }).catch(_handleError);
    };
    var _handleError = function(error) {
        LOGGER.error("[ExploreServiceBatchApiFactory] " + error.toString());
        return error;
    };
    var self = {
        create: function(method, urlKey, urlParams, extraUrlQueryParams) {
            return _create(method, urlKey, urlParams, extraUrlQueryParams);
        },
        execute: function(apiCallRequests, apiCallOrder, callback) {
            return _execute(apiCallRequests, apiCallOrder, callback);
        }
    };
    return self;
};

module.exports = new ExploreServiceBatchApiFactory();