"use strict";

var ExploreServiceBatchApiHelper = function() {
    var _responses = {};
    var _processResponse = function(data, apiCallOrder) {
        var apiRequests = apiCallOrder.length;
        if (!apiRequests) return;
        for (var responseIndex = 0; apiRequests > responseIndex; ++responseIndex) {
            var apiCallType = apiCallOrder[responseIndex];
            if (!self[apiCallType]) throw new Error("[ExploreServiceBatchApiHelper] - Api call type not recognised: " + apiCallType);
            _responses[apiCallType] = _parseBestMatchesApiCallResponse(data, responseIndex);
        }
    };
    var _parseBestMatchesApiCallResponse = function(data, responseIndex) {
        return JSON.parse(data.apiCallResponses[responseIndex].body);
    };
    var _getResponseByType = function(type) {
        return _responses.hasOwnProperty(type) ? _responses[type] : null;
    };
    var self = {
        BEST_MATCHES_RIDE: "BEST_MATCHES_RIDE",
        BEST_MATCHES_DRIVE: "BEST_MATCHES_DRIVE",
        NEXT_TRIP: "NEXT_TRIP",
        BOOKINGS: "BOOKINGS",
        processResponse: function(data, apiCallOrder) {
            _processResponse(data, apiCallOrder);
        },
        getBestMatchesResponse: function() {
            var response = {
                rideMatches: _getResponseByType(this.BEST_MATCHES_RIDE),
                driveMatches: _getResponseByType(this.BEST_MATCHES_DRIVE)
            };
            return response;
        },
        getNextTripResponse: function() {
            return _getResponseByType(this.NEXT_TRIP);
        },
        getBookingsResponse: function() {
            return _getResponseByType(this.BOOKINGS);
        }
    };
    return self;
};

module.exports = new ExploreServiceBatchApiHelper();