"use strict";

var _ = require("/vendor/underscore"), Globals = require("/v6/infrastructure/Globals"), AlloyController = require("./controllers/AlloyController"), IOCController = require("./controllers/IOCController");

var sequence = function(scope, fns) {
    return function() {
        _.each(fns, function(fn) {
            "function" == typeof fn && fn.apply(scope, []);
        });
    };
};

var ControllerClassFactory = function(ControllerClass, view, container) {
    return {
        create: function() {
            var Class = function() {
                var self = this, instance = ControllerClass.apply(this, arguments);
                _.extend(this, instance);
                _.each(instance, function(val, index) {
                    "function" == typeof fn && _.bind(instance[index], self);
                });
                var alloy = new AlloyController(view);
                var ioc = new IOCController(container);
                this.initialize = sequence(this, [ alloy.initialize, ioc.initialize, this.initialize ]);
                this.destroy = sequence(this, [ alloy.destroy, ioc.destroy, this.destroy ]);
                this.postConstruct = sequence(this, [ alloy.postConstruct, ioc.postConstruct, this.postConstruct ]);
                view.getView().addEventListener("close", function closeHandler() {
                    view.getView().removeEventListener("close", closeHandler);
                    self.destroy();
                });
                this.initialize();
            };
            Class.$inject = ControllerClass.$inject;
            return Class;
        }
    };
};

var ControllerFactory = {
    initialize: function(ControllerClass, view, args) {
        args = args || [];
        var container = Globals.IOC.create({
            onDispose: function(service) {
                service && service.destroy && service.destroy();
            }
        });
        var classFactory = new ControllerClassFactory(ControllerClass, view, container);
        var Class = classFactory.create();
        container.register("Controller", Class);
        container.register("View", view);
        container.register("Arguments", args);
        var instance = container.get("Controller");
        _.extend(view, instance);
    }
};

module.exports = ControllerFactory;