"use strict";

var _ = require("/vendor/underscore"), Backbone = require("/vendor/backbone");

var getSyncMethod = function(model) {
    if (model.config && model.config.sync) return model.config.sync;
    return Backbone.sync;
};

Backbone.sync = function(method, model, options) {
    return getSyncMethod(model).apply(this, [ method, model, options ]);
};

var _backboneModelOriginalSet = Backbone.Model.prototype.set;

Backbone.Model.prototype.set = function(key, value, options) {
    var args = [ key, value ];
    if (_.isObject(key) || null == key) {
        options = value;
        args = [ key ];
    }
    options = options || {};
    options = _.extend({
        silent: true
    }, options);
    args.push(options);
    return _backboneModelOriginalSet.apply(this, args);
};

Backbone.ExtendedModel = function() {
    Backbone.Model.apply(this, arguments);
    this.toJSON = function() {
        var json = {};
        _.each(this.attributes, function(value, index) {
            json[index] = value instanceof Backbone.Model || value instanceof Backbone.Collection ? value.toJSON() : value;
        });
        return json;
    };
};

Backbone.ExtendedModel.prototype = Backbone.Model.prototype;

Backbone.ExtendedModel.extend = Backbone.Model.extend;

module.exports = Backbone;