"use strict";

function guid() {
    var S4 = function() {
        return Math.floor(65536 * (1 + Math.random())).toString(16).substring(1);
    };
    return S4() + S4() + "-" + S4() + "-" + S4() + "-" + S4() + "-" + S4() + S4() + S4();
}

var _ = require("/vendor/underscore"), Backbone = require("/v6/infrastructure/backbone/BackboneExtend");

var wORM_MODEL_DEFAULTS = {
    module: "",
    name: "",
    id: "",
    modelFactory: "/v6/infrastructure/ModelFactory",
    cascadeDelete: void 0,
    fields: {}
};

var MODEL_ID_ATTRIBUTE = "id";

var wORM_FIELD_DEFAULTS = {
    module: "",
    name: "",
    create: false,
    cascadeDelete: true,
    literal: false
};

var Config = {
    get: function(model) {
        if (model instanceof Backbone.Model || model instanceof Backbone.Collection) {
            if (model.wORM) {
                var config = _.extend({}, wORM_MODEL_DEFAULTS, model.wORM);
                if (!config.module) throw new Error(_.template("wORM.<%= field %> is undefined", {
                    field: "module"
                }));
                if (!config.name) throw new Error(_.template("wORM.<%= field %> is undefined", {
                    field: "name"
                }));
                if (model instanceof Backbone.Model) {
                    var defaults = {
                        cascadeDelete: null != config.cascadeDelete ? !!config.cascadeDelete : wORM_FIELD_DEFAULTS.cascadeDelete
                    };
                    _.each(model.attributes, function(value, index) {
                        (model instanceof Backbone.Model || model instanceof Backbone.Collection) && (config.fields[index] = _.extend({}, wORM_FIELD_DEFAULTS, defaults, config.fields[index]));
                    });
                }
                if (model instanceof Backbone.Collection) {
                    config.name = config.name.replace(/Collection$/, "");
                    delete config.fields;
                }
                return config;
            }
            throw new Error("Missing wORM model configuration");
        }
        throw new Error("Argument must be a Backbone Model or Collection");
    },
    getId: function(model, generate) {
        var id = 0, config = Config.get(model);
        if (model instanceof Backbone.Model) {
            id = model.get(MODEL_ID_ATTRIBUTE);
            if (!id && generate) {
                id = config.id ? model.get(config.id) : guid();
                if (!id || "object" == typeof id) throw new Error("wORM.Config.getId returned an invalid id:" + JSON.stringify(id));
                model.set(MODEL_ID_ATTRIBUTE, id);
            }
        }
        return id;
    },
    getClass: function(factoryClassName, module, name) {
        var ModelFactory = null, Class = null;
        factoryClassName || (factoryClassName = "/v6/infrastructure/ModelFactory");
        try {
            ModelFactory = require(factoryClassName);
        } catch (e) {
            throw new Error(_.template("Error while loading config.modelFactory '<%= factoryClassName %>'", {
                factoryClassName: factoryClassName
            }));
        }
        try {
            Class = ModelFactory.getClass(module, name);
        } catch (e) {}
        if (!Class) throw new Error(_.template("Could not load module <%= module %>.<%= name %>", {
            module: module,
            name: name
        }));
        return Class;
    }
};

module.exports = Config;