"use strict";

var _ = require("/vendor/underscore"), Backbone = require("/v6/infrastructure/backbone/BackboneExtend"), Config = require("./Config"), SerializedModel = require("./SerializedModel");

var Internal = {};

Internal.createSerializedReference = function(model) {
    var config = Config.get(model), id = Config.getId(model);
    if (!id) throw new Error(_.template("Id is missing on <%= model %>", {
        model: config.name
    }));
    return new SerializedModel({
        module: config.module,
        model: config.name,
        modelFactory: config.modelFactory,
        referenceOf: id
    });
};

var Serializer = {
    serialize: function(model, options) {
        options = options || {};
        options.original = options.original || void 0;
        options.parent = options.parent;
        options.index = options.index;
        if (model instanceof Backbone.Model) {
            var config = Config.get(model), result = null;
            if (options.parent) {
                if (options.original instanceof Backbone.Model && Config.getId(options.original) != Config.getId(model)) {
                    var pconfig = Config.get(options.parent);
                    pconfig.fields[options.index].cascadeDelete && options.original.destroy();
                }
                result = Internal.createSerializedReference(model);
            } else {
                result = new SerializedModel({
                    module: config.module,
                    model: config.name,
                    modelFactory: config.modelFactory,
                    data: {}
                });
                _.each(config.fields, function(fieldConfig, name) {
                    if (fieldConfig.create && !model.attributes[name]) {
                        var Class = Config.getClass(config.modelFactory, fieldConfig.module, fieldConfig.name);
                        var instance = new Class().save();
                        model.set(name, instance);
                    }
                });
                _.each(model.attributes, function(value, index) {
                    var _options = {
                        parent: model,
                        index: index
                    };
                    options.original && (_options.original = options.original.attributes[index]);
                    model[index] instanceof Backbone.Collection && (value = model[index]);
                    _options.original instanceof Backbone.Model && !value && config.fields[index].cascadeDelete && _options.original.destroy();
                    result.data[index] = Serializer.serialize(value, _options);
                });
            }
            return result;
        }
        if (model instanceof Backbone.Collection) {
            var config = Config.get(model), references = [];
            for (var i = 0; i < model.length; i++) {
                var _model = model.at(i), reference = Internal.createSerializedReference(_model);
                references.push(reference);
            }
            if (options.original && config.cascadeDelete) for (var i = 0; i < options.original.length; i++) {
                var _original = options.original.at(i);
                var _model = model.get(_original.get("id"));
                if (!_model) {
                    options.original.remove(_original);
                    _original.destroy();
                }
            }
            var modelName = config.name + "Collection";
            Config.getClass(config.modelFactory, config.module, modelName);
            return new SerializedModel({
                module: config.module,
                model: modelName,
                modelFactory: config.modelFactory,
                data: references
            });
        }
        if (void 0 === model && options.original) return options.original;
        return model;
    },
    deserialize: function(data, options) {
        if ("object" != typeof data) return data;
        if (null == data) return null;
        var sdata = SerializedModel(data), ModelClass = Config.getClass(sdata.modelFactory, sdata.module, sdata.model), model = new ModelClass(), config = Config.get(model);
        if (model instanceof Backbone.Model) {
            if (sdata.referenceOf) model = model.fetch({
                id: sdata.referenceOf
            }); else {
                var mdata = {};
                _.each(sdata.data, function(data, index) {
                    var value = data;
                    config.fields[index] && !config.fields[index].literal && (value = Serializer.deserialize(data, options));
                    mdata[index] = value;
                });
                model.set(mdata, {
                    silent: true
                });
            }
            return model;
        }
        if (model instanceof Backbone.Collection) {
            _.each(sdata.data, function(idata) {
                var value = Serializer.deserialize(idata, options);
                value && model.push(value);
            });
            return model;
        }
        throw new Error("Deserialize unexpected error");
    }
};

module.exports = Serializer;