"use strict";

function Sync(method, model, opts) {
    var config = Config.get(model);
    var storage = Extensions.getStorage(config.name);
    storage || (storage = new ObjectStorage(config.name));
    switch (method) {
      case "read":
        opts = opts || {};
        if (model instanceof Backbone.Model) {
            var id = opts.id || 0;
            var stored = storage.get(id);
            if (stored) {
                var smodel = Serializer.deserialize(stored);
                model.set(smodel.attributes, {
                    silent: true
                });
            } else model = null;
        } else {
            if (!(model instanceof Backbone.Collection)) throw new Error("Unexpected model variable. Must be a Backbone.Model or Backbone.Collection");
            model.reset();
            var stored = storage.all();
            _.each(stored, function(id) {
                var value = storage.get(id);
                var smodel = Serializer.deserialize(value);
                model.push(smodel);
            });
        }
        break;

      case "create":
      case "update":
        var id = Config.getId(model, true);
        var error = model.validate();
        if (error instanceof Error) throw error;
        if ("string" == typeof error && error) throw new Error(error);
        if (error) throw new Error("Model.validation() returned error!");
        var stored = storage.get(id);
        var previous = null;
        stored && (previous = Serializer.deserialize(stored));
        stored = Serializer.serialize(model, {
            original: previous
        });
        stored.data.id = id;
        storage.set(id, stored);
        var smodel = Serializer.deserialize(stored);
        model.set(smodel.attributes);
        break;

      case "delete":
        var id = Config.getId(model);
        storage.remove(id);
        model.clear();
    }
    return model;
}

var _ = require("/vendor/underscore"), Backbone = require("/v6/infrastructure/backbone/BackboneExtend"), ObjectStorage = require("/v6/infrastructure/storage/ObjectStorage"), Config = require("./Config"), Serializer = require("./Serializer"), Extensions = require("./Extensions");

module.exports = Sync;