"use strict";

var _ = require("/vendor/underscore");

var delegateEventSplitter = /^(\S+)\s*(.*)$/, platformSpecificEventRegex = /(.*)\[(.+)\]/, platformSpecificEventSplitter = /\s*,\s*/, currentPlatform = "android", ui = require("/utils/ui");

var Internal = {};

Internal.isPlatformMatch = function(platform) {
    if (!platform) return false;
    platform = platform.toLowerCase();
    if (platform === currentPlatform) return true;
    if ("ios" === platform && ("iphone" === currentPlatform || "ipad" === currentPlatform)) return true;
    return false;
};

Internal.attachEvent = function($, viewId, eventName, method) {
    var view = null;
    if (-1 !== viewId.indexOf(".")) {
        var widgetView = viewId.split(".");
        view = $.getView(widgetView[0]).getView(widgetView[1]);
    } else view = $.getView(viewId);
    view && view.addEventListener(eventName, method);
};

Internal.delegateEvents = function(view, $) {
    if (!view.events) return view;
    for (var key in view.events) {
        var method = view.events[key], match = key.match(delegateEventSplitter), eventName = match[1], viewId = match[2];
        _.isFunction(method) || (method = view[method]);
        if (!method || !eventName || !viewId) continue;
        var platformSpecificEvents = eventName.match(platformSpecificEventRegex);
        if (platformSpecificEvents) {
            eventName = platformSpecificEvents[1];
            var platforms = platformSpecificEvents[2].split(platformSpecificEventSplitter);
            for (var i = 0; i < platforms.length; i++) {
                var platform = platforms[i];
                Internal.isPlatformMatch(platform) && Internal.attachEvent($, viewId, eventName, method);
            }
        } else Internal.attachEvent($, viewId, eventName, method);
    }
};

var DEFAULT_OPTIONS = {
    CLOSE_WINDOW_TIME_OUT: 3e3
};

var AlloyController = function($, options) {
    options = _.extend({}, DEFAULT_OPTIONS, options);
    return {
        events: {},
        initialize: function() {},
        destroy: function() {},
        postConstruct: function() {
            Internal.delegateEvents(this, $);
        }
    };
};

module.exports = AlloyController;