"use strict";

var _ = require("/vendor/underscore"), ObjectStoragePrefixIndex = require("./ObjectStoragePrefixIndex");

var ObjectStorage = function(prefix) {
    if (!prefix) throw new Error("ObjectStorage needs a prefix");
    var index = new ObjectStoragePrefixIndex(prefix);
    var getKey = function(key) {
        return prefix + "_" + key;
    };
    var isValidStorageKey = function(key) {
        key = key || "";
        return 0 === key.indexOf(prefix + "_");
    };
    var storage = {
        all: function() {
            return index.all();
        },
        get: function(key) {
            key = getKey(key);
            return Ti.App.Properties.getObject(key);
        },
        set: function(key, value) {
            index.add(key);
            key = getKey(key);
            return Ti.App.Properties.setObject(key, value);
        },
        remove: function(key) {
            index.remove(key);
            key = getKey(key);
            return Ti.App.Properties.removeProperty(key);
        },
        clear: function() {
            index.clear();
            _.each(Ti.App.Properties.listProperties(), function(key) {
                isValidStorageKey(key) && Ti.App.Properties.removeProperty(key);
            });
        }
    };
    return storage;
};

module.exports = ObjectStorage;