"use strict";

var _ = require("/vendor/underscore");

var ObjectStoragePrefixIndex = function(prefix) {
    if (!prefix) throw new Error("ObjectStoragePrefixIndex needs a prefix");
    var indexKey = prefix + "__index";
    var getIndex = function() {
        return Ti.App.Properties.getObject(indexKey) || {};
    };
    var clearIndex = function() {
        return Ti.App.Properties.removeProperty(indexKey);
    };
    var addToIndex = function(key) {
        var index = Ti.App.Properties.getObject(indexKey) || {};
        if (!index[key]) {
            index[key] = key;
            Ti.App.Properties.setObject(indexKey, index);
            return true;
        }
        return false;
    };
    var removeFromIndex = function(key) {
        var index = Ti.App.Properties.getObject(indexKey) || {};
        if (index[key]) {
            delete index[key];
            Ti.App.Properties.setObject(indexKey, index);
            return true;
        }
        return false;
    };
    var self = {
        all: function() {
            return _.keys(getIndex(indexKey));
        },
        add: function(key) {
            return addToIndex(key);
        },
        remove: function(key) {
            return removeFromIndex(key);
        },
        clear: function() {
            clearIndex();
        }
    };
    return self;
};

module.exports = ObjectStoragePrefixIndex;