"use strict";

var _ = require("/vendor/underscore"), DeclarativeObject = require("/v6/common/DeclarativeObject");

var APIDataValue_DEFAULTS = {
    required: true,
    "default": null,
    bind: "",
    bindArg: ""
};

var APIDataValue = function(validationRule) {
    validationRule = _.extend({}, APIDataValue_DEFAULTS, validationRule);
    return {
        literal: function(data, bind, options) {
            data = data || {};
            options = options || {};
            options.bindArgs = options.bindArgs || [];
            bind = validationRule.bind || bind;
            var value = null;
            switch (typeof bind) {
              case "function":
                validationRule.bindArg && (data = DeclarativeObject(data).get(validationRule.bindArg));
                null == data && (data = validationRule.default);
                data = options.bindArgs.concat(data);
                try {
                    value = bind.apply({}, data);
                } catch (e) {
                    throw new Error(_.template("[APIDataValue] binding function: <%= message %>", {
                        bind: bind,
                        message: e.toString()
                    }));
                }
                break;

              default:
                value = DeclarativeObject(data).get(bind).shift();
            }
            null == value && (value = validationRule.default);
            if (null == value && validationRule.required) throw new Error(_.template("[APIDataValue] <%= bind %> is required", {
                bind: bind
            }));
            if (null != value) {
                var type = typeof value;
                switch (type) {
                  case "number":
                  case "string":
                  case "boolean":
                    break;

                  default:
                    throw new Error(_.template("[APIDataValue] <%= bind %> type is invalid", {
                        bind: bind
                    }));
                }
            }
            return value;
        },
        object: function(data, bind, options) {
            data = data || {};
            options = options || {};
            options.bindArgs = options.bindArgs || [];
            var validationKey = bind;
            try {
                var obj = DeclarativeObject(data);
                var values = obj.get(validationRule.bind);
                var result = DeclarativeObject();
                result.set(bind, values);
                return result.get(bind);
            } catch (e) {
                throw new Error(_.template("[APIDataValue] <%= bind %> type is invalid. <%= inner %>", {
                    bind: bind,
                    inner: e.toString()
                }));
            }
        }
    };
};

var APIData = function(rules) {
    var values = {};
    _.each(rules, function(rule, key) {
        values[key] = new APIDataValue(rule);
    });
    return {
        literals: function(data, options) {
            var result = new DeclarativeObject();
            _.each(values, function(apiValue, key) {
                var value = apiValue.literal(data, key, options);
                null != value && result.set(key, value);
            });
            return result.get();
        },
        complex: function(data, options) {
            var result = new DeclarativeObject();
            _.each(values, function(apiValue, key) {
                var obj = apiValue.object(data, key, options);
                result.set(key, obj);
            });
            return result.get();
        }
    };
};

module.exports = APIData;