"use strict";

var APICallType = require("./APICallType"), APIUrl = require("./APIUrl"), APIData = require("./APIData");

var MIN_TIME = new Date(1985, 11, 5).getTime();

var DefaultErrorData = new APIData({
    status: {
        required: true,
        bind: "status"
    },
    code: {
        required: true,
        bind: "errorCode"
    },
    message: {
        required: true,
        bind: "description"
    }
});

var ApiUrls = {};

ApiUrls[APICallType.SystemDetails] = new APIUrl({
    url: "/api/rtr/v1.0/object/systemDetails",
    verb: "GET",
    urlData: new APIData({}),
    queryData: new APIData({}),
    postData: new APIData({}),
    responseData: new APIData({}),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.StartTrip] = new APIUrl({
    url: "/v1/object/users/SELF/trips/<%= tripId %>/start",
    verb: "POST",
    urlData: new APIData({
        userId: {
            required: true
        },
        tripId: {
            required: false,
            "default": -1
        }
    }),
    queryData: new APIData({
        timestamp: {
            required: true,
            bind: "drive.startTimestamp"
        },
        lon: {
            required: true,
            bind: "drive.startLon"
        },
        lat: {
            required: true,
            bind: "drive.startLat"
        },
        accuracy: {
            required: true,
            bind: "drive.startAccuracy"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        driveId: {
            required: true,
            bind: "id"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.EndTrip] = new APIUrl({
    url: "/v2/users/<%= userId %>/drives/<%= driveId %>/end",
    verb: "POST",
    urlData: new APIData({
        userId: {
            required: true
        },
        driveId: {
            required: true,
            bind: "drive.driveId"
        }
    }),
    queryData: new APIData({
        timestamp: {
            required: true,
            bind: "drive.endTimestamp"
        },
        lon: {
            required: true,
            bind: "drive.endLon"
        },
        lat: {
            required: true,
            bind: "drive.endLat"
        },
        accuracy: {
            required: true,
            bind: "drive.endAccuracy"
        },
        sendPush: {
            "default": true
        },
        dropoffRiders: {
            "default": true
        },
        driveFields: {
            "default": "DISTANCE,RIDES,DRIVER_TOTAL_EARNINGS,ACHIEVEMENTS,REWARDS,REWARD_TOTALS,START_LOCATION,START_ADDRESS,END_LOCATION,END_ADDRESS,UNREALISTIC_CAUSE"
        },
        rideFields: {
            "default": "DURATION,OTHER_USER"
        },
        userFields: {
            "default": "IS_FREE_RIDER,IS_FREE_DRIVER"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        state: {
            required: true,
            bind: "state"
        },
        distance: {
            required: true,
            bind: "distance"
        },
        unrealisticCause: {
            required: true,
            bind: "unrealisticCause",
            "default": ""
        },
        "rides[].driveId": {
            required: true,
            bind: "rides[].driveId"
        },
        "rides[].riderId": {
            required: true,
            bind: "rides[].rider.userId"
        },
        "rides[].duration": {
            required: true,
            bind: "rides[].durationMinutes"
        },
        "rides[].freeRide": {
            required: true,
            bind: "rides[].rider.canUserTakeFreeRide"
        },
        startAddress: {
            required: true,
            bind: "startLocation.address"
        },
        startTimestamp: {
            required: true,
            bind: "startLocation.timestamp"
        },
        endAddress: {
            required: true,
            bind: "endLocation.address"
        },
        endTimestamp: {
            required: true,
            bind: "endLocation.timestamp"
        },
        "earning.amount": {
            required: true,
            bind: "earnings[0].formattedAmount"
        },
        "earning.currency": {
            required: true,
            bind: "earnings[0].currency"
        },
        "rewards[].type": {
            required: false,
            bind: "rewards[].rewardType",
            "default": ""
        },
        "rewards[].iconUrl": {
            required: true,
            bind: "rewards[].iconUrl",
            "default": ""
        },
        "rewards[].amount": {
            required: false,
            bind: "rewards[].rewardAmount.formattedAmount",
            "default": "0"
        },
        "rewards[].currency": {
            required: false,
            bind: "rewards[].rewardAmount.currency",
            "default": "US"
        },
        "rewards[].relatedUserId": {
            required: false,
            bind: "rewards[].relatedUser.userId",
            "default": ""
        },
        "rewards[].relatedUsername": {
            required: false,
            bind: "rewards[].relatedUser.firstName",
            "default": ""
        },
        "rewardTotals[].type": {
            required: false,
            bind: "rewardTotals[].rewardType",
            "default": ""
        },
        "rewardTotals[].amount": {
            required: false,
            bind: "rewardTotals[].totals[0].formattedAmount",
            "default": "0"
        },
        "rewardTotals[].currency": {
            required: false,
            bind: "rewardTotals[].totals[0].currency",
            "default": "US"
        },
        "achievements[].timestamp": {
            required: true,
            bind: "achievements[].timestamp"
        },
        "achievements[].title": {
            required: true,
            bind: "achievements[].title"
        },
        "achievements[].iconUrl": {
            required: true,
            bind: "achievements[].iconUrl",
            "default": ""
        },
        "achievements[].description": {
            required: true,
            bind: "achievements[].description"
        },
        "achievements[].value": {
            required: true,
            bind: "achievements[].value"
        },
        "achievements[].type": {
            required: true,
            bind: "achievements[].type"
        },
        "creditBalance.payableBalance": {
            required: false,
            bind: "creditBalances[0].payableBalance"
        },
        "creditBalance.bonusBalance": {
            required: false,
            bind: "creditBalances[0].bonusBalance"
        },
        "creditBalance.totalBalance": {
            required: false,
            bind: "creditBalances[0].totalBalance"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.JoinCar] = new APIUrl({
    url: "/v1/object/users/<%= driverId %>/riders",
    verb: "POST",
    urlData: new APIData({
        driverId: {
            required: true,
            bind: "drive.driverId"
        }
    }),
    queryData: new APIData({
        riderId: {
            required: true,
            bind: "ride.riderId"
        },
        currentLon: {
            required: true,
            bind: "ride.startLon"
        },
        currentLat: {
            required: true,
            bind: "ride.startLat"
        },
        accuracy: {
            required: true,
            bind: "ride.startAccuracy"
        },
        pickupTimestamp: {
            required: true,
            bind: "ride.startTimestamp"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        "ride.rideId": {
            required: true,
            bind: "id"
        },
        "drive.driverId": {
            required: true,
            bind: "driver.userId"
        },
        "driver.firstName": {
            required: true,
            bind: "driver.firstName"
        },
        "driver.alias": {
            required: true,
            bind: "driver.alias"
        },
        "driver.photoUrl": {
            required: true,
            bind: "driver.photoUrl"
        },
        "ride.startLat": {
            required: true,
            bind: "pickupLocation.latitude"
        },
        "ride.startLon": {
            required: true,
            bind: "pickupLocation.longitude"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.JoinCarByPhoneNumber] = new APIUrl({
    url: "/v1/object/users/<%= riderId %>/joinCarByPhoneNumber",
    verb: "POST",
    urlData: new APIData({
        riderId: {
            required: true,
            bind: "ride.riderId"
        }
    }),
    queryData: new APIData({
        phoneNumber: {
            required: true,
            bind: "ride.driverPhoneNumber"
        },
        pickupLon: {
            required: true,
            bind: "ride.startLon"
        },
        pickupLat: {
            required: true,
            bind: "ride.startLat"
        },
        pickupLocationAccuracy: {
            required: true,
            bind: "ride.startAccuracy"
        },
        pickupTimestamp: {
            required: true,
            bind: "ride.startTimestamp"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        "driver.firstName": {
            required: true,
            bind: "driver.firstName"
        },
        "driver.alias": {
            required: true,
            bind: "driver.alias"
        },
        "driver.photoUrl": {
            required: true,
            bind: "driver.photoUrl"
        },
        "ride.rideId": {
            required: true,
            bind: "id"
        },
        "drive.driveId": {
            required: true,
            bind: "driveId"
        },
        "drive.driverId": {
            required: true,
            bind: "driver.userId"
        },
        "ride.startLat": {
            required: true,
            bind: "pickupLocation.latitude"
        },
        "ride.startLon": {
            required: true,
            bind: "pickupLocation.longitude"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.JoinCarByBeacon] = new APIUrl({
    url: "/v1/object/users/<%= beaconId %>/riders/joinCarByBeacon",
    verb: "POST",
    urlData: new APIData({
        beaconId: {
            required: true,
            bind: "ride.driverBeaconId"
        }
    }),
    queryData: new APIData({
        pickupLon: {
            required: true,
            bind: "ride.startLon"
        },
        pickupLat: {
            required: true,
            bind: "ride.startLat"
        },
        pickupLocationAccuracy: {
            required: true,
            bind: "ride.startAccuracy"
        },
        pickupTimestamp: {
            required: true,
            bind: "ride.startTimestamp"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        "driver.firstName": {
            required: true,
            bind: "driver.firstName"
        },
        "driver.alias": {
            required: true,
            bind: "driver.alias"
        },
        "driver.photoUrl": {
            required: true,
            bind: "driver.photoUrl"
        },
        "ride.rideId": {
            required: true,
            bind: "id"
        },
        "drive.driveId": {
            required: true,
            bind: "driveId"
        },
        "drive.driverId": {
            required: true,
            bind: "driver.userId"
        },
        "ride.startLat": {
            required: true,
            bind: "pickupLocation.latitude"
        },
        "ride.startLon": {
            required: true,
            bind: "pickupLocation.longitude"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.LeaveCar] = new APIUrl({
    url: "/v2/users/<%= driverId %>/dropoff",
    verb: "POST",
    urlData: new APIData({
        driverId: {
            required: true,
            bind: "drive.driverId"
        }
    }),
    queryData: new APIData({
        journeyId: {
            required: true,
            bind: "ride.rideId"
        },
        currentLon: {
            required: true,
            bind: "ride.endLon"
        },
        currentLat: {
            required: true,
            bind: "ride.endLat"
        },
        accuracy: {
            required: true,
            bind: "ride.endAccuracy"
        },
        dropoffTimestamp: {
            required: true,
            bind: "ride.endTimestamp"
        },
        rideFields: {
            "default": "DISTANCE,RIDER_COST,ACHIEVEMENTS,REWARDS,REWARD_TOTALS,PICKUP_LOCATION,PICKUP_ADDRESS,DROPOFF_LOCATION,DROPOFF_ADDRESS,UNREALISTIC_CAUSE,OTHER_USER"
        },
        userFields: {
            "default": "IS_FREE_RIDER,IS_FREE_DRIVER"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        unrealisticCause: {
            required: false,
            bind: "unrealisticCostCause",
            "default": ""
        },
        distance: {
            required: true,
            bind: "distance"
        },
        freeRide: {
            required: true,
            bind: "driver.canUserGiveFreeDrive"
        },
        "ride.rideId": {
            required: true,
            bind: "id"
        },
        "drive.driveId": {
            required: true,
            bind: "driveId"
        },
        startAddress: {
            required: true,
            bind: "pickupLocation.address"
        },
        startTimestamp: {
            required: true,
            bind: "pickupLocation.timestamp"
        },
        endAddress: {
            required: true,
            bind: "dropoffLocation.address"
        },
        endTimestamp: {
            required: true,
            bind: "dropoffLocation.timestamp"
        },
        "earning.currency": {
            required: true,
            bind: "riderCost.currency"
        },
        "earning.amount": {
            required: true,
            bind: "riderCost.formattedAmount"
        },
        "rewards[].type": {
            required: true,
            bind: "rewards[].rewardType"
        },
        "rewards[].iconUrl": {
            required: true,
            bind: "rewards[].iconUrl",
            "default": ""
        },
        "rewards[].amount": {
            required: false,
            bind: "rewards[].rewardAmount.formattedAmount",
            "default": "0"
        },
        "rewards[].currency": {
            required: false,
            bind: "rewards[].rewardAmount.currency",
            "default": "US"
        },
        "rewards[].relatedUserId": {
            required: false,
            bind: "rewards[].relatedUser.userId",
            "default": ""
        },
        "rewards[].relatedUsername": {
            required: false,
            bind: "rewards[].relatedUser.firstName",
            "default": ""
        },
        "rewardTotals[].type": {
            required: true,
            bind: "rewardTotals[].rewardType"
        },
        "rewardTotals[].amount": {
            required: false,
            bind: "rewardTotals[].totals[0].formattedAmount",
            "default": "0"
        },
        "rewardTotals[].currency": {
            required: false,
            bind: "rewardTotals[].totals[0].currency",
            "default": "US"
        },
        "achievements[].timestamp": {
            required: true,
            bind: "achievements[].timestamp"
        },
        "achievements[].title": {
            required: true,
            bind: "achievements[].title"
        },
        "achievements[].iconUrl": {
            required: true,
            bind: "achievements[].iconUrl",
            "default": ""
        },
        "achievements[].description": {
            required: true,
            bind: "achievements[].description"
        },
        "achievements[].value": {
            required: true,
            bind: "achievements[].value"
        },
        "achievements[].type": {
            required: true,
            bind: "achievements[].type"
        },
        "creditBalance.payableBalance": {
            required: false,
            bind: "creditBalances[0].payableBalance"
        },
        "creditBalance.bonusBalance": {
            required: false,
            bind: "creditBalances[0].bonusBalance"
        },
        "creditBalance.totalBalance": {
            required: false,
            bind: "creditBalances[0].totalBalance"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.Status] = new APIUrl({
    url: "/v1/object/users/<%= userId %>/updates/status",
    verb: "GET",
    urlData: new APIData({
        userId: {
            required: true,
            bind: "userId"
        }
    }),
    queryData: new APIData({
        since: {
            required: true,
            bind: "localServerStatus.updatedOn",
            "default": MIN_TIME
        },
        includeAll: {
            required: true,
            "default": true
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        updatedOn: {
            required: true,
            bind: "startProcessingTimestamp"
        },
        hasStartedRides: {
            required: true,
            bind: "hasStartedRides"
        },
        hasFinishedRides: {
            required: true,
            bind: "hasFinishedRides"
        },
        hasStartedDrives: {
            required: true,
            bind: "hasStartedDrives"
        },
        hasFinishedDrives: {
            required: true,
            bind: "hasFinishedDrives"
        }
    }),
    errorData: new APIData({
        code: {
            required: true,
            bind: "error"
        },
        message: {
            required: true,
            bind: "error_description"
        }
    })
});

ApiUrls[APICallType.ActiveDrive] = new APIUrl({
    url: "/v1/object/users/SELF/activeJourneys",
    verb: "GET",
    urlData: new APIData({}),
    queryData: new APIData({
        userFields: {
            required: true,
            "default": "IS_FREE_RIDER,IS_FREE_DRIVER"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        "drives[].driveId": {
            required: true,
            bind: "drives[].id"
        },
        "drives[].driverId": {
            required: true,
            bind: "drives[].driverId"
        },
        "drives[].startTimestamp": {
            required: true,
            bind: "drives[].startLocation.timestamp"
        },
        "drives[].startLat": {
            required: true,
            bind: "drives[].startLocation.latitude"
        },
        "drives[].startLon": {
            required: true,
            bind: "drives[].startLocation.longitude"
        },
        "drives[].startAccuracy": {
            required: true,
            bind: "drives[].startLocation.accuracy"
        },
        "rides[].rideId": {
            required: true,
            bind: "rides[].id"
        },
        "rides[].riderId": {
            required: true,
            bind: "rides[].rider.userId"
        },
        "rides[].driveId": {
            required: true,
            bind: "rides[].driveId"
        },
        "rides[].driverId": {
            required: true,
            bind: "rides[].driver.userId"
        },
        "rides[].freeRide": {
            required: true,
            bind: "rides[].driver.canUserGiveFreeDrive"
        },
        "rides[].startTimestamp": {
            required: true,
            bind: "rides[].pickupTimestamp"
        },
        "rides[].startAccuracy": {
            required: true,
            bind: "rides[].pickupLocation.accuracy"
        },
        "rides[].startLat": {
            required: true,
            bind: "rides[].pickupLocation.latitude"
        },
        "rides[].startLon": {
            required: true,
            bind: "rides[].pickupLocation.longitude"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.FullUserProfile] = new APIUrl({
    url: "/api/rtr/v2/object/users/<%=targetUserId%>",
    verb: "GET",
    urlData: new APIData({
        targetUserId: {
            required: true,
            bind: "targetUserId"
        }
    }),
    queryData: new APIData({
        userFields: {
            required: true,
            "default": "NAME,ALIAS,HOME_CITY,PHOTO_URL,GENDER,GROUPS,IS_FAVOURITE,REGISTRATION_TIME,RATING,WEBPAGE,BIO,CREDIT_BALANCES,LAST_KNOWN_LOCATION_ADDRESS_STREET,LAST_SEEN_TIMESTAMP,PHONE_NUMBER,IS_FREE_RIDER,IS_FREE_DRIVER,NUM_FAVOURITES,LAST_CHANGED_TIMESTAMP_BASIC,VALIDATIONS_EMAIL,VALIDATIONS_PHONE,VALIDATIONS_NUM_POSITIVE_REVIEWS,VALIDATIONS_CONNECTED_FACEBOOK,VALIDATIONS_AFFILIATES,GROUPS,RIDE_COUNTS,MESSAGING_STATS,VEHICLES,BEACONS"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        id: {
            required: true,
            bind: "userId"
        },
        alias: {
            required: true,
            bind: "alias"
        },
        phoneNumber: {
            required: false,
            bind: "phoneNumber"
        },
        firstName: {
            required: false,
            bind: "firstName"
        },
        lastName: {
            required: false,
            bind: "lastName"
        },
        gender: {
            required: true,
            bind: "gender"
        },
        photoUrl: {
            required: true,
            bind: "photoUrl"
        },
        webpage: {
            required: false,
            bind: "webpage"
        },
        bio: {
            required: false,
            bind: "bio"
        },
        homeCity: {
            required: false,
            bind: "homeCity.name"
        },
        homeCountry: {
            required: false,
            bind: "homeCity.country.name"
        },
        registrationTime: {
            required: false,
            bind: "registrationTime"
        },
        lastKnownLocation: {
            required: false,
            bind: "lastKnownLocation.address"
        },
        lastSeen: {
            required: false,
            bind: "lastKnownLocation.timestamp"
        },
        isFavourite: {
            required: true,
            bind: "isFavourite"
        },
        numFavourites: {
            required: true,
            bind: "numFavourites"
        },
        canUserTakeFreeRide: {
            required: true,
            bind: "canUserTakeFreeRide"
        },
        canUserGiveFreeDrive: {
            required: true,
            bind: "canUserGiveFreeDrive"
        },
        avgUserRating: {
            required: true,
            bind: "avgUserRating"
        },
        numUserRatings: {
            required: true,
            bind: "numUserRatings"
        },
        numUserComments: {
            required: true,
            bind: "numUserComments"
        },
        numSharedRides: {
            required: true,
            bind: "numSharedRides"
        },
        numSharedDrives: {
            required: true,
            bind: "numSharedDrives"
        },
        "groups[].id": {
            required: false,
            bind: "groups[].id"
        },
        "creditBalance.currency": {
            required: false,
            bind: "creditBalances[0].currency"
        },
        "creditBalance.currencySymbol": {
            required: false,
            bind: "creditBalances[0].currencySymbol"
        },
        "creditBalance.payableBalance": {
            required: false,
            bind: "creditBalances[0].payableBalance"
        },
        "creditBalance.bonusBalance": {
            required: false,
            bind: "creditBalances[0].bonusBalance"
        },
        "creditBalance.totalBalance": {
            required: false,
            bind: "creditBalances[0].totalBalance"
        },
        credit: {
            required: false,
            bind: "credit"
        },
        "messagingStats.firstResponsePercentage": {
            required: true,
            bind: "messagingStats.firstResponsePercentage"
        },
        "messagingStats.averageResponseTimeInMilliseconds": {
            required: true,
            bind: "messagingStats.averageResponseTimeInMilliseconds"
        },
        "messagingStats.hasUserEverReceivedMessage": {
            required: true,
            bind: "messagingStats.hasUserEverReceivedMessage"
        },
        "validations.phoneValidated": {
            required: false,
            bind: "validations.phoneValidated"
        },
        "validations.emailValidated": {
            required: false,
            bind: "validations.emailValidated"
        },
        "validations.numPositiveReviews": {
            required: false,
            bind: "validations.numPositiveReviews"
        },
        "validations.connectedFacebook": {
            required: false,
            bind: "validations.connectedFacebook"
        },
        "validations.numberOfFaceBookFriends": {
            required: false,
            bind: "validations.numberOfFaceBookFriends"
        },
        "affiliateNames[]": {
            required: false,
            bind: "validations.affiliateNames[]"
        },
        "vehicles[].id": {
            required: false,
            bind: "vehicles[].id"
        },
        "vehicles[].licensePlateRegion": {
            required: false,
            bind: "vehicles[].licensePlateRegion"
        },
        "vehicles[].licensePlateNumber": {
            required: false,
            bind: "vehicles[].licensePlateNumber"
        },
        "vehicles[].tollTagId": {
            required: false,
            bind: "vehicles[].tollTagId"
        },
        "vehicles[].tollTagType": {
            required: false,
            bind: "vehicles[].tollTagType"
        },
        "beacons[].id": {
            required: false,
            bind: "beacons[].id"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.BasicUserProfile] = new APIUrl({
    url: "/api/rtr/v2/object/users/<%=targetUserId%>",
    verb: "GET",
    urlData: new APIData({
        targetUserId: {
            required: true,
            bind: "targetUserId"
        }
    }),
    queryData: new APIData({
        userFields: {
            required: true,
            "default": "NAME,ALIAS,PHOTO_URL,GENDER,IS_FAVOURITE,IS_FREE_RIDER,IS_FREE_DRIVER"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        firstName: {
            required: true,
            bind: "firstName"
        },
        lastName: {
            required: true,
            bind: "lastName"
        },
        id: {
            required: true,
            bind: "userId"
        },
        alias: {
            required: true,
            bind: "alias"
        },
        gender: {
            required: true,
            bind: "gender"
        },
        photoUrl: {
            required: true,
            bind: "photoUrl"
        },
        isFavourite: {
            required: true,
            bind: "isFavourite"
        },
        canUserTakeFreeRide: {
            required: true,
            bind: "canUserTakeFreeRide"
        },
        canUserGiveFreeDrive: {
            required: true,
            bind: "canUserGiveFreeDrive"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.GetListOfProfiles] = new APIUrl({
    url: "/api/rtr/v1.0/object/users/profiles/",
    verb: "GET",
    urlData: new APIData({}),
    queryData: new APIData({
        ids: {
            required: true,
            bind: "uids"
        },
        userFields: {
            required: true,
            "default": "NAME,ALIAS,PHOTO_URL,GENDER,IS_FAVOURITE,IS_FREE_RIDER,IS_FREE_DRIVER"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        "users[].firstName": {
            required: true,
            bind: "users[].firstName"
        },
        "users[].lastName": {
            required: true,
            bind: "users[].lastName"
        },
        "users[].id": {
            required: true,
            bind: "users[].userId"
        },
        "users[].userId": {
            required: true,
            bind: "users[].userId"
        },
        "users[].alias": {
            required: true,
            bind: "users[].alias"
        },
        "users[].gender": {
            required: true,
            bind: "users[].gender"
        },
        "users[].photoUrl": {
            required: true,
            bind: "users[].photoUrl"
        },
        "users[].isFavourite": {
            required: true,
            bind: "users[].isFavourite"
        },
        "users[].canUserTakeFreeRide": {
            required: true,
            bind: "users[].canUserTakeFreeRide"
        },
        "users[].canUserGiveFreeDrive": {
            required: true,
            bind: "users[].canUserGiveFreeDrive"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.GetTrips] = new APIUrl({
    url: "/api/rtr/v2/users/<%= targetUserId %>/trips",
    verb: "GET",
    urlData: new APIData({
        targetUserId: {
            required: true,
            bind: "targetUserId"
        }
    }),
    queryData: new APIData({
        tripFields: {
            required: true,
            "default": "LOCATION_ALIASES,ALIAS,USER_ROLE,ESTIMATED_EARNINGCOST,SCHEDULE,BOOKINGS"
        },
        relativeTime: {
            required: true,
            bind: "relativeTime"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        "trips[].id": {
            required: true,
            bind: "trips[].id"
        },
        "trips[].schedule.type": {
            required: true,
            bind: "trips[].schedule.type"
        },
        "trips[].schedule.daysOfOperation[]": {
            required: true,
            bind: "trips[].schedule.daysOfOperation[]"
        },
        "trips[].schedule.startMinutes": {
            required: true,
            bind: "trips[].schedule.startMinutes"
        },
        "trips[].schedule.timeZone": {
            required: true,
            bind: "trips[].schedule.timeZone"
        },
        "trips[].schedule.effectiveTime": {
            required: false,
            bind: "trips[].schedule.effectiveTime"
        },
        "trips[].schedule.baseEffectiveTime": {
            required: false,
            bind: "trips[].schedule.baseEffectiveTime"
        },
        "trips[].userRole": {
            required: true,
            bind: "trips[].userRole"
        },
        "trips[].enabled": {
            required: true,
            bind: "trips[].enabled"
        },
        "trips[].estimatedEarning.currency": {
            required: true,
            bind: "trips[].estimatedEarning.currency"
        },
        "trips[].estimatedEarning.formattedAmount": {
            required: true,
            bind: "trips[].estimatedEarning.formattedAmount"
        },
        "trips[].estimatedEarning.amount": {
            required: true,
            bind: "trips[].estimatedEarning.amount"
        },
        "trips[].estimatedCost.currency": {
            required: true,
            bind: "trips[].estimatedCost.currency"
        },
        "trips[].estimatedCost.formattedAmount": {
            required: true,
            bind: "trips[].estimatedCost.formattedAmount"
        },
        "trips[].estimatedCost.amount": {
            required: true,
            bind: "trips[].estimatedCost.amount"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.BatchAPICall] = new APIUrl({
    url: "/api/rtr/v1/object/batchApiCall",
    verb: "POST"
});

ApiUrls[APICallType.DriveHistory] = new APIUrl({
    url: "/v2/users/<%= driverId %>/driveHistory/<%= driveId %>",
    verb: "GET",
    urlData: new APIData({
        driveId: {
            required: true,
            bind: "drive.driveId"
        },
        driverId: {
            required: true,
            bind: "drive.driverId"
        }
    }),
    queryData: new APIData({
        driveFields: {
            "default": "DISTANCE,DRIVER_TOTAL_EARNINGS,RIDES,ACHIEVEMENTS,REWARDS,REWARD_TOTALS,START_LOCATION,START_ADDRESS,END_LOCATION,END_ADDRESS,NUM_RIDERS,UNREALISTIC_CAUSE"
        },
        rideFields: {
            "default": "DURATION,OTHER_USER"
        },
        userFields: {
            "default": "IS_FREE_RIDER,IS_FREE_DRIVER"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        distance: {
            required: true,
            bind: "distance"
        },
        unrealisticCause: {
            required: false,
            bind: "unrealisticCause",
            "default": ""
        },
        startAddress: {
            required: true,
            bind: "startLocation.address"
        },
        startTimestamp: {
            required: true,
            bind: "startLocation.timestamp"
        },
        endAddress: {
            required: true,
            bind: "endLocation.address"
        },
        endTimestamp: {
            required: true,
            bind: "endLocation.timestamp"
        },
        "rides[].driveId": {
            required: false,
            bind: "rides[].driveId"
        },
        "rides[].riderId": {
            required: false,
            bind: "rides[].rider.userId"
        },
        "rides[].duration": {
            required: true,
            bind: "rides[].durationMinutes"
        },
        "rides[].freeRide": {
            required: false,
            bind: "rides[].rider.canUserTakeFreeRide"
        },
        "earning.currency": {
            required: true,
            bind: "earnings[0].currency"
        },
        "earning.amount": {
            required: true,
            bind: "earnings[0].formattedAmount"
        },
        "rewards[].type": {
            required: true,
            bind: "rewards[].rewardType"
        },
        "rewards[].iconUrl": {
            required: true,
            bind: "rewards[].iconUrl",
            "default": ""
        },
        "rewards[].amount": {
            required: false,
            bind: "rewards[].rewardAmount.formattedAmount",
            "default": "0"
        },
        "rewards[].currency": {
            required: false,
            bind: "rewards[].rewardAmount.currency",
            "default": "US"
        },
        "rewards[].relatedUserId": {
            required: false,
            bind: "rewards[].relatedUser.userId",
            "default": ""
        },
        "rewards[].relatedUsername": {
            required: false,
            bind: "rewards[].relatedUser.firstName",
            "default": ""
        },
        "rewardTotals[].type": {
            required: true,
            bind: "rewardTotals[].rewardType"
        },
        "rewardTotals[].amount": {
            required: false,
            bind: "rewardTotals[].totals[0].formattedAmount",
            "default": "0"
        },
        "rewardTotals[].currency": {
            required: false,
            bind: "rewardTotals[].totals[0].currency",
            "default": "US"
        },
        "achievements[].timestamp": {
            required: true,
            bind: "achievements[].timestamp"
        },
        "achievements[].title": {
            required: true,
            bind: "achievements[].title"
        },
        "achievements[].iconUrl": {
            required: true,
            bind: "achievements[].iconUrl",
            "default": ""
        },
        "achievements[].description": {
            required: true,
            bind: "achievements[].description"
        },
        "achievements[].value": {
            required: true,
            bind: "achievements[].value"
        },
        "achievements[].type": {
            required: true,
            bind: "achievements[].type"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.RideHistory] = new APIUrl({
    url: "/v2/users/<%= userId %>/rideHistory/<%= driveId %>",
    verb: "GET",
    urlData: new APIData({
        userId: {
            required: true
        },
        driveId: {
            required: true,
            bind: "drive.rides[0].rideId"
        }
    }),
    queryData: new APIData({
        rideFields: {
            "default": "DISTANCE,DRIVER_EARNING,RIDER_COST,ACHIEVEMENTS,REWARDS,REWARD_TOTALS,PICKUP_LOCATION,PICKUP_ADDRESS,DROPOFF_LOCATION,DROPOFF_ADDRESS,OTHER_USER,UNREALISTIC_CAUSE"
        },
        userFields: {
            "default": "IS_FREE_RIDER,IS_FREE_DRIVER"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        distance: {
            required: true,
            bind: "distance"
        },
        unrealisticCause: {
            required: false,
            bind: "unrealisticCostCause",
            "default": ""
        },
        freeRide: {
            required: false,
            bind: "driver.canUserGiveFreeDrive",
            "default": false
        },
        startAddress: {
            required: true,
            bind: "pickupLocation.address"
        },
        startTimestamp: {
            required: true,
            bind: "pickupLocation.timestamp"
        },
        endAddress: {
            required: true,
            bind: "dropoffLocation.address"
        },
        endTimestamp: {
            required: true,
            bind: "dropoffLocation.timestamp"
        },
        "earning.currency": {
            required: true,
            bind: "riderCost.currency"
        },
        "earning.amount": {
            required: true,
            bind: "riderCost.formattedAmount"
        },
        "rewards[].type": {
            required: true,
            bind: "rewards[].rewardType"
        },
        "rewards[].iconUrl": {
            required: true,
            bind: "rewards[].iconUrl",
            "default": ""
        },
        "rewards[].amount": {
            required: false,
            bind: "rewards[].rewardAmount.formattedAmount",
            "default": "0"
        },
        "rewards[].currency": {
            required: false,
            bind: "rewards[].rewardAmount.currency",
            "default": "US"
        },
        "rewards[].relatedUserId": {
            required: false,
            bind: "rewards[].relatedUser.userId",
            "default": ""
        },
        "rewards[].relatedUsername": {
            required: false,
            bind: "rewards[].relatedUser.firstName",
            "default": ""
        },
        "rewardTotals[].type": {
            required: true,
            bind: "rewardTotals[].rewardType"
        },
        "rewardTotals[].amount": {
            required: false,
            bind: "rewardTotals[].totals[0].formattedAmount",
            "default": "0"
        },
        "rewardTotals[].currency": {
            required: false,
            bind: "rewardTotals[].totals[0].currency",
            "default": "US"
        },
        "achievements[].timestamp": {
            required: true,
            bind: "achievements[].timestamp"
        },
        "achievements[].title": {
            required: true,
            bind: "achievements[].title"
        },
        "achievements[].iconUrl": {
            required: true,
            bind: "achievements[].iconUrl",
            "default": ""
        },
        "achievements[].description": {
            required: true,
            bind: "achievements[].description"
        },
        "achievements[].value": {
            required: true,
            bind: "achievements[].value"
        },
        "achievements[].type": {
            required: true,
            bind: "achievements[].type"
        }
    }),
    errorData: DefaultErrorData
});

ApiUrls[APICallType.GetCreditCards] = new APIUrl({
    url: "/api/rtr/v1.0/object/users/<%= userId %>/paymentTokens/<%= paymentProvider %>",
    verb: "GET",
    urlData: new APIData({
        userId: {
            required: true,
            bind: "userId"
        },
        paymentProvider: {
            required: true,
            bind: "paymentProvider"
        }
    }),
    postData: new APIData({}),
    responseData: new APIData({
        "tokens[].alias": {
            required: true,
            bind: "tokens[].alias"
        },
        "tokens[].brand": {
            required: true,
            bind: "tokens[].brand"
        },
        "tokens[].id": {
            required: true,
            bind: "tokens[].id"
        },
        "tokens[].metadata": {
            required: true,
            bind: "tokens[].metadata"
        },
        "tokens[].paymentProvider": {
            required: true,
            bind: "tokens[].paymentProvider"
        },
        "tokens[].providerAccountCurrencyCode": {
            required: true,
            bind: "tokens[].providerAccountCurrencyCode"
        },
        "tokens[].status": {
            required: true,
            bind: "tokens[].status"
        },
        "tokens[].timestamp": {
            required: true,
            bind: "tokens[].timestamp"
        },
        "tokens[].token": {
            required: true,
            bind: "tokens[].token"
        },
        "tokens[].userId": {
            required: true,
            bind: "tokens[].userId"
        }
    }),
    errorData: DefaultErrorData
});

module.exports = ApiUrls;