"use strict";

var TimeInfo = require("../models/TimeInfo"), TimeInfoCollection = require("../models/TimeInfoCollection");

var SERVERTIME_EXPIRES_IN_MSEC = 864e5;

var TimeManager = function(options) {
    options = options || {};
    options.serverTimeExpiresIn = options.serverTimeExpiresIn >= 0 ? options.serverTimeExpiresIn : SERVERTIME_EXPIRES_IN_MSEC;
    var timeInfoModel = new TimeInfoCollection().fetch().pop();
    timeInfoModel || (timeInfoModel = new TimeInfo({}).save());
    var manager = {
        getServerTime: function() {
            return timeInfoModel.get("serverTime");
        },
        setServerTime: function(date) {
            date instanceof Date && (date = date.getTime());
            date = Math.floor(date);
            timeInfoModel.set({
                serverTime: date,
                updatedOn: Date.now()
            }).save();
        },
        isServerTimeValid: function() {
            var now = Date.now(), createdOn = timeInfoModel.get("createdOn"), updatedOn = timeInfoModel.get("updatedOn");
            if (createdOn == updatedOn) return false;
            if (Math.abs(now - updatedOn) > options.serverTimeExpiresIn) return false;
            return true;
        },
        getOffset: function() {
            return manager.isServerTimeValid() ? timeInfoModel.get("serverTime") - timeInfoModel.get("updatedOn") : 0;
        },
        getTime: function(date) {
            date instanceof Date && (date = date.getTime());
            var offset = manager.getOffset();
            return date + offset;
        }
    };
    return manager;
};

module.exports = TimeManager;