"use strict";

var _ = require("/vendor/underscore"), NetworkError = require("/v6/common/errors/NetworkError");

var validateStrategyExistence = function(strategyObj, strategyName) {
    strategyName = strategyName || "";
    if (!strategyObj[strategyName]) throw new Error(_.template("Invalid strategy: <%= name %>", {
        name: strategyName
    }));
};

var APICallResponseStrategy = function(strategies, logger) {
    if (!strategies) throw new Error("Strategies are required!");
    if (!logger) throw new Error("Logger is required!");
    return {
        execute: function(apiCall, data) {
            var strategyName = apiCall.type;
            if (data instanceof Error) {
                var err = data;
                if (err instanceof NetworkError && err.isOffline) throw new Error(_.template("Cannot execute strategy because the device is offline", {
                    type: apiCall.type
                }));
                logger.log(_.template("APIUrls.<%= type %> Error", {
                    type: apiCall.type
                }));
                logger.log(_.template("APIUrls.<%= type %> <%= error %>", {
                    type: apiCall.type,
                    error: err.toString()
                }));
                validateStrategyExistence(strategies.error, strategyName);
                return strategies.error[strategyName](apiCall, err);
            }
            logger.log(_.template("APIUrls.<%= type %> Success", {
                type: apiCall.type
            }));
            validateStrategyExistence(strategies.success, strategyName);
            return strategies.success[strategyName](apiCall, data);
        }
    };
};

module.exports = APICallResponseStrategy;