"use strict";

var _ = require("/vendor/underscore"), EventEmitter = require("/v6/infrastructure/EventEmitter/Extend"), Globals = require("/v6/infrastructure/Globals"), completenessService = Globals.IOC.get("completenessService"), exploreService = Globals.IOC.get("exploreService");

var _weeklySchedule = function(hour, min, weekdays) {
    var date = new Date();
    var firstAvailableDay = -1;
    var currentDayOfWeek = date.getDay();
    var currentDate = date.getDate();
    var daysOfOperation = [];
    for (var i = 0; i < weekdays.length; i++) weekdays[i] && -1 === firstAvailableDay ? firstAvailableDay = i : daysOfOperation.push(weekdays[i]);
    var dayGap = currentDayOfWeek - firstAvailableDay;
    date.setDate(currentDate - dayGap);
    while (firstAvailableDay >= 1) {
        var value = daysOfOperation.shift();
        daysOfOperation.push(value);
        firstAvailableDay--;
    }
    date.setHours(hour);
    date.setMinutes(min);
    return {
        date: date.getTime(),
        daysOfOperation: daysOfOperation
    };
};

var TripService = function() {
    var eventDispatcher = new EventEmitter();
    var _requestTimestamp = null, _executeSaveCallback = false, _executeDeleteCallback = false;
    var _resolveCallbacks = function(localRequestTimestamp) {
        if (localRequestTimestamp !== _requestTimestamp) return;
        _executeDeleteCallback ? _requestCallback({
            deleteTrip: true
        }) : _executeSaveCallback && _requestCallback();
        _executeSaveCallback = false;
        _executeDeleteCallback = false;
    };
    var _requestCallback = function(args) {
        args = args || {};
        eventDispatcher.emit(self.TRIP_UPDATED_EVENT, args);
        completenessService.refresh();
    };
    var self = {
        TRIP_UPDATED_EVENT: "trip_updated",
        getTrip: function(tripId) {
            if (!tripId) {
                LOGGER.error("[TripService] - getTrip, no tripId provided");
                return;
            }
            return Alloy.createModel("trip").getTrip(tripId);
        },
        getTripV2: function(tripId, options) {
            if (!tripId) {
                LOGGER.error("[TripService] - getTripV2, no tripId provided");
                return;
            }
            options = options || {};
            options.tripFields = "LOCATIONS,LOCATION_ADDRESSES,LOCATION_ALIASES,USER_ROLE,EFFECTIVE_TIME,BOOKINGS";
            options.tripId = tripId;
            return Alloy.createModel("tripV2").getTrip(options);
        },
        loadTrips: function(options) {
            options = options || {};
            return Alloy.createModel("trips").getTrips(options);
        },
        saveTrip: function(attrs, options) {
            attrs = attrs || {};
            options = options || {};
            _requestTimestamp = Date.now();
            var localRequestTimestamp = _requestTimestamp;
            return Alloy.createModel("trip").saveTrip(attrs, options).then(function(data) {
                exploreService.resetSearchType();
                _executeSaveCallback = true;
                return data;
            }).finally(function(data) {
                _resolveCallbacks(localRequestTimestamp);
                return data;
            });
        },
        deleteTrip: function(tripId) {
            if (!tripId) {
                LOGGER.error("[TripService] - deleteTrip, no tripId provided");
                return;
            }
            _requestTimestamp = Date.now();
            var localRequestTimestamp = _requestTimestamp;
            return Alloy.createModel("trip", {
                id: tripId
            }).deleteTrip(tripId).then(function(data) {
                exploreService.resetSearchType();
                _executeDeleteCallback = true;
                return data;
            }).finally(function(data) {
                _resolveCallbacks(localRequestTimestamp);
                return data;
            });
        },
        createSchedule: function(daysOfOperation, selectTime) {
            var hour = selectTime.getHours();
            var minute = selectTime.getMinutes();
            var weeklySchedule = _weeklySchedule(hour, minute, daysOfOperation);
            return {
                type: "REPEATING_WEEKLY",
                daysOfOperation: weeklySchedule.daysOfOperation,
                startTime: weeklySchedule.date
            };
        },
        createOneOffSchedule: function(selectedDate) {
            var date = new Date(), selectTime = selectedDate;
            selectTime.getMonth() < date.getMonth() && selectTime.setFullYear(selectTime.getFullYear() + 1);
            return {
                type: "ONE_OFF",
                daysOfOperation: [ false, false, false, false, false, false ],
                startTime: selectTime.getTime()
            };
        },
        destroy: function() {
            eventDispatcher.removeEvent();
        },
        on: function(event, callback) {
            eventDispatcher.on(event, callback);
        },
        removeAllListeners: function() {
            eventDispatcher.removeAllListeners();
        }
    };
    return self;
};

TripService.$isRevealingModule = true;

module.exports = TripService;