"use strict";

var ObjectValidator = require("/validator/object_validator"), _gpsPositionMaxAccuracy = 5e3, _gpsPositionMaxTTL = 6e5, _tripMaxAccuracy = 3e3, _tripMaxTTL = 12e4;

var firstAcceptableTS = null;

module.exports = {
    isGPSPositionValid: function(location, ignoreTimestamp) {
        if (!location) return false;
        if (0 === location.lat && 0 === location.lon) return false;
        var accuracy = location.accuracy, timestamp = location.timestamp;
        if (ObjectValidator.isEmpty(accuracy) || 0 > accuracy || accuracy > _gpsPositionMaxAccuracy) {
            LOGGER.info("[GPS Manager] - LocationValidator - not accurate enough: " + accuracy + " Max: " + _gpsPositionMaxAccuracy);
            return false;
        }
        if (ObjectValidator.isEmpty(timestamp) || Date.now() - timestamp > _gpsPositionMaxTTL) {
            if (!ignoreTimestamp) {
                LOGGER.debug("[GPS Manager] - LocationValidator - position expired: " + timestamp + " - Now: " + Date.now());
                return false;
            }
            LOGGER.info("[GPS Manager] - LocationValidator - position expired: " + timestamp + " - Now: " + Date.now() + " - But was told to ignore");
        }
        return true;
    },
    isGPSPositionMoved: function(newLocation, oldLocation) {
        var newLat = newLocation.lat, newLon = newLocation.lon, newAccuracy = newLocation.accuracy, newTimestamp = newLocation.timestamp, oldLat = oldLocation.lat, oldLon = oldLocation.lon, oldAccuracy = oldLocation.accuracy, oldTimestamp = oldLocation.timestamp;
        if (!module.exports.isGPSPositionValid(newLocation)) {
            LOGGER.info("[GPS Manager] - LocationValidator - not valid position");
            return false;
        }
        if (oldTimestamp > newTimestamp) {
            LOGGER.info("[GPS Manager] - LocationValidator - old location");
            return false;
        }
        if (newLat === oldLat && newLon === oldLon && newAccuracy === oldAccuracy && 6e4 > newTimestamp - oldTimestamp) {
            LOGGER.info("[GPS Manager] - LocationValidator - same location");
            return false;
        }
        LOGGER.debug("LocationValidator - moved");
        return true;
    },
    isLocationValidForPickupDropoff: function(location, strict) {
        if (!location) {
            LOGGER.info("[GPS Manager] - LocationValidator - LOCATION NOT VALID FOR PICK-UP/DROP-OFF: No Location");
            return false;
        }
        var accuracy = location.accuracy, timestamp = location.timestamp, lat = location.lat, lon = location.lon;
        if (ObjectValidator.isEmpty(accuracy) || ObjectValidator.isEmpty(timestamp) || ObjectValidator.isEmpty(lat) || ObjectValidator.isEmpty(lon)) {
            LOGGER.error("[GPS Manager] - LocationValidator - LOCATION NOT VALID FOR PICK-UP/DROP-OFF: Invalid: " + JSON.stringify(location));
            return false;
        }
        if (!location.userGenerated && 0 > accuracy || accuracy > _tripMaxAccuracy) {
            LOGGER.error("[GPS Manager] - LocationValidator - LOCATION NOT VALID FOR PICK-UP/DROP-OFF: Bad accuracy: " + JSON.stringify(location));
            return false;
        }
        if (strict && Date.now() - timestamp > _tripMaxTTL) {
            LOGGER.error("[GPS Manager] - LocationValidator - LOCATION NOT VALID FOR PICK-UP/DROP-OFF: Manual and too old (" + (Date.now() - timestamp) + "ms): " + JSON.stringify(location));
            return false;
        }
        return true;
    },
    getLocationCollectionBegin: function() {
        return firstAcceptableTS;
    },
    setLocationCollectionBegin: function() {
        firstAcceptableTS = Date.now();
    },
    setLocationCollectionEnd: function() {
        firstAcceptableTS = null;
    }
};