!function(define) {
    define(function() {
        function meld(target, pointcut, aspect) {
            var pointcutType, remove;
            if (arguments.length < 3) return addAspectToFunction(target, pointcut);
            if (isArray(pointcut)) remove = addAspectToAll(target, pointcut, aspect); else {
                pointcutType = typeof pointcut;
                "string" === pointcutType ? "function" == typeof target[pointcut] && (remove = addAspectToMethod(target, pointcut, aspect)) : remove = "function" === pointcutType ? addAspectToAll(target, pointcut(target), aspect) : addAspectToMatches(target, pointcut, aspect);
            }
            return remove;
        }
        function Advisor(target, func) {
            var orig, advisor, advised;
            this.target = target;
            this.func = func;
            this.aspects = {};
            orig = this.orig = target[func];
            advisor = this;
            advised = this.advised = function() {
                function callOrigAndOn(args) {
                    var result = callOrig(args);
                    advisor._callSimpleAdvice("on", context, args);
                    return result;
                }
                function callAfter(afterType, args) {
                    advisor._callSimpleAdvice(afterType, context, args);
                }
                var context, joinpoint, args, callOrig, afterType;
                if (this instanceof advised) {
                    context = objectCreate(orig.prototype);
                    callOrig = function(args) {
                        return applyConstructor(orig, context, args);
                    };
                } else {
                    context = this;
                    callOrig = function(args) {
                        return orig.apply(context, args);
                    };
                }
                args = slice.call(arguments);
                afterType = "afterReturning";
                joinpoint = pushJoinpoint({
                    target: context,
                    method: func,
                    args: args
                });
                try {
                    advisor._callSimpleAdvice("before", context, args);
                    try {
                        joinpoint.result = advisor._callAroundAdvice(context, func, args, callOrigAndOn);
                    } catch (e) {
                        joinpoint.result = joinpoint.exception = e;
                        afterType = "afterThrowing";
                    }
                    args = [ joinpoint.result ];
                    callAfter(afterType, args);
                    callAfter("after", args);
                    if (joinpoint.exception) throw joinpoint.exception;
                    return joinpoint.result;
                } finally {
                    popJoinpoint();
                }
            };
            defineProperty(advised, "_advisor", {
                value: advisor,
                configurable: true
            });
        }
        function addAspectToFunction(func, aspect) {
            var name, placeholderTarget;
            name = func.name || "_";
            placeholderTarget = {};
            placeholderTarget[name] = func;
            addAspectToMethod(placeholderTarget, name, aspect);
            return placeholderTarget[name];
        }
        function addAspectToMethod(target, method, aspect) {
            var advisor = Advisor.get(target, method);
            return advisor && advisor.add(aspect);
        }
        function addAspectToAll(target, methodArray, aspect) {
            var removers, added, f, i;
            removers = [];
            i = 0;
            while (f = methodArray[i++]) {
                added = addAspectToMethod(target, f, aspect);
                added && removers.push(added);
            }
            return createRemover(removers);
        }
        function addAspectToMatches(target, pointcut, aspect) {
            var removers = [];
            for (var p in target) "function" == typeof target[p] && pointcut.test(p) && removers.push(addAspectToMethod(target, p, aspect));
            return createRemover(removers);
        }
        function createRemover(removers) {
            return {
                remove: function() {
                    for (var i = removers.length - 1; i >= 0; --i) removers[i].remove();
                }
            };
        }
        function adviceApi(type) {
            return function(target, method, adviceFunc) {
                var aspect = {};
                if (2 === arguments.length) {
                    aspect[type] = method;
                    return meld(target, aspect);
                }
                aspect[type] = adviceFunc;
                return meld(target, method, aspect);
            };
        }
        function insertAspect(aspectList, aspect) {
            var adviceType, advice, advices;
            for (adviceType in iterators) {
                advice = aspect[adviceType];
                if (advice) {
                    advices = aspectList[adviceType];
                    advices || (aspectList[adviceType] = advices = []);
                    advices.push({
                        aspect: aspect,
                        advice: advice
                    });
                }
            }
        }
        function removeAspect(aspectList, aspect) {
            var adviceType, advices, remaining;
            remaining = 0;
            for (adviceType in iterators) {
                advices = aspectList[adviceType];
                if (advices) {
                    remaining += advices.length;
                    for (var i = advices.length - 1; i >= 0; --i) if (advices[i].aspect === aspect) {
                        advices.splice(i, 1);
                        --remaining;
                        break;
                    }
                }
            }
            return remaining;
        }
        function applyConstructor(C, instance, args) {
            try {
                defineProperty(instance, "constructor", {
                    value: C,
                    enumerable: false
                });
            } catch (e) {}
            C.apply(instance, args);
            return instance;
        }
        function forEach(array, func) {
            for (var i = 0, len = array.length; len > i; i++) func(array[i]);
        }
        function forEachReverse(array, func) {
            for (var i = array.length - 1; i >= 0; --i) func(array[i]);
        }
        function joinpoint() {
            return currentJoinpoint;
        }
        function pushJoinpoint(newJoinpoint) {
            joinpointStack.push(currentJoinpoint);
            return currentJoinpoint = newJoinpoint;
        }
        function popJoinpoint() {
            return currentJoinpoint = joinpointStack.pop();
        }
        function definePropertyWorks() {
            try {
                return "x" in Object.defineProperty({}, "x", {});
            } catch (e) {}
        }
        meld.before = adviceApi("before");
        meld.around = adviceApi("around");
        meld.on = adviceApi("on");
        meld.afterReturning = adviceApi("afterReturning");
        meld.afterThrowing = adviceApi("afterThrowing");
        meld.after = adviceApi("after");
        meld.joinpoint = joinpoint;
        meld.add = function() {
            return meld.apply(null, arguments);
        };
        Advisor.prototype = {
            _callSimpleAdvice: function(adviceType, context, args) {
                var iterator, advices;
                advices = this.aspects[adviceType];
                if (!advices) return;
                iterator = iterators[adviceType];
                iterator(this.aspects[adviceType], function(aspect) {
                    var advice = aspect.advice;
                    advice && advice.apply(context, args);
                });
            },
            _callAroundAdvice: function(context, method, args, applyOriginal) {
                function callNext(i, args) {
                    return 0 > i ? applyOriginal(args) : callAround(aspects[i].advice, i, args);
                }
                function callAround(around, i, args) {
                    function proceedCount() {
                        return proceedCalled;
                    }
                    function proceedCall() {
                        return proceed(arguments.length > 0 ? slice.call(arguments) : args);
                    }
                    function proceedApply(newArgs) {
                        return proceed(newArgs || args);
                    }
                    function proceed(args) {
                        proceedCalled++;
                        return callNext(i - 1, args);
                    }
                    var proceedCalled, joinpoint;
                    proceedCalled = 0;
                    joinpoint = pushJoinpoint({
                        target: context,
                        method: method,
                        args: args,
                        proceed: proceedCall,
                        proceedApply: proceedApply,
                        proceedCount: proceedCount
                    });
                    try {
                        return around.call(context, joinpoint);
                    } finally {
                        popJoinpoint();
                    }
                }
                var len, aspects;
                aspects = this.aspects.around;
                len = aspects ? aspects.length : 0;
                return callNext(len - 1, args);
            },
            add: function(aspect) {
                var advisor, aspects;
                advisor = this;
                aspects = advisor.aspects;
                insertAspect(aspects, aspect);
                return {
                    remove: function() {
                        var remaining = removeAspect(aspects, aspect);
                        remaining || advisor.remove();
                    }
                };
            },
            remove: function() {
                delete this.advised._advisor;
                this.target[this.func] = this.orig;
            }
        };
        Advisor.get = function(target, methodName) {
            if (!(methodName in target)) return;
            var advisor, advised;
            advised = target[methodName];
            if ("function" != typeof advised) throw new Error("Advice can only be applied to functions: " + methodName);
            advisor = advised._advisor;
            if (!advisor) {
                advisor = new Advisor(target, methodName);
                target[methodName] = advisor.advised;
            }
            return advisor;
        };
        var currentJoinpoint, joinpointStack, ap, prepend, append, iterators, slice, isArray, defineProperty, objectCreate;
        joinpointStack = [];
        ap = Array.prototype;
        prepend = ap.unshift;
        append = ap.push;
        slice = ap.slice;
        isArray = Array.isArray || function(it) {
            return "[object Array]" == Object.prototype.toString.call(it);
        };
        defineProperty = definePropertyWorks() ? Object.defineProperty : function(obj, prop, descriptor) {
            obj[prop] = descriptor.value;
        };
        objectCreate = Object.create || function() {
            function F() {}
            return function(proto) {
                F.prototype = proto;
                var instance = new F();
                F.prototype = null;
                return instance;
            };
        }();
        iterators = {
            before: forEachReverse,
            around: false
        };
        iterators.on = iterators.afterReturning = iterators.afterThrowing = iterators.after = forEach;
        return meld;
    });
}("function" == typeof define && define.amd ? define : function(factory) {
    module.exports = factory();
});