!function(global) {
    function decode(s) {
        if (s) {
            s = decodeURIComponent(s);
            s = s.replace(re.pluses, " ");
        }
        return s;
    }
    function parseUri(str) {
        var parser = re.uri_parser;
        var parserKeys = [ "source", "protocol", "authority", "userInfo", "user", "password", "host", "port", "relative", "path", "directory", "file", "query", "anchor" ];
        var m = parser.exec(str || "");
        var parts = {};
        parserKeys.forEach(function(key, i) {
            parts[key] = m[i] || "";
        });
        return parts;
    }
    function parseQuery(str) {
        var i, ps, p, n, k, v;
        var pairs = [];
        if ("undefined" == typeof str || null === str || "" === str) return pairs;
        0 === str.indexOf("?") && (str = str.substring(1));
        ps = str.toString().split(re.query_separator);
        for (i = 0; i < ps.length; i++) {
            p = ps[i];
            n = p.indexOf("=");
            if (0 !== n) {
                k = decodeURIComponent(p.substring(0, n));
                v = decodeURIComponent(p.substring(n + 1));
                pairs.push(-1 === n ? [ p, null ] : [ k, v ]);
            }
        }
        return pairs;
    }
    function Uri(str) {
        this.uriParts = parseUri(str);
        this.queryPairs = parseQuery(this.uriParts.query);
        this.hasAuthorityPrefixUserPref = null;
    }
    var re = {
        starts_with_slashes: /^\/+/,
        ends_with_slashes: /\/+$/,
        pluses: /\+/g,
        query_separator: /[&;]/,
        uri_parser: /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
    };
    Array.prototype.forEach || (Array.prototype.forEach = function(fn, scope) {
        for (var i = 0, len = this.length; len > i; ++i) fn.call(scope || this, this[i], i, this);
    });
    [ "protocol", "userInfo", "host", "port", "path", "anchor" ].forEach(function(key) {
        Uri.prototype[key] = function(val) {
            "undefined" != typeof val && (this.uriParts[key] = val);
            return this.uriParts[key];
        };
    });
    Uri.prototype.hasAuthorityPrefix = function(val) {
        "undefined" != typeof val && (this.hasAuthorityPrefixUserPref = val);
        return null === this.hasAuthorityPrefixUserPref ? -1 !== this.uriParts.source.indexOf("//") : this.hasAuthorityPrefixUserPref;
    };
    Uri.prototype.query = function(val) {
        var s = "", i, param;
        "undefined" != typeof val && (this.queryPairs = parseQuery(val));
        for (i = 0; i < this.queryPairs.length; i++) {
            param = this.queryPairs[i];
            s.length > 0 && (s += "&");
            if (null === param[1]) s += param[0]; else {
                s += param[0];
                s += "=";
                param[1] && (s += encodeURIComponent(param[1]));
            }
        }
        return s.length > 0 ? "?" + s : s;
    };
    Uri.prototype.getQueryParamValue = function(key) {
        var param, i;
        for (i = 0; i < this.queryPairs.length; i++) {
            param = this.queryPairs[i];
            if (key === param[0]) return param[1];
        }
    };
    Uri.prototype.getQueryParamValues = function(key) {
        var arr = [], i, param;
        for (i = 0; i < this.queryPairs.length; i++) {
            param = this.queryPairs[i];
            key === param[0] && arr.push(param[1]);
        }
        return arr;
    };
    Uri.prototype.deleteQueryParam = function(key, val) {
        var arr = [], i, param, keyMatchesFilter, valMatchesFilter;
        for (i = 0; i < this.queryPairs.length; i++) {
            param = this.queryPairs[i];
            keyMatchesFilter = decode(param[0]) === decode(key);
            valMatchesFilter = param[1] === val;
            (1 !== arguments.length || keyMatchesFilter) && (2 !== arguments.length || keyMatchesFilter && valMatchesFilter) || arr.push(param);
        }
        this.queryPairs = arr;
        return this;
    };
    Uri.prototype.addQueryParam = function(key, val, index) {
        if (3 === arguments.length && -1 !== index) {
            index = Math.min(index, this.queryPairs.length);
            this.queryPairs.splice(index, 0, [ key, val ]);
        } else arguments.length > 0 && this.queryPairs.push([ key, val ]);
        return this;
    };
    Uri.prototype.replaceQueryParam = function(key, newVal, oldVal) {
        var index = -1, i, param;
        if (3 === arguments.length) {
            for (i = 0; i < this.queryPairs.length; i++) {
                param = this.queryPairs[i];
                if (decode(param[0]) === decode(key) && decodeURIComponent(param[1]) === decode(oldVal)) {
                    index = i;
                    break;
                }
            }
            this.deleteQueryParam(key, oldVal).addQueryParam(key, newVal, index);
        } else {
            for (i = 0; i < this.queryPairs.length; i++) {
                param = this.queryPairs[i];
                if (decode(param[0]) === decode(key)) {
                    index = i;
                    break;
                }
            }
            this.deleteQueryParam(key);
            this.addQueryParam(key, newVal, index);
        }
        return this;
    };
    [ "protocol", "hasAuthorityPrefix", "userInfo", "host", "port", "path", "query", "anchor" ].forEach(function(key) {
        var method = "set" + key.charAt(0).toUpperCase() + key.slice(1);
        Uri.prototype[method] = function(val) {
            this[key](val);
            return this;
        };
    });
    Uri.prototype.scheme = function() {
        var s = "";
        if (this.protocol()) {
            s += this.protocol();
            this.protocol().indexOf(":") !== this.protocol().length - 1 && (s += ":");
            s += "//";
        } else this.hasAuthorityPrefix() && this.host() && (s += "//");
        return s;
    };
    Uri.prototype.origin = function() {
        var s = this.scheme();
        if ("file://" == s) return s + this.uriParts.authority;
        if (this.userInfo() && this.host()) {
            s += this.userInfo();
            this.userInfo().indexOf("@") !== this.userInfo().length - 1 && (s += "@");
        }
        if (this.host()) {
            s += this.host();
            this.port() && (s += ":" + this.port());
        }
        return s;
    };
    Uri.prototype.addTrailingSlash = function() {
        var path = this.path() || "";
        "/" !== path.substr(-1) && this.path(path + "/");
        return this;
    };
    Uri.prototype.toString = function() {
        var path, s = this.origin();
        if (this.path()) {
            path = this.path();
            if (re.ends_with_slashes.test(s) || re.starts_with_slashes.test(path)) {
                s && s.replace(re.ends_with_slashes, "/");
                path = path.replace(re.starts_with_slashes, "/");
            } else s += "/";
            s += path;
        } else this.host() && (this.query().toString() || this.anchor()) && (s += "/");
        if (this.query().toString()) {
            0 !== this.query().toString().indexOf("?") && (s += "?");
            s += this.query().toString();
        }
        if (this.anchor()) {
            0 !== this.anchor().indexOf("#") && (s += "#");
            s += this.anchor();
        }
        return s;
    };
    Uri.prototype.clone = function() {
        return new Uri(this.toString());
    };
    "function" == typeof define && define.amd ? define(function() {
        return Uri;
    }) : "undefined" != typeof module && "undefined" != typeof module.exports ? module.exports = Uri : global.Uri = Uri;
}(this);