"use strict";

var i18n = require("i18n/i18n"), resourceService = require("services/resource_owner_service").singleton(), avatar = require("ui/common/Avatar");

var MESSAGE_TYPE = "USER_TEXT", USER_POKE_TYPE = "USER_POKE", BOOKING_REQUEST_TYPE = "BOOKING_REQUESTED", BOOKING_ACCEPTED_TYPE = "BOOKING_ACCEPTED", BOOKING_CANCELLED_TYPE = "BOOKING_CANCELLED";

var Message = {};

Message.isBookingRequest = function(message) {
    return message.messageType === BOOKING_REQUEST_TYPE;
};

Message.isBookingAccepted = function(message) {
    return message.messageType === BOOKING_ACCEPTED_TYPE;
};

Message.isBookingCancelled = function(message) {
    return message.messageType === BOOKING_CANCELLED_TYPE;
};

Message.formatDate = function(message) {
    var timestamp = message.timestamp;
    var now = moment(), time = moment(timestamp), diff = Math.abs(time.diff(now, "days"));
    return diff ? i18n.formatDateTime(moment(timestamp)) : i18n.formatTime(moment(timestamp));
};

Message.getUserAvatar = function(message, users) {
    var userId = message.sendingUser;
    var user = _.findWhere(users, {
        userId: userId
    });
    return avatar.getLocalPhoto(user);
};

Message.isOwnMessage = function(message) {
    var userId = resourceService.get("userId");
    return message.sendingUser == userId;
};

module.exports = Message;