"use strict";

var rootWindow = TABS_HELPER.rootWindow();

var MessagesRefreshSystem = function() {
    this.init = function() {
        _.bindAll(this, "_loadCount");
        this._loadCount();
        this._messageRefreshInterval = setInterval(this._loadCount, 6e4);
        var self = this;
        rootWindow.addAppEventListener("carma:life.cycle.refresh", function() {
            Alloy.Globals.clearInterval(self._messageRefreshInterval);
            self._loadCount();
            self._messageRefreshInterval = setInterval(self._loadCount, 6e4);
        });
        rootWindow.addAppEventListener("carma:life.cycle.pause", function() {
            Alloy.Globals.clearInterval(self._messageRefreshInterval);
        });
        rootWindow.addAppEventListener("carma:update.overall.counter", function(params) {
            params = params || {};
            var silent = params.hasOwnProperty("silent") ? params.silent : true;
            params.silent = silent;
            TM.push(function() {
                self._loadCount(params);
            });
        });
    };
};

_.extend(MessagesRefreshSystem.prototype, {
    _messageRefreshInterval: null,
    _notificationsCount: 0,
    _messagesCount: 0,
    _loadCount: function(params) {
        try {
            process.backgroundDomain.enter();
            this._loadNotifications(params);
            this._loadMessages(params);
        } finally {
            process.backgroundDomain.exit();
        }
    },
    _loadNotifications: function(params) {
        var self = this;
        require("services/notification_service").unreadCount().then(function(data) {
            var count = data ? parseInt(data) : 0;
            if (count >= 0) {
                self._notificationsCount = count;
                self._updateCounts(params);
                params && params.silent || Ti.App.fireEvent("carma:notifications.count.change", {
                    count: count
                });
            }
        }).catch(function() {});
    },
    _loadMessages: function(params) {
        var self = this;
        require("services/message_service").unreadCount().then(function(data) {
            var count = data && data.response ? parseInt(data.response) : 0;
            if (count >= 0) {
                self._messagesCount = count;
                self._updateCounts(params);
                params && params.silent || Ti.App.fireEvent("carma:messages.count.change", {
                    count: count,
                    source: "messages_refresh"
                });
            }
        }).catch(function() {});
    },
    _updateCounts: function(params) {
        var count = this._notificationsCount + this._messagesCount;
        params && params.isInsideThread || Ti.App.fireEvent("carma:overall.count.change", {
            count: count
        });
    }
});

module.exports = new MessagesRefreshSystem();